/* ======================================================================
**  TCrosswordGrid.cpp - Исходный файл модуля сетки кроссворда.
**  Является частью программы Клёст-кроссворд, распространяемой под
**  Стандартной общественной лицензией, версии 2 (GNU GPL v2).
** ======================================================================
**  Copyleft 2009 - 2012 by Lab Free Technologies <admin@LabFreeTech.org>
** ==============================================================================
**  Copyleft 2009 - 2012 Лаборатория свободных технологий <admin@LabFreeTech.org>
** ==============================================================================
*/

#include "TCrosswordGrid.h"

TCrosswordGrid::TCrosswordGrid(QTableWidget *pwgt /* = 0*/) : QTableWidget(pwgt),
    b_Saved(true),
    b_AcrossLite(ACROSS_DISABLE),
    b_SelectWord(false),
    b_InsertCoordinates(false),
    i_NumberRowFirstLetterWord(0),
    i_NumberColumnFirstLetterWord(0),
    i_CountLetterInWord(0),
    i_NumberCurrentQuestion(65000),
    b_EditDataCrossword(NULL),
    b_GridEditHand(NULL)
{

}

TCrosswordGrid::~TCrosswordGrid()
{

}

//При щелчке правой кнопкой мыши, отправляется сигнал изменения
//направления
/*virtual*/
void TCrosswordGrid::mouseReleaseEvent(QMouseEvent* pe)
{
    if(pe->button() == Qt::RightButton) emit ChangeLine();

    QTableWidget::mouseReleaseEvent(pe);
}

/*virtual void CrosswordGrid::mouseMoveEvent(QMouseEvent* pe)
{
    if(pe->button() == Qt::LeftButton) emit MouseMove();

  QTableWidget::mouseMoveEvent(pe);
}*/

bool TCrosswordGrid::isSaved() const
{
    return b_Saved;
}

void TCrosswordGrid::setSaved(bool bSaved)
{
    b_Saved = bSaved;
}

bool TCrosswordGrid::isAcrossLite() const
{
    return b_AcrossLite;
}

void TCrosswordGrid::setAcrossLite(bool bAcrossLite)
{
    b_AcrossLite = bAcrossLite;
}

bool TCrosswordGrid::isSelectWord() const
{
    return b_SelectWord;
}

void TCrosswordGrid::setSelectWord(bool bSelectWord)
{
    b_SelectWord = bSelectWord;
}

bool TCrosswordGrid::isInsertCoordinates() const
{
    return b_InsertCoordinates;
}

void TCrosswordGrid::setInsertCoordinates(bool bInsertCoordinates)
{
    b_InsertCoordinates = bInsertCoordinates;
}

int TCrosswordGrid::numberRowFirstLetterWord() const
{
    return i_NumberRowFirstLetterWord;
}

void TCrosswordGrid::setNumberRowFirstLetterWord(int iNumberRowFirstLetterWord)
{
    i_NumberRowFirstLetterWord = iNumberRowFirstLetterWord;
}

int TCrosswordGrid::numberColumnFirstLetterWord() const
{
    return i_NumberColumnFirstLetterWord;
}

void TCrosswordGrid::setNumberColumnFirstLetterWord(int iNumberColumnFirstLetterWord)
{
    i_NumberColumnFirstLetterWord = iNumberColumnFirstLetterWord;
}

int TCrosswordGrid::countLetterInWord() const
{
    return i_CountLetterInWord;
}

void TCrosswordGrid::setCountLetterInWord(int iCountLetterInWord)
{
    i_CountLetterInWord = iCountLetterInWord;
}

int TCrosswordGrid::numberCurrentQuestion() const
{
    return i_NumberCurrentQuestion;
}

void TCrosswordGrid::setNumberCurrentQuestion(int iNumberCurrentQuestion)
{
    i_NumberCurrentQuestion = iNumberCurrentQuestion;
}

bool TCrosswordGrid::isEditDataCrossword() const
{
    return b_EditDataCrossword;
}

void TCrosswordGrid::setEditDataCrossword(bool bEditDataCrossword)
{
    b_EditDataCrossword = bEditDataCrossword;
}

bool TCrosswordGrid::isGridEditHand() const
{
    return b_GridEditHand;
}

void TCrosswordGrid::setGridEditHand(bool bGridEditHand)
{
    b_GridEditHand = bGridEditHand;
}

//Сохранение кроссворда в файл
void TCrosswordGrid::saveToFile(QString sFile)
{
    QFile file;

    if(sFile.isEmpty()) file.setFileName(sFileNameCrossword);
    else file.setFileName(sFile);

    QDataStream sfile(&file);

    if(file.open(QIODevice::WriteOnly))
    {
        QBrush BrushBlack = QBrush(Qt::black, Qt::Dense1Pattern);
        QChar sdata;
        ushort idata;

        sfile << QCoreApplication::applicationVersion();
        sfile << rowCount();
        sfile << columnCount();
        sfile << rowHeight(0);
        sfile << font();

        //Если ячейка чёрная, то в файл записывается код буквы 'b', с прибавлением 1-цы
        //Если ячейка белая, то в файл записывается код буквы 'w', с прибавлением 1-цы
        for(int i = 0; i < rowCount(); i++)
            for(int j = 0; j < columnCount(); j++)
            {
            if(item(i,j)->background() == BrushBlack)
            {
                sdata = 'b';
                idata = sdata.unicode() + 1;
                sfile << idata;
            }
            else
            {
                sdata = 'w';
                idata = sdata.unicode() + 1;
                sfile << idata;
            }
            sdata = 0;

        }

        if(sFile.isEmpty())
        {
            crosswordToData(2);
            sfile << vBeginCoordWordRow;
            sfile << vBeginCoordWordColumn;
            sfile << vDirectionWord;
            sfile << vLengthWord;
            sfile << slWordT;
            sfile << slQuestionL;
            sfile << slQuestionT;
        }

        file.close();
        file.open(QIODevice::ReadOnly);
        QByteArray baExport;
        baExport = file.readAll();
        file.close();
        baExport = baExport.toBase64();
        baExport = qCompress(baExport, 9);
        file.open(QIODevice::WriteOnly);
        file.write(baExport);
        file.close();
        setSaved(true);
    }
}

//Передача данных из кроссворда в переменную
void TCrosswordGrid::crosswordToData(uchar State)
{
    if(State == 0)
    {
        vBeginCoordWordRow.append(ushort(numberRowFirstLetterWord()));
        vBeginCoordWordColumn.append(ushort(numberColumnFirstLetterWord()));
        vDirectionWord.append(cDirectionWord);
    }

    if(State == 2)
    {

        QString CurrentString;
        vLengthWord.clear();

        for(int i = 0; i < slWordT.count(); i++)
        {
            CurrentString.clear();
            CurrentString.append(slWordT.at(i));
            vLengthWord.append(ushort(CurrentString.length()));
        }
    }
}

void TCrosswordGrid::clearVariable()
{
    sFileNameCrossword.clear();
    setRowCount(0);
    setColumnCount(0);
    vBeginCoordWordRow.clear();
    vBeginCoordWordColumn.clear();
    vLengthWord.clear();
    vDirectionWord.clear();
    slWordT.clear();
    slQuestionT.clear();
    slQuestionL.clear();
    slWordT.clear();
    vNumbersDeletedWord.clear();
    setSaved(true);
}

void TCrosswordGrid::saveToHTMLFile(QString sExportName, QString strHTML, QString strNumberImage)
{
    QString sFolderName;

    sFolderName = sExportName;
    sFolderName.truncate(sFolderName.lastIndexOf("/"));
    sFolderName += "/img";
    QDir htmlDir;
    htmlDir.mkdir(sFolderName);

    strHTML.replace("<IMG src=\"" + strNumberImage + "\"" , "<IMG src=\"img/img.png\"");

    QFile file(sExportName);

    file.open(QIODevice::WriteOnly);
    QTextStream stext(&file);
    stext.setCodec("UTF-8");
    stext << strHTML;
    file.close();
}

void TCrosswordGrid::saveToPortable(QString sExportName, QString sFormat, QImage imgExport, QString sHtml)
{
#ifndef QT_NO_PRINTER

    QPrinter printer(QPrinter::HighResolution);

    if (sFormat == "PDF") printer.setOutputFormat(QPrinter::PdfFormat);
    if (sFormat == "PS") printer.setOutputFormat(QPrinter::PostScriptFormat);

    printer.setOutputFileName(sExportName);
    QTextDocument *textDoc = new QTextDocument;
    textDoc->setHtml(sHtml);
    textDoc->addResource(QTextDocument::ImageResource, QUrl(QString::number(imgExport.serialNumber())), imgExport);
    textDoc->print(&printer);
    delete textDoc;

#endif
}

void TCrosswordGrid::saveToAcrossLiteTXT(QString sExportName)
{
    QFile file(sExportName);
    QString strText,
    sNum1,
    sNum2;
    strText.clear();
    strText.append("<ACROSS PUZZLE>\015\n");
    strText.append("<TITLE>\015\n");
    strText.append("\tMiscellaneous\015\n");
    strText.append("<AUTHOR>\015\n");
    strText.append("\tGenerated by the program Klest-crossword\015\n");
    strText.append("<COPYRIGHT>\015\n");
    strText.append("\tLab Free Technologies\015\n");
    strText.append("<SIZE>\015\n");
    sNum1.setNum(rowCount());
    sNum2.setNum(columnCount());
    strText.append("\t" + sNum1 +"x" + sNum2 + "\015\n");
    strText.append("<GRID>\015\n");

    for(int i = 0; i < rowCount(); i++)
    {
        strText.append("\t");

        for(int j = 0; j < columnCount(); j++)
            if(item(i,j)->text() == "") strText.append(".");
        else strText.append(item(i,j)->text());

        if(i != rowCount() - 1) strText.append("\015\n");
    }

    strText.append("\015\n<ACROSS>\015\n");

    for(int i = slQuestionT.count() / 2; i < slQuestionT.count(); i++)
    {
        strText.append("\t" + slQuestionT.at(i));

        if(i != slQuestionT.count() - 1) strText.append("\015\n");
    }

    strText.append("\015\n<DOWN>\015\n");

    for(int i = 0; i < slQuestionT.count() / 2; i++)
    {
        strText.append("\t" + slQuestionT.at(i));

        if(i != slQuestionT.count() / 2 - 1) strText.append("\015\n");
    }

    file.open(QIODevice::WriteOnly);
    QTextStream stext(&file);
    stext << strText;
    file.close();
}

QString TCrosswordGrid::makeOneHTML(QString ListWordH, QString ListWordV, QStringList ListOutV, QStringList ListOutH, QString sImage)
{
    QString strHTML;
    QString strText1;
    QString strText2;
    QString strText3;

    for(int i = 0; i < ListOutH.count(); i++)
    {
        strText1.append(ListOutH.at(i) + "<BR>\n");
    }

    strText2.clear();

    for(int i = 0; i < ListOutV.count(); i++)
    {
        strText2.append(ListOutV.at(i) + "<BR>\n");
    }

    strText3.clear();
    strText3 = tr("Horizontal") + ":\n" + ListWordH + "<BR>\n" + tr("Vertical") +":\n" + ListWordV + "</P>\n";

    strHTML.append("<HTML>\n"
                   " <head> <META http-equiv=Content-Type content=\"text/html; charset=utf-8\"> </head>\n"
                   " <BODY>\n"
                   " <IMG src=\"" + sImage + "\"></IMG>\n"
                   " <BR>\n"
                   " <P>" + tr("Horizontal") + ":<BR>\n"
                   " " + strText1 + "</P>\n"
                   " <P>" + tr("Vertical") + ":<BR>\n"
                   " " + strText2 + "</P>\n"
                   " <P>" + tr("Answers") + ":<BR>\n"
                   " " + strText3 + "</P>\n"
                   " </BODY>\n"
                   "</HTML>\n");
    return strHTML;
}

void TCrosswordGrid::slotRotateRight()
{
    if(rowCount() != columnCount()) return;

    QBrush BrushBlack = QBrush(Qt::black, Qt::Dense1Pattern);
    QBrush BrushWhite = QBrush(Qt::SolidPattern);
    QColor col1(227, 226, 228);
    col1.setAlpha(255);
    BrushWhite.setColor(col1);

    for(ushort i = 0; i < rowCount(); i++)
        for(ushort j = 0; j < columnCount(); j++)
            if(item(i,j)->background() == BrushBlack)
            {
        usTable[j][rowCount() - i - 1] = 1;
        item(i,j)->setBackground(BrushWhite);
    }

    for(ushort i = 0; i < rowCount(); i++)
        for(ushort j = 0; j < columnCount(); j++)
            if(usTable[i][j] == 1)
            {
        item(i,j)->setBackground(BrushBlack);
        usTable[i][j] = 0;
    }
}

void TCrosswordGrid::slotRotateLeft()
{
    if(rowCount() != columnCount()) return;

    QBrush BrushBlack = QBrush(Qt::black, Qt::Dense1Pattern);
    QBrush BrushWhite = QBrush(Qt::SolidPattern);
    QColor col1(227, 226, 228);
    col1.setAlpha(255);
    BrushWhite.setColor(col1);

    for(ushort i = 0; i < rowCount(); i++)
        for(ushort j = 0; j < columnCount(); j++)
            if(item(i,j)->background() == BrushBlack)
            {
        usTable[columnCount() - j - 1][i] = 1;
        item(i,j)->setBackground(BrushWhite);
    }

    for(ushort i = 0; i < rowCount(); i++)
        for(ushort j = 0; j < columnCount(); j++)
            if(usTable[i][j] == 1)
            {
        item(i,j)->setBackground(BrushBlack);
        usTable[i][j] = 0;
    }
}

void TCrosswordGrid::makeTableToMassive()
{
    QBrush brush(Qt::black, Qt::Dense1Pattern);
    QBrush gbrush(Qt::green);
    QBrush wbrush(Qt::SolidPattern);
    QColor col1(227, 226, 228);
    col1.setAlpha(255);
    wbrush.setColor(col1);

    for(int i = 0; i < rowCount(); i++)
        for(int j = 0; j < columnCount(); j++)
        {
        if(item(i, j)->text() == "" && item(i, j)->background() == wbrush)
            usTable[i][j] = CELL_BACK;

        if(item(i, j)->text() == "" && item(i, j)->background() == brush)
            usTable[i][j] = CELL_BLACK;

        if(item(i, j)->text() == "" && item(i, j)->background() == gbrush)
            usTable[i][j] = CELL_LIME;

        if(item(i, j)->text() != "" && item(i, j)->background() == wbrush)
            usTable[i][j] = CELL_BACK_FILL;
    }
}

//Снятие выделения со всего масиива кроссворда
void TCrosswordGrid::usTableUnselectAll()
{
    for(int i = 0; i < rowCount(); i++)
        for(int j = 0; j < columnCount(); j++)
        {
        if(usTable[i][j] == CELL_LIME || usTable[i][j] == CELL_BACK_FILL)
            usTable[i][j] = CELL_BACK;
    }
}

//Автоматическое создание сетки кроссворда
void TCrosswordGrid::slotAutoGrid()
{
    QBrush BrushBlack = QBrush(Qt::black, Qt::Dense1Pattern);
    QBrush BrushWhite = QBrush(Qt::SolidPattern);
    QColor col1(227, 226, 228);
    col1.setAlpha(255);
    BrushWhite.setColor(col1);
    int CountCellH,
    CountCellV,
    HalfCountCellH,
    HalfCountCellHC,
    HalfCountCellV,
    HalfCountCellVC,
    HalfCountCellWordH,
    BeginCellWordH,
    ColvoIteraciy,
    CountCellWordH,
    oldCountCellWordH;
    bool bChetnoeH,
    bNechetnoeH,
    bChetnoeV,
    bNechetnoeV,
    bCenterWord,
    bWallWord;

    //Очистка сетки кроссворда от текста
    for(int i = 0; i < rowCount(); i++)
        for(int j = 0; j < columnCount(); j++)
            if(item(i, j)->text() != "") item(i, j)->setText("");

    CountCellH = columnCount();
    CountCellV = rowCount();
    HalfCountCellH = CountCellH / 2;
    HalfCountCellV = CountCellV / 2;
    bChetnoeH = false;
    bNechetnoeH = false;
    bChetnoeV = false;
    bNechetnoeV = false;
    bCenterWord = false;
    bWallWord = false;

    if(CountCellH % 2 == 0) bChetnoeH = true;
    else bNechetnoeH = true;

    if(CountCellV % 2 == 0) bChetnoeV = true;
    else bNechetnoeV = true;

    HalfCountCellWordH = 0;
    CountCellWordH = 0;
    oldCountCellWordH = 0;
    ColvoIteraciy = 0;
    HalfCountCellHC = 0;

    if(bChetnoeV == true) HalfCountCellVC = HalfCountCellV - 1;
    else
        if(bNechetnoeV == true) HalfCountCellVC = HalfCountCellV;

    if(bChetnoeH == true) HalfCountCellHC = HalfCountCellH - 1;
    else
        if(bNechetnoeH == true) HalfCountCellHC = HalfCountCellH;

    if(CountCellV > 0 && CountCellH > 0)
    {

        //Закрашивание ячеек сетки кроссворда в белый цвет(очистка)
        for(int i = 0; i < CountCellH; i++)
            for(int j = 0; j < CountCellV; j++)
                item(j, i)->setBackground(BrushWhite);


        for(int j = 1; j < HalfCountCellVC; j += 2)
        {

            srand((unsigned) time(NULL));

            if(CountCellH <= 18) CountCellWordH = rand() % (CountCellH + 1);
            else CountCellWordH = rand() % (18 + 1);

            if(oldCountCellWordH == CountCellWordH)
            {
                if(CountCellWordH <= CountCellH - 1)
                {
                    if(CountCellH <= 18) CountCellWordH = rand() %
                                                          (CountCellH - CountCellWordH);
                    else if(CountCellWordH < 18) CountCellWordH = rand() %
                                                                  (18 - CountCellWordH);
                }
                else CountCellWordH = 0;
            }

            if(oldCountCellWordH <= HalfCountCellH && CountCellWordH <= HalfCountCellH)
            {
                srand((unsigned) QTime::currentTime().msec());
                CountCellWordH += rand() % (CountCellH - CountCellWordH);
            }

            if(oldCountCellWordH >= HalfCountCellH && CountCellWordH >= HalfCountCellH)
            {
                srand((unsigned) QTime::currentTime().msec());
                CountCellWordH -= rand() % (CountCellH - CountCellWordH);
            }

            oldCountCellWordH = CountCellWordH;

            while(CountCellWordH < 2)
            {
                srand((unsigned) QTime::currentTime().msec());

                if(CountCellH <= 18)  CountCellWordH = rand() % (CountCellH
                                                                 + 1);
                else CountCellWordH = rand() % (18 + 1);

                switch(CountCellWordH)
                {
                case 0:
                    CountCellWordH += 4;
                    break;

                case 1:
                    CountCellWordH += 6;
                    break;

                    /*case 2:
                                                         CountCellWordH += 1;
                                                    break;*/
                }
                ColvoIteraciy++;
            }

            /*           for(int i = 0; i < CountCellH; i++)
           item(j, i)->setBackground(BrushWhite);

           for(int i = 0; i < CountCellH; i++)
           item((CountCellV - 1) - j, i)->setBackground(BrushWhite);*/

            if(CountCellWordH >= HalfCountCellH) bCenterWord = true;
            else bWallWord = true;

            if(bChetnoeH == true && bCenterWord == true && CountCellWordH % 2 != 0
               && CountCellWordH < CountCellH) CountCellWordH++;

            if(bNechetnoeH == true && bCenterWord == true && CountCellWordH % 2 == 0
               && CountCellWordH < CountCellH) CountCellWordH++;

            if(CountCellWordH > CountCellH) CountCellWordH = CountCellH;

            HalfCountCellWordH = CountCellWordH / 2;
            BeginCellWordH = HalfCountCellH - HalfCountCellWordH;

            //Закрашивание ячеек по-центру
            if(bCenterWord == true)
            {
                for(int i = BeginCellWordH; i <= BeginCellWordH + CountCellWordH
                    - 1; i++)
                    item(j, i)->setBackground(BrushBlack);

                for(int i = BeginCellWordH; i <= BeginCellWordH + CountCellWordH
                    - 1; i++)
                    item((CountCellV - 1) - j, i)
                    ->setBackground(BrushBlack);

            }

            //Закрашивание ячеек по-краям
            if(bWallWord == true)
            {
                for(int i = 0; i <= CountCellWordH - 1; i++)
                    item(j, i)->setBackground(BrushBlack);

                for(int i = CountCellH - 1; i >= CountCellH - CountCellWordH; i--)
                    item(j, i)->setBackground(BrushBlack);

                for(int i = 0; i <= CountCellWordH - 1; i++)
                    item((CountCellV - 1) - j, i)
                    ->setBackground(BrushBlack);

                for(int i = CountCellH - 1; i >= CountCellH - CountCellWordH; i--)
                    item((CountCellV - 1) - j, i)
                    ->setBackground(BrushBlack);

            }

            if(j < HalfCountCellHC)
            {

                if(CountCellWordH >= CountCellV)
                {
                    srand((unsigned) time(NULL));

                    if(CountCellV <= 18) CountCellWordH = rand()
                        % (CountCellV + 1);
                    else CountCellWordH = rand() % (18 + 1);
                }

                if(CountCellWordH >= HalfCountCellV) bCenterWord = true;
                else bWallWord = true;

                if(bChetnoeV == true && bCenterWord == true && CountCellWordH
                   % 2 != 0 && CountCellWordH < CountCellV) CountCellWordH++;

                if(bNechetnoeV == true && bCenterWord == true && CountCellWordH
                   % 2 == 0 && CountCellWordH < CountCellV) CountCellWordH++;

                HalfCountCellWordH = CountCellWordH / 2;
                BeginCellWordH = HalfCountCellV - HalfCountCellWordH;

                //Закрашивание ячеек по-центру
                if(bCenterWord == true)
                {
                    for(int i = BeginCellWordH; i <= BeginCellWordH
                        + CountCellWordH - 1; i++)
                        item(i, j)->setBackground(
                                BrushBlack);

                    for(int i = BeginCellWordH; i <= BeginCellWordH
                        + CountCellWordH - 1; i++)
                        item(i, (CountCellH - 1) - j)
                        ->setBackground(BrushBlack);
                }

                //Закрашивание ячеек по-краям
                if(bWallWord == true)
                {
                    for(int i = 0; i <= CountCellWordH - 1; i++)
                        item(i, j)->setBackground(
                                BrushBlack);

                    for(int i = CountCellV - 1; i >= CountCellV
                        - CountCellWordH; i--)
                        item(i, j)->setBackground(
                                BrushBlack);

                    for(int i = 0; i <= CountCellWordH - 1; i++)
                        item(i, (CountCellH - 1) - j)
                        ->setBackground(BrushBlack);

                    for(int i = CountCellV - 1; i >= CountCellV
                        - CountCellWordH; i--)
                        item(i, (CountCellH - 1) - j)
                        ->setBackground(BrushBlack);

                }

            }

            /*bChetnoeH = false;
       bNechetnoeH = false;
       bChetnoeV = false;
       bNechetnoeV = false;*/
            bCenterWord = false;
            bWallWord = false;
        }
        /*          int iCountBlack = 0;
           for(int i = 0; i < CountCellH; i++)
            {
              if(iCountBlack == CountCellH)
               {
                  item(i, 0)->setBackground(BrushWhite);
                  item(i, CountCellH - 1)->setBackground(BrushWhite);
               }

              iCountBlack = 0;

             for(int j = 0; j < CountCellV; j++)
               {

                 if(item(i, j)->background() == BrushBlack)
                     iCountBlack++ ;

               }
            }

           for(int i = 0; i < CountCellV; i++)
            {
              if(iCountBlack == CountCellV)
               {
                  item(i, 0)->setBackground(BrushWhite);
                  item(i, CountCellH - 1)->setBackground(BrushWhite);
               }

              iCountBlack = 0;

             for(int j = 0; j < CountCellH; j++)
               {

                 if(item(i, j)->background() == BrushBlack)
                     iCountBlack++ ;

               }
            }*/

        //Инверсия цвета ячеек(чёрный / белый)
        for(int i = 0; i < CountCellH; i++)
            for(int j = 0; j < CountCellV; j++)
                if(item(j, i)->background() == BrushBlack)
                    item(j, i)->setBackground(BrushWhite);
        else item(j, i)->setBackground(BrushBlack);
    }
}

void TCrosswordGrid::slotInversion()
{
    QBrush BrushBlack = QBrush(Qt::black, Qt::Dense1Pattern);
    QBrush BrushWhite = QBrush(Qt::SolidPattern);
    QColor col1(227, 226, 228);
    col1.setAlpha(255);
    BrushWhite.setColor(col1);

    for(int i = 0; i < rowCount(); i++)
        for(int j = 0; j < columnCount(); j++)
        {
        if(item(i, j)->background() == BrushWhite)
            item(i, j)->setBackground(BrushBlack);
        else item(i, j)->setBackground(BrushWhite);
    }
}

void TCrosswordGrid::makeCheck4Cell()
{
    /*for(int i = 0; i < rowCount(); i++)
                    for(int j = 0; j < columnCount(); j++)
                      {
                          if(item(i, j)->background() == BrushWhite)
                              item(i, j)->setBackground(BrushBlack);
                          else item(i, j)->setBackground(BrushWhite);
                      }*/
}

void TCrosswordGrid::slotMirrorVertically()
{
    QBrush BrushBegin, BrushEnd;

    for(int i = 0; i < rowCount(); i++)
        for(int j = 0; j < columnCount() / 2; j++)
        {
        BrushBegin = item(i, j)->background();
        BrushEnd = item(i, columnCount() - 1 - j)->background();
        item(i, j)->setBackground(BrushEnd);
        item(i, columnCount() - 1 - j)->setBackground(BrushBegin);

    }
}

void TCrosswordGrid::slotMirrorHorisontally()
{
    QBrush BrushBegin, BrushEnd;

    for(int i = 0; i < columnCount(); i++)
        for(int j = 0; j < rowCount() / 2; j++)
        {
        BrushBegin = item(j, i)->background();
        BrushEnd = item(rowCount() - 1 - j, i)->background();
        item(j, i)->setBackground(BrushEnd);
        item(rowCount() - 1 - j, i)->setBackground(BrushBegin);

    }
}

//Экспорт кроссворда в изображения
void TCrosswordGrid::slotExport(bool State, int iNum, int iTypeExport)
{
    if(rowCount() == 0 && columnCount() == 0) return;

    QString sNum;

    if(State == false) sNum.setNum(1);
    else sNum.setNum(iNum);

    //Выбор типа экспортируемого изображения
    QString sExportName,
    sExportFolder,
    sFilter;
    QFileDialog *DialogExportImage;
    DialogExportImage = new QFileDialog(0, tr("Export"), tr("crossword1"),
                                        "PDF - Portable Document Format (*.pdf);;"
                                        "PS - PostScript Files (*.ps);;"
                                        "Rich Text Format (*.rtf);;" +
                                        tr("Document") + " HTML (*.html);; " + tr("Text") + " AcrossLite (*.txt);;"
                                        "JPG - Joint Photographic Experts Group (*.jpg);;"
                                        "JPEG - Joint Photographic Experts Group (*.jpeg);;"
                                        "BMP - Windows Bitmap (*.bmp);;"
                                        "TIFF - Tagged Image File Format (*.tiff);;"
                                        "PNG - Portable Network Graphic (*.png);;"
                                        "XPM - X PixMap (*.xpm);;"
                                        "XBM - X BitMap Image (*.xbm);;"
                                        "PPM - Potable PixelMap (*.ppm);;"
                                        "OK - OpenKlest Format (*.ok)");
    DialogExportImage->setAcceptMode(QFileDialog::AcceptSave);

    if(State == true)
    {
        switch(iTypeExport)
        {
        case TYPE_PDF:
            sFilter = ".pdf";
            break;

        case TYPE_POST_SCRIPT:
            sFilter = ".ps";
            break;

        case TYPE_RTF:
            sFilter = ".rtf";
            break;

        case TYPE_HTML:
            sFilter = ".html";
            break;

        case TYPE_TXT:
            sFilter = ".txt";
            break;

        case TYPE_JPG:
            sFilter = ".jpg";
            break;

        case TYPE_JPEG:
            sFilter = ".jpeg";
            break;

        case TYPE_BMP:
            sFilter = ".bmp";
            break;

        case TYPE_TIFF:
            sFilter = ".tiff";
            break;

        case TYPE_PNG:
            sFilter = ".png";
            break;

        case TYPE_XPM:
            sFilter = ".xpm";
            break;

        case TYPE_XBM:
            sFilter = ".xbm";
            break;

        case TYPE_PPM:
            sFilter = ".ppm";
            break;

        case TYPE_OPEN_KLEST:
            sFilter = ".ok";
            break;
        }
    }
    else
    {
        if(DialogExportImage->exec() == QFileDialog::Accepted)
        {
            sExportName = DialogExportImage->selectedFiles().at(0);
            sFilter = DialogExportImage->selectedFilter();
            sFilter.remove(0, sFilter.indexOf("*"));
            sFilter.remove(sFilter.length() - 1, 1);
            sFilter.replace("*", "");

            if(sExportName.indexOf(sFilter) == -1) sExportName += sFilter;
        }
        else return;
    }

    delete DialogExportImage;
    QImage imgExport;
    imgExport = makeCrosswordImage();
    TCrosswordText CrosswordText;
    CrosswordText = makeCrosswordText();
    QString sHtml;

    if(sFilter == ".rtf" && State == false)
    {
        sHtml = makeRTFText(CrosswordText.ListWordH, CrosswordText.ListWordV, CrosswordText.ListOutV, CrosswordText.ListOutH);
        saveToRTF(sExportName, sHtml, imgExport);
        return;
    }

    if(sFilter == ".rtf" && State == true)
    {
        sHtml = makeRTFText(CrosswordText.ListWordH, CrosswordText.ListWordV, CrosswordText.ListOutV, CrosswordText.ListOutH);
        saveToRTF(FileOptions->sPathCrossword + "/crossword" + sNum + ".rtf", sHtml, imgExport);
        return;
    }

    if(sFilter == ".html" && State == false)
    {
        sHtml = makeOneHTML(CrosswordText.ListWordH, CrosswordText.ListWordV, CrosswordText.ListOutV, CrosswordText.ListOutH, QString::number(imgExport.serialNumber()));
        saveToHTMLFile(sExportName, sHtml, QString::number(imgExport.serialNumber()));
        sExportFolder = sExportName;
        sExportFolder.truncate(sExportFolder.lastIndexOf("/"));
        imgExport.save(sExportFolder + "/img/img.png", "PNG", 0);
        return;
    }

    if(sFilter == ".html" && State == true)
    {
        sHtml = makeOneHTML(CrosswordText.ListWordH, CrosswordText.ListWordV, CrosswordText.ListOutV, CrosswordText.ListOutH, QString::number(imgExport.serialNumber()));
        saveToHTMLFile(FileOptions->sPathCrossword + "/crossword" + sNum + ".html", sHtml, QString::number(imgExport.serialNumber()));
        sExportFolder = FileOptions->sPathCrossword;
        imgExport.save(sExportFolder + "/img/img.png", "PNG",0);
        return;
    }

    if(sFilter == ".pdf" && State == false)
    {
        sHtml = makeOneHTML(CrosswordText.ListWordH, CrosswordText.ListWordV, CrosswordText.ListOutV, CrosswordText.ListOutH, QString::number(imgExport.serialNumber()));
        saveToPortable(sExportName, "PDF", imgExport, sHtml);
        return;
    }

    if(sFilter == ".pdf" && State == true)
    {
        sHtml = makeOneHTML(CrosswordText.ListWordH, CrosswordText.ListWordV, CrosswordText.ListOutV, CrosswordText.ListOutH, QString::number(imgExport.serialNumber()));
        saveToPortable(FileOptions->sPathCrossword + "/crossword" + sNum + ".pdf", "PDF", imgExport, sHtml);
        return;
    }

    if(sFilter == ".ps" && State == false)
    {
        sHtml = makeOneHTML(CrosswordText.ListWordH, CrosswordText.ListWordV, CrosswordText.ListOutV, CrosswordText.ListOutH, QString::number(imgExport.serialNumber()));
        saveToPortable(sExportName, "PS", imgExport, sHtml);
        return;
    }

    if(sFilter == ".ps" && State == true)
    {
        sHtml = makeOneHTML(CrosswordText.ListWordH, CrosswordText.ListWordV, CrosswordText.ListOutV, CrosswordText.ListOutH, QString::number(imgExport.serialNumber()));
        saveToPortable(FileOptions->sPathCrossword + "/crossword" + sNum + ".ps", "PS", imgExport, sHtml);
        return;
    }

    if(sFilter == ".txt" && State == false)
    {
        saveToAcrossLiteTXT(sExportName);
        return;
    }

    if(sFilter == ".txt" && State == true)
    {
        saveToAcrossLiteTXT(FileOptions->sPathCrossword + "/crossword" + sNum + ".txt");
        return;
    }

    if(sFilter == ".ok" && State == false)
    {
        saveToOpenKlest(sExportName);
        return;
    }

    if(sFilter == ".ok" && State == true)
    {
        saveToOpenKlest(FileOptions->sPathCrossword + "/crossword" + sNum + ".ok");
        return;
    }

    if(sFilter == ".jpg" && State == true) imgExport.save(FileOptions->sPathCrossword + tr("/crossword") + "1.jpg","JPG",100);
    if(sFilter == ".jpg" && State == false) imgExport.save(sExportName,"JPG",100);

    if(sFilter == ".jpeg" && State == true) imgExport.save(FileOptions->sPathCrossword + tr("/crossword") + "1.jpeg","JPEG",100);
    if(sFilter == ".jpeg" && State == false) imgExport.save(sExportName,"JPEG",100);

    if(sFilter == ".bmp" && State == true) imgExport.save(FileOptions->sPathCrossword + tr("/crossword") + "1.bmp","BMP",0);
    if(sFilter == ".bmp" && State == false) imgExport.save(sExportName,"BMP",0);

    if(sFilter == ".tiff" && State == true) imgExport.save(FileOptions->sPathCrossword + tr("/crossword") + "1.tiff","TIFF",0);
    if(sFilter == ".tiff" && State == false) imgExport.save(sExportName,"TIFF",0);

    if(sFilter == ".png" && State == true) imgExport.save(FileOptions->sPathCrossword + tr("/crossword") + "1.png","PNG",0);
    if(sFilter == ".png" && State == false) imgExport.save(sExportName,"PNG",0);

    if(sFilter == ".xpm" && State == true) imgExport.save(FileOptions->sPathCrossword + tr("/crossword") + "1.xpm","XPM",0);
    if(sFilter == ".xpm" && State == false) imgExport.save(sExportName,"XPM",0);

    if(sFilter == ".xbm" && State == true) imgExport.save(FileOptions->sPathCrossword + tr("/crossword") + "1.xbm","XBM",0);
    if(sFilter == ".xbm" && State == false) imgExport.save(sExportName,"XBM",0);

    if(sFilter == ".ppm" && State == true) imgExport.save(FileOptions->sPathCrossword + tr("/crossword") + "1.ppm","PPM",0);
    if(sFilter == ".ppm" && State == false) imgExport.save(sExportName,"PPM",0);

    //setWindowTitle();

    /*       QString item;
        bool ok;

            if(State == true)
                {
                    item = sExport;
                    ok = true;
                }
            else
                {
                     item = QInputDialog::getItem(this, tr("Select image format"), tr("File types:"), ListExport, 0, false, &ok);
                }

            if(ok && !item.isEmpty())
                {
                    if(item == "*.jpg") pix.save(PathExport + tr("/image") + "1.jpg","JPG",100);
                    if(item == "*.jpeg") pix.save(PathExport + tr("/image") + "1.jpeg","JPEG",100);
                    if(item == "*.bmp") pix.save(PathExport + tr("/image") + "1.bmp","BMP",0);
                    if(item == "*.tiff") pix.save(PathExport + tr("/image") + "1.tiff","TIFF",0);
                    if(item == "*.png") pix.save(PathExport + tr("/image") + "1.png","PNG",0);
                    if(item == "*.xpm") pix.save(PathExport + tr("/image") + "1.xpm","XPM",0);
                    if(item == "*.xbm") pix.save(PathExport + tr("/image") + "1.xbm","XBM",0);
                    if(item == "*.ppm") pix.save(PathExport + tr("/image") + "1.ppm","PPM",0);
                }*/

    sExportName.truncate(sExportName.lastIndexOf('/'));
    QFile file;

    if(State == true) file.setFileName(FileOptions->sPathCrossword +  "/" + tr("Questions") + sNum + ".txt");
    else file.setFileName(sExportName + "/" + tr("Questions") + sNum + ".txt");

    file.open(QIODevice::WriteOnly);
    QTextStream stext(&file);
    QString strQuestions;

    stext << tr("Horizontal") + ":" << "\n";

    for(int i = 0; i < CrosswordText.ListOutH.count(); i++)
    {
        strQuestions.clear();
        strQuestions.append(CrosswordText.ListOutH.at(i));
        stext << strQuestions << "\n";
    }

    stext << "\n";

    stext << tr("Vertical") + ":" << "\n";

    for(int i = 0; i <CrosswordText.ListOutV.count(); i++)
    {
        strQuestions.clear();
        strQuestions.append(CrosswordText.ListOutV.at(i));
        stext << strQuestions << "\n";
    }

    stext << "\n";

    file.close();

    QFile file2;

    if(State == true) file2.setFileName(FileOptions->sPathCrossword + "/" + tr("Answers") + sNum +".txt");
    else file2.setFileName(sExportName + "/" + tr("Answers") + sNum +".txt");

    file2.open(QIODevice::WriteOnly);
    QTextStream stext2(&file2);

    stext2 << tr("Horizontal") + ":" << "\n" << CrosswordText.ListWordH << "\n" << tr("Vertical") +":" <<"\n" << CrosswordText.ListWordV << "\n";
    file2.close();

}

void TCrosswordGrid::saveToOpenKlest(QString FileName)
{
    QFile wfile(FileName);
    wfile.open(QIODevice::WriteOnly);
    QString sFileBegin,
    sFileEnd;
    QBrush BrushBlack = QBrush(Qt::black, Qt::Dense1Pattern);
    QChar sdata;
    ushort usdata;
    uint uidata;
    sFileBegin = "OpenKlest-1.0~";
    sFileEnd = "~OpenKlest";

    wfile.write(sFileBegin.toStdString().c_str(), sFileBegin.length());
    uidata =  rowCount();
    wfile.write(reinterpret_cast<char*>(&uidata), sizeof(uidata));
    uidata =  columnCount();
    wfile.write(reinterpret_cast<char*>(&uidata), sizeof(uidata));
    uidata =  rowHeight(0);
    wfile.write(reinterpret_cast<char*>(&uidata), sizeof(uidata));

    //Если ячейка чёрная, то в файл записывается код буквы 'b', с прибавлением 1-цы
    //Если ячейка белая, то в файл записывается код буквы 'w', с прибавлением 1-цы
    for(int i = 0; i < rowCount(); i++)
        for(int j = 0; j < columnCount();
    j++)
    {
        if(item(i,j)->background()
            == BrushBlack)
            {
            sdata = 'b';
            usdata = sdata.unicode() + 1;
            wfile.write(reinterpret_cast<char*>(&usdata), sizeof(ushort));
        }
        else
        {
            sdata = 'w';
            usdata = sdata.unicode() + 1;
            wfile.write(reinterpret_cast<char*>(&usdata), sizeof(ushort));
        }

        sdata = 0;
    }

    crosswordToData(2);
    uidata = vBeginCoordWordRow.count();
    wfile.write(reinterpret_cast<char*>(&uidata), sizeof(uint));

    for(int i = 0; i < vBeginCoordWordRow.count(); i++)
    {
        usdata = vBeginCoordWordRow.at(i);
        wfile.write(reinterpret_cast<char*>(&usdata), sizeof(ushort));
    }


    for(int i = 0; i < vBeginCoordWordColumn.count(); i++)
    {
        usdata = vBeginCoordWordColumn.at(i);
        wfile.write(reinterpret_cast<char*>(&usdata), sizeof(ushort));
    }


    for(int i = 0; i < vDirectionWord.count(); i++)
    {
        usdata = vDirectionWord.at(i).unicode();
        wfile.write(reinterpret_cast<char*>(&usdata), sizeof(ushort));
    }

    for(int i = 0; i < vLengthWord.count(); i++)
    {
        usdata = vLengthWord.at(i);
        wfile.write(reinterpret_cast<char*>(&usdata), sizeof(ushort));
    }

    for(int i = 0; i < slWordT.count(); i++)
    {
        QString slWord;
        slWord = slWordT.at(i);
        wfile.write(slWord.toStdString().c_str(), slWord.length());
    }


    for(int i = 0; i < slQuestionL.count(); i++)
    {
        QString slQL;
        slQL = slQuestionL.at(i);
        usdata = slQL.at(0).unicode();
        wfile.write(reinterpret_cast<char*>(&usdata), sizeof(ushort));
    }

    for(int i = 0; i < slQuestionT.count(); i++)
    {
        QString slQT;
        slQT = slQuestionT.at(i);
        wfile.write(slQT.toStdString().c_str(), slQT.length());
    }

    wfile.write(sFileEnd.toStdString().c_str(), sFileEnd.length());
    wfile.close();
}

void TCrosswordGrid::print()
{
    QPrinter printer;
    QPrintDialog *pPrintDialog = new QPrintDialog(&printer);

    if(pPrintDialog->exec() == QDialog::Accepted)
    {
        if(rowCount() == 0 && columnCount() == 0) return;

        QString sPrint;
        QImage imgPrint;
        TCrosswordText CrosswordText;
        CrosswordText = makeCrosswordText();
        imgPrint = makeCrosswordImage();
        sPrint = makeOneHTML(CrosswordText.ListWordH, CrosswordText.ListWordV, CrosswordText.ListOutV, CrosswordText.ListOutH,
                             QString::number(imgPrint.serialNumber()));
        QTextDocument *textDoc = new QTextDocument;
        textDoc->setHtml(sPrint);
        textDoc->addResource(QTextDocument::ImageResource, QUrl(QString::number(imgPrint.serialNumber())), imgPrint);
        textDoc->print(&printer);
        delete textDoc;
    }

    delete pPrintDialog;
}

QImage TCrosswordGrid::makeCrosswordImage()
{
    QStringList ListSelectWord2;
    ushort NumberWord;

    int iWidth;
    int iHeight;
    int iAllWidth;
    int iAllHeight;
    int iPixWidth;
    int iPixHeight;

    NumberWord = 0;
    iWidth = 0;
    iHeight = 0;

    if(columnCount() != 0) iWidth = columnWidth(0);
    if(rowCount() != 0) iHeight = rowHeight(0);

    iAllWidth = iWidth * columnCount() + 3;
    iAllHeight = iHeight * rowCount() + 3;

    iPixWidth = iWidth - (iWidth / 10 * 3);
    iPixHeight = iHeight - (iHeight / 10 * 3);

    QPainter *painter;
    painter = new QPainter;
    QPixmap pix(iAllWidth, iAllHeight);

    pix.fill();

    QRect rect;
    QFont FontCrossword(font());
    FontCrossword.setPointSize(font().pointSize() - 3);

    //pix.fill();
    painter->begin(&pix);
    painter->setFont(FontCrossword);

    //Рисование на холсте горизонтальных линий
    for(int i = 0; i < iAllHeight; i += iHeight)
    {
        painter->drawLine(0, i, iAllWidth - 1, i);
        painter->drawLine(0, i + 1, iAllWidth - 1, i + 1);
        painter->drawLine(0, i + 2, iAllWidth - 1, i + 2);
    }

    //Рисование на холсте вертикальных линий
    for(int i = 0; i < iAllWidth; i += iWidth)
    {
        painter->drawLine(i, 0, i, iAllHeight - 1);
        painter->drawLine(i + 1, 0, i + 1, iAllHeight - 1);
        painter->drawLine(i + 2, 0, i + 2, iAllHeight - 1);
    }

    ListSelectWord2 = slSelectWord;
    vNumbersDeletedWord.clear();

    for(int y = 0; y < ListSelectWord2.count(); y++)
    {
        QString strList;
        QString strList2;
        QString strList3;

        strList.clear();
        strList.append(ListSelectWord2.at(y));

        strList2.clear();
        strList2.append(strList.at(0));
        strList2.append(strList.at(1));

        for (int x = 0; x < ListSelectWord2.count(); x++)
        {
            strList.clear();
            strList.append(ListSelectWord2.at(x));

            strList3.clear();
            strList3.append(strList.at(0));
            strList3.append(strList.at(1));

            if(strList2 == strList3 && x != y)
            {
                ListSelectWord2.removeAt(x);
            }
        }
    }

    for(int y = 0; y < slSelectWord.count(); y++)
    {
        QString strList;
        QString strList2;
        QString strList3;

        strList.clear();
        strList.append(slSelectWord.at(y));

        strList2.clear();
        strList2.append(strList.at(0));
        strList2.append(strList.at(1));

        for (int x = 0; x < slSelectWord.count(); x++)
        {
            strList.clear();
            strList.append(slSelectWord.at(x));

            strList3.clear();
            strList3.append(strList.at(0));
            strList3.append(strList.at(1));

            if(strList2 == strList3 && y > x)
            {
                vNumbersDeletedWord.append(y - vNumbersDeletedWord.count());
            }
        }
    }

    QPixmap pix2(iWidth, iHeight);
    pix2.fill(Qt::black);
    QBrush brushBlack(Qt::black, Qt::Dense1Pattern);

    QString strListSelectWord;
    ushort NumberPlain = 1;
    for(int i = 0; i < rowCount(); i++)
        for(int j = 0; j < columnCount(); j++)
        {
        if(item(i, j)->background() == brushBlack)
            painter->drawPixmap((j * iWidth) + 3, (i * iHeight) + 3, iWidth - 3, iHeight - 3, pix2);

        if(ListSelectWord2.count() != 0)
        {
            for(int z = 0; z < ListSelectWord2.count(); z++)
            {
                strListSelectWord.clear();
                strListSelectWord.append(ListSelectWord2.at(z));

                if(i == int(strListSelectWord.at(0).unicode()) && j == int(strListSelectWord.at(1).unicode()))
                {
                    NumberWord = NumberPlain;
                    NumberPlain++;
                    rect.setRect((j * iWidth) + 3, (i * iHeight) + 3, iPixWidth - 1, iPixHeight - 1);
                    painter->drawText(rect, QString::number(NumberWord));
                }
            }
        }
    }

    painter->end();

    QImage imgExport;
    QByteArray baImage;

    QBuffer buffer(&baImage);
    buffer.open(QIODevice::WriteOnly);
    pix.save(&buffer, "PNG");
    imgExport.loadFromData(baImage);
    return imgExport;
}

TCrosswordText TCrosswordGrid::makeCrosswordText()
{
    TCrosswordText CrosswordText;
    QString strListSelect2;
    int iNumber = 1;
    int iDeletedWord = 0;

    for(int x = 0; x < slSelectWord.count(); x++)
    {
        strListSelect2.clear();
        strListSelect2.append(slSelectWord.at(x));

        if(strListSelect2.at(2) == 'h'/* && ui->lwQuestions->count() > x*/)
        {
            CrosswordText.ListOutH .append(QString::number(iNumber) + ". " + slQuestionT.at(x));
            CrosswordText.ListWordH.append(QString::number(iNumber) + ". " + slWordT.at(x) + ". ");
        }
        else
            if(strListSelect2.at(2) == 'v')
            {
            CrosswordText.ListOutV.append(QString::number(iNumber) + ". " + slQuestionT.at(x));
            CrosswordText.ListWordV.append(QString::number(iNumber) + ". " + slWordT.at(x) + ". ");
        }

        iNumber++;

        if (iDeletedWord < vNumbersDeletedWord.count() && iNumber - 1 == int(vNumbersDeletedWord.at(iDeletedWord)))
        {
            iNumber--;
            iDeletedWord++;
        }
    }

    return CrosswordText;
}

void TCrosswordGrid::openOpenKlestFile(QString sImportName)
{
    QFile rfile(sImportName);
    QTableWidgetItem* CellCrossword;
    QString sFileBegin;
    char *csdata;
    QBrush BrushBlack = QBrush(Qt::black, Qt::Dense1Pattern);
    QBrush BrushWhite = QBrush(Qt::SolidPattern);
    QColor col1(227, 226, 228);
    col1.setAlpha(255);
    BrushWhite.setColor(col1);
    ushort usdata;
    uint uidata;
    rfile.open(QIODevice::ReadOnly);
    csdata = new char[14];
    rfile.read(csdata, 14);
    sFileBegin = QString(csdata);
    delete [] csdata;
    rfile.read(reinterpret_cast<char*>(&uidata), sizeof(uidata));
    setRowCount(uidata);
    rfile.read(reinterpret_cast<char*>(&uidata), sizeof(uidata));
    setColumnCount(uidata);
    rfile.read(reinterpret_cast<char*>(&uidata), sizeof(uidata));

    usTable = new ushort*[rowCount()];

    for(int i = 0; i < rowCount(); i++)
        usTable[i] = new ushort[columnCount()];

    for(int i = 0; i < rowCount(); i++)
        for(int j = 0; j < columnCount();j++)
        {
        //Создание ячеек сетки
        CellCrossword = new QTableWidgetItem;
        setItem(i,j,CellCrossword);
        setRowHeight(i,uidata);
        setColumnWidth(j,uidata);
        rfile.read(reinterpret_cast<char*>(&usdata), sizeof(usdata));
        usdata -= 1;

        if(QChar(usdata) == 'b')
            item(i,j)->setBackground(BrushBlack);
        else
            item(i,j)->setBackground(BrushWhite);
    }

    //Загрузка данных кроссворда из файла в переменную
    rfile.read(reinterpret_cast<char*>(&uidata), sizeof(uidata));

    for(uint i = 0; i < uidata; i++)
    {
        rfile.read(reinterpret_cast<char*>(&usdata), sizeof(usdata));
        vBeginCoordWordRow.append(usdata);
    }

    for(uint i = 0; i < uidata; i++)
    {
        rfile.read(reinterpret_cast<char*>(&usdata), sizeof(usdata));
        vBeginCoordWordColumn.append(usdata);
    }

    for(uint i = 0; i < uidata; i++)
    {
        rfile.read(reinterpret_cast<char*>(&usdata), sizeof(usdata));
        vDirectionWord.append(usdata);
    }

    for(uint i = 0; i < uidata; i++)
    {
        rfile.read(reinterpret_cast<char*>(&usdata), sizeof(usdata));
        vLengthWord.append(usdata);
    }

    for(uint i = 0; i < uidata; i++)
    {
        QString strText;
        usdata = vLengthWord.at(i);
        csdata = new char[usdata];
        rfile.read(csdata, usdata);
        strText = QString(csdata);
        strText.truncate(usdata);
        delete [] csdata;
        slWordCheck << strText;
    }

    if(FileOptions->bModeWork == MODE_GUESSING)
    {
        slWordT.clear();

        for(int i = 0; i < vBeginCoordWordRow.count(); i++)
            slWordT << " ";
    }
    else
        slWordT = slWordCheck;

    for(uint i = 0; i < uidata; i++)
    {
        rfile.read(reinterpret_cast<char*>(&usdata), sizeof(usdata));
        slQuestionL.append(usdata);
    }

    slQuestionT.clear();

    for(uint i = 0; i < uidata; i++)
    {
        QString strText;
        strText = slQuestionL.at(i);
        usdata = strText.at(0).unicode();
        csdata = new char[usdata];
        rfile.read(csdata, usdata);
        strText = QString(csdata);
        strText.truncate(usdata);
        delete [] csdata;
        slQuestionT << strText;
    }

    emit sigCallFunction(DATA_TO_CROSSWORD);

    rfile.close();
    sFileNameCrossword = sImportName;
    sFileNameCrossword.replace(".ok", ".xwc");
    setSaved(true);
}

//Передача данных из переменной в кроссворд
void TCrosswordGrid::dataToCrossword()
{
    int CountCell;
    int j;
    int l;

    j = 0;
    l = 0;

    //if(FileOptions->bModeWork == MODE_EDITING)
    for(int i = 0; i < slWordT.count(); i++)
    {
        sWord = slWordT.at(i);
        CountCell = sWord.count();
        j = vBeginCoordWordRow.at(i);
        l = vBeginCoordWordColumn.at(i);

        if(vDirectionWord.at(i) == 'h')
            for(int k = 0; k < CountCell; k++)
            {
            QString Letter;
            Letter.append(sWord.data()[k]);

            if(Letter != "" && Letter != " ")
                item(j, l)->setText(Letter);

            Letter.clear();
            l++;
        }
        else
            if(vDirectionWord.at(i) == 'v')
                for(int k = 0; k < CountCell; k++)
                {
            QString Letter;
            Letter.append(sWord.data()[k]);

            if(Letter != "" && Letter != " ")
                item(j, l)->setText(Letter);

            Letter.clear();
            j++;
        }
    }
}

//Открытие кроссворда
bool TCrosswordGrid::openFile(QString sFile)
{
    QFile file;

    if(sFile.isEmpty()) file.setFileName(sFileNameCrossword);
    else file.setFileName(sFile);

    file.open(QIODevice::ReadOnly);
    QByteArray baImport;
    baImport = file.readAll();
    file.close();
    baImport = qUncompress(baImport);
    baImport = QByteArray::fromBase64(baImport);

    QTemporaryFile *tfCross = new QTemporaryFile;
    tfCross->open();
    tfCross->write(baImport);
    tfCross->flush();
    baImport.clear();
    file.setFileName(tfCross->fileName());
    QDataStream sfile(&file);

    if(file.open(QIODevice::ReadOnly))
    {
        QTableWidgetItem* CellCrossword;
        QBrush BrushBlack = QBrush(Qt::black, Qt::Dense1Pattern);
        QBrush BrushWhite = QBrush(Qt::SolidPattern);
        QColor col1(227, 226, 228);
        col1.setAlpha(255);
        BrushWhite.setColor(col1);
        QFont TableFont;
        ushort idata;
        int IntCount;
        QString sappVersion;

        try
        {
            sfile >> sappVersion;

            if (sappVersion != QCoreApplication::applicationVersion()) throw TError::WrongFileFormat();
        }
        catch(TError::WrongFileFormat)
        {
            QMessageBox::warning(0, tr("Detection warning"), tr("Wrong file format!"));
            emit sigCallFunction(S_CLOSE);
            return false;
        }
        catch(...)
        {
            QMessageBox::critical(0, tr("Detection error"), tr("An unknown error occurred!"));
            emit sigCallFunction(S_CLOSE);
            return false;
        }

        sfile >> IntCount;
        setRowCount(IntCount);
        sfile >> IntCount;
        setColumnCount(IntCount);
        sfile >> IntCount;
        sfile >> TableFont;
        setFont(TableFont);

        usTable = new ushort*[rowCount()];

        for(int i = 0; i < rowCount(); i++)
            usTable[i] = new ushort[columnCount()];

        for(int i = 0; i < rowCount(); i++)
            for(int j = 0; j < columnCount(); j++)
            {
            //Создание ячеек сетки
            CellCrossword = new QTableWidgetItem;
            setItem(i,j,CellCrossword);
            setRowHeight(i,IntCount);
            setColumnWidth(j,IntCount);
            sfile >> idata;
            idata -= 1;

            if(QChar(idata) == 'b')
                item(i,j)->setBackground(BrushBlack);
            else
                item(i,j)->setBackground(BrushWhite);
        }

        //Загрузка данных кроссворда из файла в переменную
        if(sFile.isEmpty())
        {
            if(!file.atEnd())
            {
                sfile >> vBeginCoordWordRow;
                sfile >> vBeginCoordWordColumn;
                sfile >> vDirectionWord;
                sfile >> vLengthWord;
                sfile >> slWordCheck;

                if(FileOptions->bModeWork == MODE_EDITING)
                {
                    slWordT = slWordCheck;
                }
                else
                    if (slWordT.isEmpty())
                    {
                    for(int i = 0; i < vBeginCoordWordRow.count(); i++)
                        slWordT << " ";
                }

                sfile >> slQuestionL;
                sfile >> slQuestionT;
                emit sigCallFunction(DATA_TO_CROSSWORD);
            }
        }

        file.close();
        tfCross->close();
        delete tfCross;
        setSaved(true);
    }

    return true;
}

//Сообщение об изменении таблицы кроссворда
int TCrosswordGrid::messageChangeGrid()
{
    QMessageBox*  mbox = new QMessageBox(tr("Change Grid"),
                                         tr("Grid was change, save") + "?",
                                         QMessageBox::Information,
                                         QMessageBox::Yes,
                                         QMessageBox::No,
                                         QMessageBox::Cancel);
    int inbox = mbox->exec();
    delete mbox;
    return inbox;
}

//Снятие выделения со всей сетки кроссворда
void TCrosswordGrid::tableUnselectAll()
{
    QBrush gbrush(Qt::green);
    QBrush wbrush(Qt::SolidPattern);
    QColor col1(227, 226, 228);
    col1.setAlpha(255);
    wbrush.setColor(col1);
    for(int i = 0; i < rowCount(); i++)
        for(int j = 0; j < columnCount(); j++)
        {
        item(i, j)->setSelected(false);

        if(item(i, j)->background() == gbrush) item(i, j)->setBackground(wbrush);
    }
}

//Определение координат первой ячейки выделенного слова в сетке
void TCrosswordGrid::firstCellWord(int Row, int Column)
{
    if(numberRowFirstLetterWord() > Row) setNumberRowFirstLetterWord(Row);
    if(numberColumnFirstLetterWord() > Column) setNumberColumnFirstLetterWord(Column);
}

//Открытие сохранения неполностью отгаданного кроссворда(загрузка данных)
int TCrosswordGrid::loadTemporary()
{
    QFile file(sFileNameCrossword);

    if(file.exists())
    {
        file.open(QIODevice::ReadOnly);
        QDataStream sfile(&file);
        slWordT.clear();
        sfile >> slWordT;
        file.close();
    }

    return 0;
}

void TCrosswordGrid::makeSelectWordCrossword(int Row, int Column, int State)
{
    setCountLetterInWord(0);

    if(State < 3)
    {
        setNumberRowFirstLetterWord(rowCount());
        setNumberColumnFirstLetterWord(columnCount());
    }

    if (State > 1)
    {
        QBrush brush(Qt::black, Qt::Dense1Pattern);
        QBrush gbrush(Qt::green);

        if(cDirectionWord == 'h' && Column != 0 && Column != columnCount() - 1 && item(Row, Column - 1)->background() == brush && item(
                   Row, Column + 1)->background() == brush && isAcrossLite() == ACROSS_DISABLE && !isSelectWord())
            cDirectionWord = 'v';
        else
            if(cDirectionWord == 'v' && Row != 0 && Row != rowCount() - 1 && item(Row - 1, Column)->background() == brush && item(
                    Row + 1, Column)->background() == brush && isAcrossLite() == ACROSS_DISABLE && !isSelectWord())
                cDirectionWord = 'h';
        else
            if(cDirectionWord == 'h' && Column == 0 && item(Row, Column + 1)->background() == brush && isAcrossLite() == ACROSS_DISABLE  &&
               !isSelectWord())
                cDirectionWord = 'v';
        else
            if(cDirectionWord == 'v' && Row == 0 && item(Row + 1, Column)->background() == brush && isAcrossLite() == ACROSS_DISABLE &&
               !isSelectWord())
                cDirectionWord = 'h';

        if(cDirectionWord == 'h' && Column == columnCount() - 1 && item(Row, Column - 1)->background() == brush && isAcrossLite() == ACROSS_DISABLE
           && !isSelectWord())
            cDirectionWord = 'v';
        else
            if(cDirectionWord == 'v' && Row == rowCount() - 1 && item(Row - 1, Column)->background() == brush && isAcrossLite() == ACROSS_DISABLE &&
               !isSelectWord())
                cDirectionWord = 'h';

        //Выделение слова в сетке кроссворда, соответственно направлению
        if (item(Row, Column)->background() != brush && cDirectionWord == 'h')
        {
            for(int i = Column; i >= 0; i--)
                if(item(Row, i)->background() != brush)
                {
                item(Row, i)->setBackground(gbrush);
                setCountLetterInWord(countLetterInWord() + 1);

                firstCellWord(Row, i);
            }
            else break;

            for(int i = Column; i < columnCount(); i++)
                if(item(Row,i)->background() != brush)
                {
                item(Row, i)->setBackground(gbrush);
                setCountLetterInWord(countLetterInWord() + 1);

                firstCellWord(Row, i);
            }
            else break;
        }
        else
            if (item(Row, Column)->background() != brush && cDirectionWord == 'v')
            {
            for (int i = Row; i >= 0; i--)
                if(item(i, Column)->background() != brush)
                {
                item(i, Column)->setBackground(gbrush);
                setCountLetterInWord(countLetterInWord() + 1);

                firstCellWord(i, Column);
            }
            else break;

            for (int i = Row; i < rowCount(); i++)
                if(item(i, Column)->background() != brush)
                {
                item(i, Column)->setBackground(gbrush);
                setCountLetterInWord(countLetterInWord() + 1);

                firstCellWord(i, Column);
            }
            else break;
        }

        //---------------------------------------------------------------------------------------------------------

        Row = numberRowFirstLetterWord();
        Column = numberColumnFirstLetterWord();
        SelectedWordToEdit.clear();

        if(cDirectionWord == 'h')
            for(int i = 0; i < countLetterInWord() - 1; i++)
            {
            if(item(Row, Column)->text() != "" && item(Row, Column)->text().data()[0] != ' ')
                SelectedWordToEdit.append(item(Row, Column)->text().data()[0]);
            else
            {
                SelectedWordToEdit.clear();
                break;
            }

            Column++;
        }

        if (cDirectionWord == 'v')
            for(int i = 0; i < countLetterInWord() - 1; i++)
            {
            if(item(Row, Column)->text() != "" && item(Row, Column)->text().data()[0] != ' ')
                SelectedWordToEdit.append(item(Row, Column)->text().data()[0]);
            else
            {
                SelectedWordToEdit.clear();
                break;
            }

            Row++;
        }
    }

    if (State < 2)
    {
        if(cDirectionWord == 'h' && Column != 0 && Column != columnCount() - 1 && usTable[Row][Column - 1] == CELL_BLACK && usTable[
                   Row][Column + 1] == CELL_BLACK && isAcrossLite() == ACROSS_DISABLE)
            cDirectionWord = 'v';
        else
            if(cDirectionWord == 'v' && Row != 0 && Row != rowCount() - 1 && usTable[Row - 1][Column] == CELL_BLACK && usTable[
                    Row + 1][Column] == CELL_BLACK && isAcrossLite() == ACROSS_DISABLE)
                cDirectionWord = 'h';
        else
            if(cDirectionWord == 'h' && Column == 0 && usTable[Row][Column + 1] == CELL_BLACK && isAcrossLite() == ACROSS_DISABLE)
                cDirectionWord = 'v';
        else
            if(cDirectionWord == 'v' && Row == 0 && usTable[Row + 1][Column] == CELL_BLACK && isAcrossLite() == ACROSS_DISABLE)
                cDirectionWord = 'h';

        if(cDirectionWord == 'h' && Column == columnCount() - 1 && usTable[Row][Column - 1] == CELL_BLACK && isAcrossLite() == ACROSS_DISABLE)
            cDirectionWord = 'v';
        else
            if(cDirectionWord == 'v' && Row == rowCount() - 1 && usTable[Row - 1][Column] == CELL_BLACK && isAcrossLite() == ACROSS_DISABLE)
                cDirectionWord = 'h';

        //Выделение слова в массиве кроссворда, соответственно направлению
        if(usTable[Row][Column] != CELL_BLACK && cDirectionWord == 'h')
        {
            for(int i = Column; i >= 0; i--)
                if(usTable[Row][i] != CELL_BLACK)
                {
                usTable[Row][i] = CELL_LIME;
                setCountLetterInWord(countLetterInWord() + 1);

                //Определение координаты первой ячейки слова
                firstCellWord(Row, i);
            }
            else break;

            for(int i = Column; i < columnCount(); i++)
                if(usTable[Row][i] != CELL_BLACK)
                {
                usTable[Row][i] = CELL_LIME;
                setCountLetterInWord(countLetterInWord() + 1);

                //Определение координаты первой ячейки слова
                firstCellWord(Row, i);
            }
            else break;
        }
        else
            if(usTable[Row][Column] != CELL_BLACK &&
               cDirectionWord == 'v')
            {
            for(int i = Row; i >= 0; i--)
                if(usTable[i][Column] != CELL_BLACK)
                {
                usTable[i][Column] = CELL_LIME;
                setCountLetterInWord(countLetterInWord() + 1);

                //Определение координаты первой ячейки слова
                firstCellWord(i, Column);
            }
            else break;

            for(int i = Row; i < rowCount(); i++)
                if(usTable[i][Column] != CELL_BLACK)
                {
                usTable[i][Column] = CELL_LIME;
                setCountLetterInWord(countLetterInWord() + 1);

                //Определение координаты первой ячейки слова
                firstCellWord(i, Column);
            }
            else break;
        }
    }

    //---------------------------------------------------------------------------------------------------------

    Row = numberRowFirstLetterWord();
    Column = numberColumnFirstLetterWord();
    SelectedWordToCheck.clear();

    if(cDirectionWord == 'h')
        for(int i = 0; i < countLetterInWord() - 1; i++)
        {
        if (State == 0)
        {
            if(usTable[Row][Column] == CELL_BACK)
                SelectedWordToCheck.append(" ");
            else
                SelectedWordToCheck.append(item(Row, Column)->text().data()[0]);
        }
        else
        {
            if (item(Row, Column)->text() == "")
                SelectedWordToCheck.append(" ");
            else
                SelectedWordToCheck.append(item(Row, Column)->text().data()[0]);
        }

        Column++;
    }

    if(cDirectionWord == 'v')
        for(int i = 0; i < countLetterInWord() - 1; i++)
        {
        if (State == 0)
        {
            if(usTable[Row][Column] == CELL_BACK)
                SelectedWordToCheck.append(" ");
            else
                SelectedWordToCheck.append(item(Row, Column)->text().data()[0]);
        }
        else
        {
            if(item(Row, Column)->text() == "")
                SelectedWordToCheck.append(" ");
            else
                SelectedWordToCheck.append(item(Row, Column)->text().data()[0]);
        }

        Row++;
    }

    if(SelectedWordToCheck.count() == 1)
    {
        SelectedWordToCheck.clear();

        if (State == 2) tableUnselectAll();
    }

    if (State > 1)
    {
        if(SelectedWordToEdit.count() == 1)
        {
            SelectedWordToEdit.clear();
            tableUnselectAll();
        }

        if(SelectedWordToCheck.count() == 1)
        {
            SelectedWordToCheck.clear();
            tableUnselectAll();
        }
    }
}

//Сохранение неполностью отгаданного кроссворда
void TCrosswordGrid::saveToFileTemporary()
{
    if (isAcrossLite() == ACROSS_ENABLE)
        sFileNameCrossword.replace("txt", "txtsv");
    else
        sFileNameCrossword.replace("xwc", "xwcsv");

    QFile file(sFileNameCrossword);
    if(file.open(QIODevice::WriteOnly))
    {
        QDataStream sfile(&file);

        sfile << slWordT;
        file.close();
        setSaved(true);
    }
}

//Сохранение кроссворда под другим именем
void TCrosswordGrid::slotSaveAs()
{
    QString stmpFileName;
    QString sFilter;

    QFileDialog *pSaveCrossword;

    pSaveCrossword = new QFileDialog(0, tr("Crossword save"), tr("crossword1"),
                                     tr("Crossword format program of Klest-crossword") + " (*.xwc);;" +
                                     tr("Template crossword format") + " (*.xwctpl);;");
    pSaveCrossword->setAcceptMode(QFileDialog::AcceptSave);

    if(pSaveCrossword->exec() == QFileDialog::Accepted)
    {
        stmpFileName = pSaveCrossword->selectedFiles().at(0);

        if (stmpFileName.isEmpty())
        {
            delete pSaveCrossword;
            return;
        }

        sFilter = pSaveCrossword->selectedFilter();
        sFilter.remove(0, sFilter.indexOf("*"));
        sFilter.remove(sFilter.length() - 1, 1);
        sFilter.replace("*", "");

        if(stmpFileName.indexOf(sFilter) == -1) stmpFileName += sFilter;
    }
    else
    {
        delete pSaveCrossword;
        return;
    }

    delete pSaveCrossword;

    if(sFilter == ".xwctpl")
    {
        saveToFile(stmpFileName);
    }

    if(sFilter == ".xwc")
    {
        sFileNameCrossword = stmpFileName;
        saveToFile("");
    }

    return;
}

void TCrosswordGrid::delete_usTable()
{
    for(int i = 0; i < rowCount(); i++)
        delete [] usTable[i];

    delete usTable;
}

void TCrosswordGrid::create_usTable()
{
    usTable = new ushort*[rowCount()];

    for(int i = 0; i < rowCount(); i++)
        usTable[i] = new ushort[columnCount()];
}

//Редактирование данных кроссворда
void TCrosswordGrid::dataCrosswordEdit()
{
    if (sWord.count() == 1) return;

    //Изменение данных
    if(isEditDataCrossword() == DATA_EDIT)
    {
        sWord = sWord.toUpper();
        sWord.truncate(countLetterInWord() - 1);

        if(sWord != "" && sWord != " ") slWordT.replace(numberCurrentQuestion(), sWord);
        if(sQuestion != "" && sWord != " ") slQuestionT.replace(numberCurrentQuestion(), sQuestion);
    }

    //Добавление данных
    int i, j;
    i = numberRowFirstLetterWord();
    j = numberColumnFirstLetterWord();

    if(isEditDataCrossword() == DATA_ADD)
    {
        sWord = sWord.toUpper();
        sWord.truncate(countLetterInWord() - 1);
        slWordT.append(sWord);
        slQuestionT.append(sQuestion);
        setNumberCurrentQuestion(slQuestionT.count());
    }

    //Ввод слова в сетку
    if(cDirectionWord == 'h' && sWord != "")
        for(int k = 0; k < countLetterInWord() - 1; k++)
        {
        QString Letter;
        Letter.append(sWord.data()[k].toUpper());
        item(i, j)->setText(Letter);
        Letter.clear();
        j++;
    }
    else
        if(cDirectionWord == 'v' && sWord != "")
            for(int k = 0; k < countLetterInWord() - 1; k++)
            {
        QString Letter;
        Letter.append(sWord.data()[k].toUpper());
        item(i, j)->setText(Letter);
        Letter.clear();
        i++;
    }
}

void TCrosswordGrid::openAcrossLiteTXT(QString FileName)
{
    QString sTmp;

    if(FileName != "")
    {
        sFileNameCrossword = FileName;

        QFile ifile(FileName);
        ifile.open(QIODevice::ReadOnly);

        sTmp = ifile.readLine();
        sTmp = ifile.readLine();

        if(sTmp.indexOf("<TITLE>") != -1)
        {
            sTmp = ifile.readLine();
            sTmp.replace("	", "");
            sTmp.replace("\n", "");
            sAcrossTitle = sTmp;
        }

        sTmp = ifile.readLine();

        if(sTmp.indexOf("<AUTHOR>") != -1)
        {
            sTmp = ifile.readLine();
            sTmp.replace("	", "");
            sTmp.replace("\n", "");
            sAcrossAuthor = sTmp;
        }

        sTmp = ifile.readLine();

        if(sTmp.indexOf("<COPYRIGHT>") != -1)
        {
            sTmp = ifile.readLine();
            sTmp.replace("	", "");
            sTmp.replace("\n", "");
            sAcrossCopyright = sTmp;
        }

        sTmp = ifile.readLine();

        if(sTmp.indexOf("<SIZE>") != -1)
        {
            sTmp = ifile.readLine();
            sTmp.replace("	", "");
            sTmp.replace("\n", "");
            QString s1, s2;
            s1.clear();
            s2.clear();
            int i, j, k, l;

            for(i = 0; i < sTmp.length(); i++)
            {

                if(sTmp.at(i) == 'x')
                {
                    i++;
                    break;
                }

                s1.append(sTmp.at(i));
            }

            for(i = i; i < sTmp.length(); i++)
            {
                s2.append(sTmp.at(i));
            }

            bool bOk;
            k = s1.toInt(&bOk, 10);
            s2.truncate(s2.length() - 1);
            l = s2.toInt(&bOk, 10);

            setRowCount(k);
            setColumnCount(l);

            for(i = 0; i < k; i++)
            {
                setRowHeight(i, 30);
            }

            for(i = 0; i < l; i++)
            {
                setColumnWidth(i, 30);
            }

            QTableWidgetItem* CellCrossword;
            QBrush BrushWhite = QBrush(Qt::SolidPattern);
            QColor col1(227, 226, 228);
            col1.setAlpha(255);
            BrushWhite.setColor(col1);

            usTable = new ushort*[rowCount()];

            for(int i = 0; i < rowCount(); i++)
                usTable[i] = new ushort[columnCount()];

            for(i = 0; i < rowCount(); i++)
                for(j = 0; j < columnCount(); j++)
                {
                CellCrossword = new QTableWidgetItem;
                setItem(i,j, CellCrossword);
                item(i, j)->setBackground(BrushWhite);
            }
        }

        sTmp = ifile.readLine();

        QStringList slAllWord;
        QStringList slTempAllWord;
        QStringList slQuestionT1;
        QStringList slQuestionT2;
        QStringList lListWord;
        slAllWord.clear();
        slTempAllWord.clear();
        int iLength = 0;
        if(sTmp.indexOf("<GRID>") != -1)
        {
            int j;
            j = 0;
            sTmp = ifile.readLine();
            sTmp.replace("	", "");
            sTmp.replace("\n", "");
            iLength = sTmp.length() - 1;
            QString sTempWord,
            sTempAllWord;
            QBrush BrushBlack = QBrush(Qt::black, Qt::Dense1Pattern);

            while(sTmp.indexOf("<ACROSS>") == -1)
            {
                sTmp.truncate(sTmp.length() - 1);
                sTempAllWord = sTmp;
                slAllWord << sTempAllWord;

                for(int i = 0; i < sTmp.length(); i++)
                {
                    sTempWord.append(sTmp.at(i));

                    if(sTmp.at(i) == '.') item(j, i)->setBackground(BrushBlack);

                    if(sTmp.at(i) == '.')
                    {
                        sTempWord.replace(".", "");

                        if(sTempWord != "") slWordCheck.append(sTempWord);

                        sTempWord.clear();
                    }
                }

                if(sTempWord != "")
                {
                    //sTempWord.truncate(sTempWord.length() - 1);

                    if(sTempWord != "") slWordCheck.append(sTempWord);

                    sTempWord.clear();
                }

                sTmp = ifile.readLine();
                sTmp.replace("	", "");
                sTmp.replace("\n", "");
                j++;
            }

            sTempAllWord.clear();
            sTempWord.clear();

            if(slAllWord.count() > 0)
            {
                for(int k = 0; k < iLength; k++)
                    for(int l = 0; l < slAllWord.count(); l++)
                    {
                    sTempAllWord.append(slAllWord.at(l));

                    if(sTempAllWord.at(k) == '.')
                    {
                        if(sTempWord != "") slTempAllWord << sTempWord;

                        sTempWord.clear();
                    }
                    else sTempWord.append(sTempAllWord.at(k));

                    if(l == slAllWord.count() - 1)
                    {
                        if(sTempWord != "") slTempAllWord << sTempWord;

                        sTempWord.clear();
                    }

                    sTempAllWord.clear();
                }


                if(slTempAllWord.count() > 0)
                {
                    for(int i = 0; i < slTempAllWord.count(); i++)
                        slWordCheck << slTempAllWord.at(i);

                    //ListWord.clear();
                    //lListWord.clear();
                    lListWord = slWordCheck;

                }
            }

        }

        slQuestionT1.clear();
        slQuestionT2.clear();
        slQuestionT.clear();

        while(!ifile.atEnd())
        {
            sTmp = ifile.readLine();
            if(sTmp.indexOf("<DOWN>") == -1)
            {
                sTmp.replace("	", "");
                sTmp.replace("\n", "");
                sTmp.truncate(sTmp.length() - 1);
                slQuestionT1 << sTmp;
            }
            else
            {
                slQuestionT2 = slQuestionT1;
                slQuestionT1.clear();
            }
        }

        ifile.close();

        setInsertCoordinates(InsertCoordinatesWords(ACROSS_ENABLE));

        for(int i = 0; i < slQuestionT2.count(); i++)
            slQuestionT << slQuestionT2.at(i);

        for(int i = 0; i < slQuestionT1.count(); i++)
            slQuestionT << slQuestionT1.at(i);


        if(FileOptions->bModeWork == MODE_GUESSING)
        {
            if (slWordT.isEmpty())
            {
                for(int i = 0; i < slWordCheck.count(); i++)
                    slWordT.append(" ");
            }
        }
        else slWordT = lListWord;

        setAcrossLite(ACROSS_ENABLE);
    }
}

//Вставка координат слов
bool TCrosswordGrid::InsertCoordinatesWords(bool bAcross)
{
    QString strListSelectWord;
    cDirectionWord = 'h';
    vBeginCoordWordRow.clear();
    vBeginCoordWordColumn.clear();
    vDirectionWord.clear();
    vLengthWord.clear();
    slSelectWord.clear();

    if (bAcross == ACROSS_DISABLE)
    {
        slWordT.clear();
        cDirectionWord = 'v';
    }

    slQuestionT.clear();
    makeTableToMassive();

    if (bAcross == ACROSS_DISABLE)
    {
        for(int i = 0; i < rowCount(); i++)
        {
            for(int j = 0; j < columnCount(); j++)
            {
                //TableUnselectAll();

                //Если текущая ячейка пустая(без буквы)
                if(usTable[i][j] == CELL_BACK)
                {
                    //setCurrentCell(i, j);
                    //Выделение слова
                    makeSelectWordCrossword(i, j, 0);
                    strListSelectWord.clear();
                    strListSelectWord.append(QChar(numberRowFirstLetterWord()));
                    strListSelectWord.append(QChar(numberColumnFirstLetterWord()));
                    strListSelectWord.append(cDirectionWord);
                    strListSelectWord.append(QChar(ushort(SelectedWordToCheck.length())));

                    if (SelectedWordToCheck.count() > 1 /*|| SelectedWordToEdit.count() > 1*/)
                    {
                        slSelectWord.append(strListSelectWord);
                        vBeginCoordWordRow.append(ushort(numberRowFirstLetterWord()));
                        vBeginCoordWordColumn.append(ushort(numberColumnFirstLetterWord()));
                        vDirectionWord.append(cDirectionWord);
                        vLengthWord.append(ushort(SelectedWordToCheck.length()));
                        slRightSelectedWord.append("no");
                    }

                    if (cDirectionWord == 'h')
                        for(int k = 0; k < SelectedWordToCheck.length(); k++)
                        {
                        usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                        setNumberColumnFirstLetterWord(numberColumnFirstLetterWord() + 1);
                    }
                    else
                        for(int k = 0; k < SelectedWordToCheck.length(); k++)
                        {
                        usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                        setNumberRowFirstLetterWord(numberRowFirstLetterWord() + 1);
                    }

                    if(j != 0 && j != columnCount() - 1 && i != 0 && i != rowCount() - 1)
                    {
                        if (usTable[i - 1][j] == CELL_BLACK && usTable[i][j - 1] == CELL_BLACK &&
                            (usTable[i + 1][j] == CELL_BACK || usTable[i + 1][j] == CELL_BACK_FILL) &&
                            (usTable[i][j + 1] == CELL_BACK || usTable[i][j + 1] == CELL_BACK_FILL) &&
                            isAcrossLite() == ACROSS_DISABLE)
                        {
                            if (cDirectionWord == 'h')
                                cDirectionWord = 'v';
                            else
                                cDirectionWord = 'h';

                            makeSelectWordCrossword(i, j, 0);
                            strListSelectWord.clear();
                            strListSelectWord.append(QChar(numberRowFirstLetterWord()));
                            strListSelectWord.append(QChar(numberColumnFirstLetterWord()));
                            strListSelectWord.append(cDirectionWord);
                            strListSelectWord.append(QChar(ushort(SelectedWordToCheck.length())));

                            if (SelectedWordToCheck.count() > 1 /*|| SelectedWordToEdit.count() > 1*/)
                            {
                                slSelectWord.append(strListSelectWord);
                                vBeginCoordWordRow.append(ushort(numberRowFirstLetterWord()));
                                vBeginCoordWordColumn.append(ushort(numberColumnFirstLetterWord()));
                                vDirectionWord.append(cDirectionWord);
                                vLengthWord.append(ushort(SelectedWordToCheck.length()));
                                slRightSelectedWord.append("no");
                            }

                            if (cDirectionWord == 'h')
                                for(int k = 0; k < SelectedWordToCheck.length(); k++)
                                {
                                usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                                setNumberColumnFirstLetterWord(numberColumnFirstLetterWord() + 1);
                            }
                            else
                                for(int k = 0; k < SelectedWordToCheck.length(); k++)
                                {
                                usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                                setNumberRowFirstLetterWord(numberRowFirstLetterWord() + 1);
                            }
                        }

                        if (usTable[i - 1][j + SelectedWordToCheck.length() - 1] == CELL_BLACK && usTable[i][j + SelectedWordToCheck.length() - 1 + 1] == CELL_BLACK &&
                            (usTable[i + 1][j + SelectedWordToCheck.length() - 1] == CELL_BACK || usTable[i + 1][j + SelectedWordToCheck.length() - 1] == CELL_BACK_FILL) &&
                            (usTable[i][j + SelectedWordToCheck.length() - 1 - 1] == CELL_BACK || usTable[i][j + SelectedWordToCheck.length() - 1 - 1] == CELL_BACK_FILL) &&
                            isAcrossLite() == ACROSS_DISABLE)
                        {
                            if (cDirectionWord == 'h')
                                cDirectionWord = 'v';
                            else
                                cDirectionWord = 'h';

                            makeSelectWordCrossword(i, j + SelectedWordToCheck.length() - 1, 0);
                            strListSelectWord.clear();
                            strListSelectWord.append(QChar(numberRowFirstLetterWord()));
                            strListSelectWord.append(QChar(numberColumnFirstLetterWord()));
                            strListSelectWord.append(cDirectionWord);
                            strListSelectWord.append(QChar(ushort(SelectedWordToCheck.length())));

                            if (SelectedWordToCheck.count() > 1 /*|| SelectedWordToEdit.count() > 1*/)
                            {
                                slSelectWord.append(strListSelectWord);
                                vBeginCoordWordRow.append(ushort(numberRowFirstLetterWord()));
                                vBeginCoordWordColumn.append(ushort(numberColumnFirstLetterWord()));
                                vDirectionWord.append(cDirectionWord);
                                vLengthWord.append(ushort(SelectedWordToCheck.length()));
                                slRightSelectedWord.append("no");
                            }

                            if (cDirectionWord == 'h')
                                for(int k = 0; k < SelectedWordToCheck.length(); k++)
                                {
                                usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                                setNumberColumnFirstLetterWord(numberColumnFirstLetterWord() + 1);
                            }
                            else
                                for(int k = 0; k < SelectedWordToCheck.length(); k++)
                                {
                                usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                                setNumberRowFirstLetterWord(numberRowFirstLetterWord() + 1);
                            }
                        }
                    }

                    if(j == 0 && i == 0)
                    {
                        if ((usTable[i + 1][j] == CELL_BACK || usTable[i + 1][j] == CELL_BACK_FILL) &&
                            (usTable[i][j + 1] == CELL_BACK || usTable[i][j + 1] == CELL_BACK_FILL) &&
                            isAcrossLite() == ACROSS_DISABLE)
                        {
                            if (cDirectionWord == 'h')
                                cDirectionWord = 'v';
                            else
                                cDirectionWord = 'h';

                            makeSelectWordCrossword(i, j, 0);
                            strListSelectWord.clear();
                            strListSelectWord.append(QChar(numberRowFirstLetterWord()));
                            strListSelectWord.append(QChar(numberColumnFirstLetterWord()));
                            strListSelectWord.append(cDirectionWord);
                            strListSelectWord.append(QChar(ushort(SelectedWordToCheck.length())));

                            if (SelectedWordToCheck.count() > 1 /*|| SelectedWordToEdit.count() > 1*/)
                            {
                                slSelectWord.append(strListSelectWord);
                                vBeginCoordWordRow.append(ushort(numberRowFirstLetterWord()));
                                vBeginCoordWordColumn.append(ushort(numberColumnFirstLetterWord()));
                                vDirectionWord.append(cDirectionWord);
                                vLengthWord.append(ushort(SelectedWordToCheck.length()));
                                slRightSelectedWord.append("no");
                            }

                            if (cDirectionWord == 'h')
                                for(int k = 0; k < SelectedWordToCheck.length(); k++)
                                {
                                usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                                setNumberColumnFirstLetterWord(numberColumnFirstLetterWord() + 1);
                            }
                            else
                                for(int k = 0; k < SelectedWordToCheck.length(); k++)
                                {
                                usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                                setNumberRowFirstLetterWord(numberRowFirstLetterWord() + 1);
                            }
                        }

                        if ((usTable[i + 1][j + SelectedWordToCheck.length() - 1] == CELL_BACK || usTable[i + 1][j + SelectedWordToCheck.length() - 1] == CELL_BACK_FILL) &&
                            (usTable[i][j + SelectedWordToCheck.length() - 1 - 1] == CELL_BACK || usTable[i][j + SelectedWordToCheck.length() - 1 - 1] == CELL_BACK_FILL) &&
                            isAcrossLite() == ACROSS_DISABLE)
                        {
                            if (cDirectionWord == 'h')
                                cDirectionWord = 'v';
                            else
                                cDirectionWord = 'h';

                            makeSelectWordCrossword(i, j + SelectedWordToCheck.length() - 1, 0);
                            strListSelectWord.clear();
                            strListSelectWord.append(QChar(numberRowFirstLetterWord()));
                            strListSelectWord.append(QChar(numberColumnFirstLetterWord()));
                            strListSelectWord.append(cDirectionWord);
                            strListSelectWord.append(QChar(ushort(SelectedWordToCheck.length())));

                            if (SelectedWordToCheck.count() > 1 /*|| SelectedWordToEdit.count() > 1*/)
                            {
                                slSelectWord.append(strListSelectWord);
                                vBeginCoordWordRow.append(ushort(numberRowFirstLetterWord()));
                                vBeginCoordWordColumn.append(ushort(numberColumnFirstLetterWord()));
                                vDirectionWord.append(cDirectionWord);
                                vLengthWord.append(ushort(SelectedWordToCheck.length()));
                                slRightSelectedWord.append("no");
                            }

                            if (cDirectionWord == 'h')
                                for(int k = 0; k < SelectedWordToCheck.length(); k++)
                                {
                                usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                                setNumberColumnFirstLetterWord(numberColumnFirstLetterWord() + 1);
                            }
                            else
                                for(int k = 0; k < SelectedWordToCheck.length(); k++)
                                {
                                usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                                setNumberRowFirstLetterWord(numberRowFirstLetterWord() + 1);
                            }
                        }
                    }

                    if(j != 0 && j != columnCount() - 1 && i == 0)
                    {
                        if (usTable[i][j - 1] == CELL_BLACK &&
                            (usTable[i + 1][j] == CELL_BACK || usTable[i + 1][j] == CELL_BACK_FILL) &&
                            (usTable[i][j + 1] == CELL_BACK || usTable[i][j + 1] == CELL_BACK_FILL) &&
                            isAcrossLite() == ACROSS_DISABLE)
                        {
                            if (cDirectionWord == 'h')
                                cDirectionWord = 'v';
                            else
                                cDirectionWord = 'h';

                            makeSelectWordCrossword(i, j, 0);
                            strListSelectWord.clear();
                            strListSelectWord.append(QChar(numberRowFirstLetterWord()));
                            strListSelectWord.append(QChar(numberColumnFirstLetterWord()));
                            strListSelectWord.append(cDirectionWord);
                            strListSelectWord.append(QChar(ushort(SelectedWordToCheck.length())));

                            if (SelectedWordToCheck.count() > 1 /*|| SelectedWordToEdit.count() > 1*/)
                            {
                                slSelectWord.append(strListSelectWord);
                                vBeginCoordWordRow.append(ushort(numberRowFirstLetterWord()));
                                vBeginCoordWordColumn.append(ushort(numberColumnFirstLetterWord()));
                                vDirectionWord.append(cDirectionWord);
                                vLengthWord.append(ushort(SelectedWordToCheck.length()));
                                slRightSelectedWord.append("no");
                            }

                            if (cDirectionWord == 'h')
                                for(int k = 0; k < SelectedWordToCheck.length(); k++)
                                {
                                usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                                setNumberColumnFirstLetterWord(numberColumnFirstLetterWord() + 1);
                            }
                            else
                                for(int k = 0; k < SelectedWordToCheck.length(); k++)
                                {
                                usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                                setNumberRowFirstLetterWord(numberRowFirstLetterWord() + 1);
                            }
                        }

                        if (usTable[i][j + SelectedWordToCheck.length() - 1 + 1] == CELL_BLACK &&
                            (usTable[i + 1][j + SelectedWordToCheck.length() - 1] == CELL_BACK || usTable[i + 1][j + SelectedWordToCheck.length() - 1] == CELL_BACK_FILL) &&
                            (usTable[i][j + SelectedWordToCheck.length() - 1 - 1] == CELL_BACK || usTable[i][j + SelectedWordToCheck.length() - 1 - 1] == CELL_BACK_FILL) &&
                            isAcrossLite() == ACROSS_DISABLE)
                        {
                            if (cDirectionWord == 'h')
                                cDirectionWord = 'v';
                            else
                                cDirectionWord = 'h';

                            makeSelectWordCrossword(i, j + SelectedWordToCheck.length() - 1, 0);
                            strListSelectWord.clear();
                            strListSelectWord.append(QChar(numberRowFirstLetterWord()));
                            strListSelectWord.append(QChar(numberColumnFirstLetterWord()));
                            strListSelectWord.append(cDirectionWord);
                            strListSelectWord.append(QChar(ushort(SelectedWordToCheck.length())));

                            if (SelectedWordToCheck.count() > 1 /*|| SelectedWordToEdit.count() > 1*/)
                            {
                                slSelectWord.append(strListSelectWord);
                                vBeginCoordWordRow.append(ushort(numberRowFirstLetterWord()));
                                vBeginCoordWordColumn.append(ushort(numberColumnFirstLetterWord()));
                                vDirectionWord.append(cDirectionWord);
                                vLengthWord.append(ushort(SelectedWordToCheck.length()));
                                slRightSelectedWord.append("no");
                            }

                            if (cDirectionWord == 'h')
                                for(int k = 0; k < SelectedWordToCheck.length(); k++)
                                {
                                usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                                setNumberColumnFirstLetterWord(numberColumnFirstLetterWord() + 1);
                            }
                            else
                                for(int k = 0; k < SelectedWordToCheck.length(); k++)
                                {
                                usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                                setNumberRowFirstLetterWord(numberRowFirstLetterWord() + 1);
                            }
                        }
                    }

                    if(j == 0 && i != 0 && i != rowCount() - 1)
                    {
                        if (usTable[i - 1][j] == CELL_BLACK &&
                            (usTable[i + 1][j] == CELL_BACK || usTable[i + 1][j] == CELL_BACK_FILL) &&
                            (usTable[i][j + 1] == CELL_BACK || usTable[i][j + 1] == CELL_BACK_FILL) &&
                            isAcrossLite() == ACROSS_DISABLE)
                        {
                            if (cDirectionWord == 'h')
                                cDirectionWord = 'v';
                            else
                                cDirectionWord = 'h';

                            makeSelectWordCrossword(i, j, 0);
                            strListSelectWord.clear();
                            strListSelectWord.append(QChar(numberRowFirstLetterWord()));
                            strListSelectWord.append(QChar(numberColumnFirstLetterWord()));
                            strListSelectWord.append(cDirectionWord);
                            strListSelectWord.append(QChar(ushort(SelectedWordToCheck.length())));

                            if (SelectedWordToCheck.count() > 1 /*|| SelectedWordToEdit.count() > 1*/)
                            {
                                slSelectWord.append(strListSelectWord);
                                vBeginCoordWordRow.append(ushort(numberRowFirstLetterWord()));
                                vBeginCoordWordColumn.append(ushort(numberColumnFirstLetterWord()));
                                vDirectionWord.append(cDirectionWord);
                                vLengthWord.append(ushort(SelectedWordToCheck.length()));
                                slRightSelectedWord.append("no");
                            }

                            if (cDirectionWord == 'h')
                                for(int k = 0; k < SelectedWordToCheck.length(); k++)
                                {
                                usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                                setNumberColumnFirstLetterWord(numberColumnFirstLetterWord() + 1);
                            }
                            else
                                for(int k = 0; k < SelectedWordToCheck.length(); k++)
                                {
                                usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                                setNumberRowFirstLetterWord(numberRowFirstLetterWord() + 1);
                            }
                        }

                        if (usTable[i - 1][j + SelectedWordToCheck.length() - 1] == CELL_BLACK &&
                            (usTable[i + 1][j + SelectedWordToCheck.length() - 1] == CELL_BACK || usTable[i + 1][j + SelectedWordToCheck.length() - 1] == CELL_BACK_FILL) &&
                            (usTable[i][j + SelectedWordToCheck.length() - 1 - 1] == CELL_BACK || usTable[i][j + SelectedWordToCheck.length() - 1 - 1] == CELL_BACK_FILL) &&
                            isAcrossLite() == ACROSS_DISABLE)
                        {
                            if (cDirectionWord == 'h')
                                cDirectionWord = 'v';
                            else
                                cDirectionWord = 'h';

                            makeSelectWordCrossword(i, j + SelectedWordToCheck.length() - 1, 0);
                            strListSelectWord.clear();
                            strListSelectWord.append(QChar(numberRowFirstLetterWord()));
                            strListSelectWord.append(QChar(numberColumnFirstLetterWord()));
                            strListSelectWord.append(cDirectionWord);
                            strListSelectWord.append(QChar(ushort(SelectedWordToCheck.length())));

                            if (SelectedWordToCheck.count() > 1 /*|| SelectedWordToEdit.count() > 1*/)
                            {
                                slSelectWord.append(strListSelectWord);
                                vBeginCoordWordRow.append(ushort(numberRowFirstLetterWord()));
                                vBeginCoordWordColumn.append(ushort(numberColumnFirstLetterWord()));
                                vDirectionWord.append(cDirectionWord);
                                vLengthWord.append(ushort(SelectedWordToCheck.length()));
                                slRightSelectedWord.append("no");
                            }

                            if (cDirectionWord == 'h')
                                for(int k = 0; k < SelectedWordToCheck.length(); k++)
                                {
                                usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                                setNumberColumnFirstLetterWord(numberColumnFirstLetterWord() + 1);
                            }
                            else
                                for(int k = 0; k < SelectedWordToCheck.length(); k++)
                                {
                                usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                                setNumberRowFirstLetterWord(numberRowFirstLetterWord() + 1);
                            }
                        }
                    }
                }
            }
        }

        usTableUnselectAll();
        cDirectionWord = 'v';
    }
    else
    {
        for(int z = 0; z < 2; z++)
        {
            for(int i = 0; i < rowCount(); i++)
            {
                for(int j = 0; j < columnCount(); j++)
                {
                    //TableUnselectAll();

                    //Если текущая ячейка пустая(без буквы)
                    if(usTable[i][j] == CELL_BACK)
                    {
                        //setCurrentCell(i, j);
                        //Выделение слова
                        makeSelectWordCrossword(i, j, 0);
                        strListSelectWord.clear();
                        strListSelectWord.append(QChar(numberRowFirstLetterWord()));
                        strListSelectWord.append(QChar(numberColumnFirstLetterWord()));
                        strListSelectWord.append(cDirectionWord);
                        strListSelectWord.append(QChar(ushort(SelectedWordToCheck.length())));

                        if (SelectedWordToCheck.count() > 1 /*|| SelectedWordToEdit.count() > 1*/)
                        {
                            slSelectWord.append(strListSelectWord);
                            vBeginCoordWordRow.append(ushort(numberRowFirstLetterWord()));
                            vBeginCoordWordColumn.append(ushort(numberColumnFirstLetterWord()));
                            vDirectionWord.append(cDirectionWord);
                            vLengthWord.append(ushort(SelectedWordToCheck.length()));
                            slRightSelectedWord.append("no");
                        }

                        if (cDirectionWord == 'h')
                            for(int k = 0; k < SelectedWordToCheck.length(); k++)
                            {
                            usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                            setNumberColumnFirstLetterWord(numberColumnFirstLetterWord() + 1);
                        }
                        else
                            for(int k = 0; k < SelectedWordToCheck.length(); k++)
                            {
                            usTable[numberRowFirstLetterWord()][numberColumnFirstLetterWord()] = CELL_BACK_FILL;
                            setNumberRowFirstLetterWord(numberRowFirstLetterWord() + 1);
                        }
                    }
                }
            }

            cDirectionWord = 'v';
            usTableUnselectAll();
        }
    }

    usTableUnselectAll();
    searchWordsToClear();

    return true;
}

//Поиск пересекающихся слов
QStringList TCrosswordGrid::searchWordsToClear()
{
    //QStringList ListCrossingWord;
    QString strListCrossingWord;
    QString slL2;

    slCrossingWord.clear();
    sCells.clear();

    for(int i = 0; i < vBeginCoordWordRow.count(); i++)
    {
        int j = 0;

        if(vDirectionWord.at(i) == 'h')
            for(j = int(vBeginCoordWordColumn.at(i)); j < (int(vBeginCoordWordColumn.at(i)) + int(vLengthWord.at(i))); j++)
                usTable[int(vBeginCoordWordRow.at(i))][j] = 2;

        if(vDirectionWord.at(i) == 'v')
            for(j = int(vBeginCoordWordRow.at(i)); j < (int(vLengthWord.at(i)) + int(vBeginCoordWordRow.at(i))); j++)
                usTable[j][int(vBeginCoordWordColumn.at(i))] = 2;

        strListCrossingWord.clear();

        for(j = 0; j < vBeginCoordWordRow.count(); j++)
        {
            slL2 = vLengthWord.at(j);

            if(vDirectionWord.at(j) == 'h' && i != j)
                for(int k = int(vBeginCoordWordColumn.at(j)); k < (int(vBeginCoordWordColumn.at(j)) + int(slL2.at(0).unicode())); k++)
                    if(usTable[int(vBeginCoordWordRow.at(j))][k] == 2)
                    {
                QString s1;
                s1.clear();

                s1.append(QChar(vBeginCoordWordRow.at(j)));
                s1.append(QChar(ushort(k)));
                strListCrossingWord.append(QChar(ushort(j)));

                if(sCells.indexOf(s1) == -1)
                {
                    sCells.append(QChar(vBeginCoordWordRow.at(j)));
                    sCells.append(QChar(ushort(k)));
                }
                break;
            }

            if(vDirectionWord.at(j) == 'v' && i != j)
                for(int k = int(vBeginCoordWordRow.at(j)); k < ((int(vBeginCoordWordRow.at(j)) + int(slL2.at(0).unicode()))); k++)
                    if(usTable[k][int(vBeginCoordWordColumn.at(j))] == 2)
                    {
                QString s1;
                s1.clear();

                s1.append(QChar(ushort(k)));
                s1.append(QChar(vBeginCoordWordColumn.at(j)));
                strListCrossingWord.append(QChar(ushort(j)));

                if(sCells.indexOf(s1) == -1)
                {
                    sCells.append(QChar(ushort(k)));
                    sCells.append(QChar(vBeginCoordWordColumn.at(j)));
                }
                break;
            }
        }

        if(vDirectionWord.at(i) == 'h')
            for(int k = int(vBeginCoordWordColumn.at(i)); k < (int(vLengthWord.at(i)) + int(vBeginCoordWordColumn.at(i))); k++)
                usTable[int(vBeginCoordWordRow.at(i))][k] = 0;

        if(vDirectionWord.at(i) == 'v')
            for(int k = int(vBeginCoordWordRow.at(i)); k < (int(vLengthWord.at(i)) + int(vBeginCoordWordRow.at(i))); k++)
                usTable[k][int(vBeginCoordWordColumn.at(i))] = 0;

        slCrossingWord << strListCrossingWord;
    }

    usTableUnselectAll();

    return slCrossingWord;
}

//Изменение текущего шрифта сетки кроссворда
void TCrosswordGrid::slotChangeFont()
{
    bool Accept;
    QFont fGridFont = QFontDialog::getFont(&Accept, font());

    if(Accept) setFont(fGridFont);

    return;
}

bool TCrosswordGrid::checkWordsLong18()
{
    for (int i = 0; i < vLengthWord.count(); i++)
        if (vLengthWord.at(i) > 18) return true;

    return false;
}

//Удаление слов, пересекающихся с выделенным
QString TCrosswordGrid::searchCrossingLetter(int CurrentNumber)
{
    QString strListCrossingWord;
    QString s2;

    int iCurrent;

    strListCrossingWord.clear();
    strListCrossingWord.append(slCrossingWord.at(CurrentNumber));
    slRightSelectedWord.replace(CurrentNumber, "no");

    for(int i = 0; i < strListCrossingWord.length(); i++)
    {
        slRightSelectedWord.replace(int(strListCrossingWord.at(i).unicode()), "no");

        iCurrent = int(strListCrossingWord.at(i).unicode());


        if(vDirectionWord.at(iCurrent) == 'h')
            for(int j = int(vBeginCoordWordColumn.at(iCurrent)); j < (int(vLengthWord.at(iCurrent)) + int(vBeginCoordWordColumn.at(iCurrent))); j++)
            {
            s2.clear();
            s2.append(QChar(vBeginCoordWordRow.at(iCurrent)));
            s2.append(QChar(ushort(j)));

            if(sCells.indexOf(s2) != -1 && vBeginCoordWordRow.at(iCurrent) + 1 < rowCount() && vBeginCoordWordRow.at(iCurrent) - 1 >= 0)
            {
                if(item(vBeginCoordWordRow.at(iCurrent) + 1, j)->text() != "" || item(vBeginCoordWordRow.at(iCurrent) - 1, j)->text() != "")
                    s2.clear();
                else
                    item(int(vBeginCoordWordRow.at(iCurrent)), j)->setText("");
            }
            else
                item(int(vBeginCoordWordRow.at(iCurrent)), j)->setText("");
        }

        if(vDirectionWord.at(iCurrent) == 'v')
            for(int j = int(vBeginCoordWordRow.at(iCurrent)); j < (int(vLengthWord.at(iCurrent)) + int(vBeginCoordWordRow.at(iCurrent))); j++)
            {
            s2.clear();
            s2.append(QChar(ushort(j)));
            s2.append(QChar(vBeginCoordWordColumn.at(iCurrent)));

            if(sCells.indexOf(s2) != -1 && vBeginCoordWordColumn.at(iCurrent) + 1 < columnCount() && vBeginCoordWordColumn.at(iCurrent) - 1 >= 0)
            {
                if(item(j, vBeginCoordWordColumn.at(iCurrent) + 1)->text() != "" || item(j, vBeginCoordWordColumn.at(iCurrent) - 1)->text() != "")
                    s2.clear();
                else
                    item(j, int(vBeginCoordWordColumn.at(iCurrent)))->setText("");
            }
            else
                item(j, int(vBeginCoordWordColumn.at(iCurrent)))->setText("");
        }
    }

    return "";
}

//Удаление выделенного слова из сетки
void TCrosswordGrid::deleteSelectedWords(int CurrentNumber)
{
    QString s2;

    //Слова, предназначенные для выбора их заново

    if(vDirectionWord.at(CurrentNumber) == 'h')
        for(int j = int(vBeginCoordWordColumn.at(CurrentNumber)); j < (int(vLengthWord.at(CurrentNumber)) + int(vBeginCoordWordColumn.at(CurrentNumber))); j++)
        {
        s2.clear();
        s2.append(QChar(vBeginCoordWordRow.at(CurrentNumber)));
        s2.append(QChar(ushort(j)));

        if(sCells.indexOf(s2) != -1 && vBeginCoordWordRow.at(CurrentNumber) + 1 < rowCount() && vBeginCoordWordRow.at(CurrentNumber) - 1 >= 0)
        {
            if(item(vBeginCoordWordRow.at(CurrentNumber) + 1, j)->text() != "" || item(vBeginCoordWordRow.at(CurrentNumber) - 1, j)->text() != "")
                s2.clear();
            else
                item(int(vBeginCoordWordRow.at(CurrentNumber)), j)->setText("");
        }
        else
            item(int(vBeginCoordWordRow.at(CurrentNumber)), j)->setText("");
    }

    if(vDirectionWord.at(CurrentNumber) == 'v')
        for(int j = int(vBeginCoordWordRow.at(CurrentNumber)); j < (int(vLengthWord.at(CurrentNumber)) + int(vBeginCoordWordRow.at(CurrentNumber))); j++)
        {
        s2.clear();
        s2.append(QChar(ushort(j)));
        s2.append(QChar(vBeginCoordWordColumn.at(CurrentNumber)));

        if(sCells.indexOf(s2) != -1 && vBeginCoordWordColumn.at(CurrentNumber) + 1 < columnCount() && vBeginCoordWordColumn.at(CurrentNumber) - 1 >= 0)
        {
            if(item(j, vBeginCoordWordColumn.at(CurrentNumber) + 1)->text() != "" || item(j, vBeginCoordWordColumn.at(CurrentNumber) - 1)->text() != "")
                s2.clear();
            else
                item(j, int(vBeginCoordWordColumn.at(CurrentNumber)))->setText("");
        }
        else
            item(j, int(vBeginCoordWordColumn.at(CurrentNumber)))->setText("");
    }

    sDeleteWords.clear();
}

//Выбор слов
bool TCrosswordGrid::makeCompile()
{
    setSelectWord(true);
    cDirectionWord = 'v';
    QSqlDatabase db;
    db = QSqlDatabase::addDatabase("QSQLITE");

    if(FileOptions->bCurrentDictionary == LANGUAGE_RUSSIAN)
        db.setDatabaseName(PATH_SHARE_APP  "/dictionaries/dictionary_rus.xwcb");

    if(FileOptions->bCurrentDictionary == LANGUAGE_ENGLISH)
        db.setDatabaseName(PATH_SHARE_APP  "/dictionaries/dictionary_eng.xwcb");

    db.setUserName("user");
    db.setHostName("host");
    db.setPassword("password");

    if(!db.open())
    {
        QMessageBox::critical(0, tr("Detection error"), tr(
                "Error opennig dictionary base!\nRussian or English dictionarys "
                "can download on official site of the program."));
        return false;
    }

    //int iCountSelectWord = 0;
    // ListWordToSelect = true;
    QString strQuery;
    QString CurrentTable;
    QStringList SelectFromDictionary[2][19],
    CopyOneWord[2];
    int iNumber;
    int iCountWord;

    //Очистка сетки кроссворда от букв
    for(int i = 0; i < rowCount(); i++)
        for(int j = 0; j < columnCount(); j++)
            if(item(i, j)->text() != "") item(i, j)->setText("");

    //Если выделенные ячейки в сетке пустые
    if (vBeginCoordWordRow.count() == 0)
    {
        if(!isInsertCoordinates()) setInsertCoordinates(InsertCoordinatesWords(ACROSS_DISABLE));
    }

    //Ограничение слова в сетке до 18 букв
    if(checkWordsLong18() == true)
    {
        clearVariable();
        db.close();
        usTableUnselectAll();

        QMessageBox::warning(0, tr("Detection warning"), tr(
                "Can't make a crossword puzzle!\nIn grid is a word, more than 18 letters."));

        return false;
    }

    //Выбор слов и толкований из базы в списки строк

    QSqlQuery QueryCondition;
    QSqlRecord RecordCondition;

    for(int i = 2; i < 19; i++)
    {
        CurrentTable.setNum(i);

        strQuery = "SELECT Word, Interpretation FROM w" + CurrentTable + ";";
        QueryCondition.exec(strQuery);
        RecordCondition = QueryCondition.record();

        while(QueryCondition.next())
        {
            sWord = QueryCondition.value(RecordCondition.indexOf("Word")).toString();
            sQuestion = QueryCondition.value(RecordCondition.indexOf("Interpretation")).toString();
            SelectFromDictionary[0][i] << sWord;
            SelectFromDictionary[1][i] << sQuestion;
        }

        QueryCondition.clear();
        RecordCondition.clear();
    }

    db.close();
    //------------------------------------------------------------------------------------------------------------------------------------

    sWord.clear();
    sQuestion.clear();
    QProgressDialog *SelectWordDialog = new QProgressDialog(tr("Compiling of crossword..."), tr("Cancel"), 0, vBeginCoordWordRow.count());
    SelectWordDialog->setMinimumDuration(0);
    SelectWordDialog->setWindowTitle(tr("Please wait"));

    lRun:

    bool NextRun = false;

    //Процесс выбора слов
    for(int i = 0; i < vBeginCoordWordRow.count(); i++)
    {
        //Если текущее слово не было выбрано
        if(slRightSelectedWord.at(i) == "no")
        {
            setNumberCurrentQuestion(i);
            bool AllSpace = false;
            bool AllunSpace = false;
            QString slV;

            slV = vDirectionWord.at(i);
            tableUnselectAll();
            setNumberRowFirstLetterWord(int(vBeginCoordWordRow.at(i)));
            setNumberColumnFirstLetterWord(int(vBeginCoordWordColumn.at(i)));
            cDirectionWord = slV.at(0);
            //Выделение слова
            makeSelectWordCrossword(numberRowFirstLetterWord(), numberColumnFirstLetterWord(), 1);

            if(SelectedWordToCheck.length() != 0)
            {
                for(int k = 0; k < SelectedWordToCheck.length(); k++)
                    if(SelectedWordToCheck.at(k) == ' ') AllSpace = true;
                else
                {
                    AllSpace = false;
                    break;
                }
            }

            if(SelectedWordToCheck.length() != 0)
            {
                for(int k = 0; k < SelectedWordToCheck.length(); k++)
                    if(SelectedWordToCheck.at(k) != ' ') AllunSpace = true;
                else
                {
                    AllunSpace = false;
                    break;
                }
            }

            //Если выделенные ячейки слова все пустые
            if(AllSpace == true)
            {

                iCountWord = SelectFromDictionary[0][SelectedWordToCheck.length()].count();

                srand((unsigned) QTime::currentTime().msec());

                if(iCountWord == 0)
                {
                    cDirectionWord = 'v';

                    for(int i = 0; i < slRightSelectedWord.count(); i++)
                        slRightSelectedWord.replace(i, "no");

                    delete SelectWordDialog;

                    QMessageBox::warning(0, tr("Detection warning"), tr(
                            "Can't make a crossword puzzle!\n"
                            "The loss all of words or can't find dictionary.\n"
                            "The dictionarys download can with official site."
                            " Look is information in the section menu about of the program."));
                    return false;
                }

                //Получение произвольного номера слова из списка
                iNumber = rand() % iCountWord;
                //strNumber.setNum(iNumber);

                sWord.clear();
                sQuestion.clear();

                sWord = SelectFromDictionary[0][SelectedWordToCheck.length()].at(iNumber);
                sQuestion = SelectFromDictionary[1][SelectedWordToCheck.length()].at(iNumber);

                if(sWord != "")
                {
                    if(slWordT.count() > i)
                    {
                        setEditDataCrossword(DATA_EDIT);
                        setGridEditHand(HAND_ENABLE);
                    }
                    else
                    {
                        setEditDataCrossword(DATA_ADD);
                        setGridEditHand(HAND_DISABLE);
                    }

                    dataCrosswordEdit();
                }
                //SelectedWords[0] << DictionaryWord;
                //SelectedWords[1] << CrosswordText->Question;

                //Удаление выбранного слова из списка
                SelectFromDictionary[0][SelectedWordToCheck.length()].removeAt(iNumber);
                SelectFromDictionary[1][SelectedWordToCheck.length()].removeAt(iNumber);

                sWord.clear();
                sQuestion.clear();

                slRightSelectedWord.replace(i, "yes");

            }
            else
                //Если выделенные ячейки слова не все пустые
                if(AllSpace == false)
                {
                //Если в выделенном слове нет пробелов
                if(SelectedWordToCheck.indexOf(' ') != -1)
                {
                    bool AllCoincide = false;
                    iCountWord = SelectFromDictionary[0][SelectedWordToCheck.length()].count();

                    CopyOneWord[0].clear();
                    CopyOneWord[1].clear();

                    CopyOneWord[0].append(SelectFromDictionary[0][SelectedWordToCheck.length()]);
                    CopyOneWord[1].append(SelectFromDictionary[1][SelectedWordToCheck.length()]);

                    //Перебор всех слов, с нужным количеством букв
                    while(CopyOneWord[0].count() > 0)
                    {
                        iCountWord = CopyOneWord[0].count();

                        srand((unsigned) QTime::currentTime().msec());

                        iNumber = rand() % iCountWord;

                        sWord = CopyOneWord[0].at(iNumber);
                        sQuestion = CopyOneWord[1].at(iNumber);

                        for(int k = 0; k < SelectedWordToCheck.length(); k++)
                            if(SelectedWordToCheck.at(k) != ' '  && SelectedWordToCheck.at(k) == sWord.at(k)) AllCoincide = true;
                        else
                        {
                            if(SelectedWordToCheck.at(k) != ' ')
                            {
                                AllCoincide = false;
                                break;
                            }
                        }

                        //Если буквы в слове все совпадают
                        if(AllCoincide == true)
                        {
                            if(sWord != "")
                            {
                                if(slWordT.count() > i)
                                {
                                    setEditDataCrossword(DATA_EDIT);
                                    setGridEditHand(HAND_ENABLE);
                                }
                                else
                                {
                                    setEditDataCrossword(DATA_ADD);
                                    setGridEditHand(HAND_DISABLE);
                                }

                                dataCrosswordEdit();
                            };

                            //SelectedWords[0] << DictionaryWord;
                            //SelectedWords[1] << DictionaryInterpretation;


                            SelectFromDictionary[0][SelectedWordToCheck.length()].removeAt(iNumber);
                            SelectFromDictionary[1][SelectedWordToCheck.length()].removeAt(iNumber);

                            sWord.clear();
                            sQuestion.clear();

                            slRightSelectedWord.replace(i, "yes");

                            break;
                        }
                        else
                        {
                            CopyOneWord[0].removeAt(iNumber);
                            CopyOneWord[1].removeAt(iNumber);
                        }
                    }

                    //Если слова в списке закончились
                    if(CopyOneWord[0].count() == 0)
                    {
                        sDeleteWords.clear();
                        //SearchWordsToClear(i, ListSelectWord);

                        //Удаление из сетки слов, пересекающихся с текущим
                        searchCrossingLetter(i);

                        //Удаление из сетки текушего слова
                        deleteSelectedWords(i);
                        NextRun = true;
                        break;
                    }
                }
            }

            if (i > SelectWordDialog->value()) SelectWordDialog->setValue(i);
            QApplication::processEvents();

            if(SelectWordDialog->wasCanceled())
            {
                return false;
            }
        }
    }

    //Запуск цикла выбора слов заново(продолжение с начала)
    if(NextRun == true) goto lRun;

    //SelectWordDialog->setValue(i);
    delete SelectWordDialog;

    db.close();
    cDirectionWord = 'v';
    setSelectWord(false);

    for(int i = 0; i < slRightSelectedWord.count(); i++)
        slRightSelectedWord.replace(i, "no");

    for(int i = 0; i < 2; i++)
        for(int j = 0; j < 19; j++)
            SelectFromDictionary[i][j].clear();

    return true;
}

QString TCrosswordGrid::wordToUnicode(QString sWord)
{
    QString sTemp;

    for (int i = 0; i < sWord.length(); i++)
        sTemp.append("\\u" + QString::number(sWord.at(i).unicode()) + "?");

    return sTemp;
}

void TCrosswordGrid::saveToRTF(QString sExportName, QString strRTF, QImage imgExport)
{
    QByteArray baImage;
    QBuffer buffer(&baImage);
    buffer.open(QIODevice::WriteOnly);
    imgExport.save(&buffer, "BMP");
    baImage = baImage.toHex();
    QFile ifile(sExportName);
    QTextStream istream(&ifile);
    istream.setCodec("UTF-8");
    ifile.open(QIODevice::WriteOnly);
    istream << "{\\rtf1\n{\\pict\\wmetafile8\\picw" + QString::number(columnCount() * rowHeight(0) * 27) + "\\pich"
            + QString::number(rowCount() * rowHeight(0) * 27) + "\n";
    
    int j = 0;

    for (int i = 0; i < baImage.size(); i++)
    {
        if (j == 128)
        {
            istream << "\n";
            j = 0;
        }

        istream << baImage.at(i);
        j++;
    }
    istream << "}\\par\\par\n";
    istream << strRTF;
    istream << "}";
    ifile.close();
}

QString TCrosswordGrid::makeRTFText(QString ListWordH, QString ListWordV, QStringList ListOutV, QStringList ListOutH)
{
    QString strRTF;
    QString strText1;
    QString strText2;
    QString strText3;

    for(int i = 0; i < ListOutH.count(); i++)
    {
        strText1.append(wordToUnicode(ListOutH.at(i)) + "\\par\n");
    }

    strText2.clear();

    for(int i = 0; i < ListOutV.count(); i++)
    {
        strText2.append(wordToUnicode(ListOutV.at(i)) + "\\par\n");
    }

    strText3.clear();
    strText3 = "{\\b " + wordToUnicode(tr("Horizontal")) + "}:\\par\n" + wordToUnicode(ListWordH) + "\\par\n" + "{\\b " + wordToUnicode(tr("Vertical"))
               +"}:\\par\n" + wordToUnicode(ListWordV) + "\\par\n";

    strRTF.append("{\\b " + wordToUnicode(tr("Horizontal")) + "}:\\par\n"
                   + strText1 + "\\par\n"
                   "{\\b " + wordToUnicode(tr("Vertical")) + "}:\\par\n"
                   + strText2 + "\\par\n"
                   "{\\b\\i " + wordToUnicode(tr("Answers")) + "}:\\par\n"
                   + strText3 + "\\par\n");
    return strRTF;
}
