/* ======================================================================
**  TCrosswordGrid.h - Заголовочный файл модуля сетки кроссворда.
**  Является частью программы Клёст-кроссворд, распространяемой под
**  Стандартной общественной лицензией, версии 2 (GNU GPL v2).
** ======================================================================
**  Copyleft 2009 - 2012 by Lab Free Technologies <admin@LabFreeTech.org>
** ==============================================================================
**  Copyleft 2009 - 2012 Лаборатория свободных технологий <admin@LabFreeTech.org>
** ==============================================================================
*/

#ifndef TCROSSWORDGRID_H
#define TCROSSWORDGRID_H
#include <QtCore/QCoreApplication>
#include <QtCore/QDataStream>
#include <QtCore/QFile>
#include <QtCore/QChar>
#include <QtCore/QDir>
#include <QtCore/QTextStream>
#include <QtCore/QUrl>
#include <QtCore/QTime>
#include <QtCore/QBuffer>
#include <QtCore/QRect>
#include <QtCore/QTemporaryFile>
#include <QtCore/QByteArray>
#include <QtGui/QApplication>
#include <QtGui/QTableWidget>
#include <QtGui/QMouseEvent>
#include <QtGui/QPainter>
#include <QtGui/QBrush>
#include <QtGui/QTextDocument>
#include <QtGui/QPrinter>
#include <QtGui/QFileDialog>
#include <QtGui/QPrintDialog>
#include <QtGui/QFontDialog>
#include <QtGui/QMessageBox>
#include <QtGui/QProgressDialog>
#include <QtSql/QSqlDatabase>
#include <QtSql/QSqlQueryModel>
#include <QtSql/QSqlQuery>
#include <QtSql/QSqlRecord>
#include "const.h"
#include "struct.h"
#include "error.h"

class TCrosswordGrid : public QTableWidget {
    Q_OBJECT
    Q_PROPERTY(bool saved READ isSaved() WRITE setSaved())
    Q_PROPERTY(bool acrossLite READ isAcrossLite() WRITE setAcrossLite())
    Q_PROPERTY(bool selectWord READ isSelectWord() WRITE setSelectWord())
    Q_PROPERTY(bool insertCoordinates READ isInsertCoordinates() WRITE setInsertCoordinates())
    Q_PROPERTY(int numberRowFirstLetterWord READ numberRowFirstLetterWord() WRITE setNumberRowFirstLetterWord())
    Q_PROPERTY(int numberColumnFirstLetterWord READ numberColumnFirstLetterWord() WRITE setNumberColumnFirstLetterWord())
    Q_PROPERTY(int countLetterInWord READ countLetterInWord() WRITE setCountLetterInWord())
    Q_PROPERTY(int numberCurrentQuestion READ numberCurrentQuestion() WRITE setNumberCurrentQuestion())
    Q_PROPERTY(bool editDataCrossword READ isEditDataCrossword() WRITE setEditDataCrossword())
    Q_PROPERTY(bool gridEditHand READ isGridEditHand() WRITE setGridEditHand())

public:
    QString sFileNameCrossword;//Название кроссворда
    QString sQuestion;//Вопрос
    QString sWord;//Слово
    QString SelectedWordToEdit;//Выделенное слово для редактирования
    QString SelectedWordToCheck;//Выделенное слово для проверки
    QString sAcrossTitle;
    QString sAcrossAuthor;
    QString sAcrossCopyright;
    QString sCells;//Координаты пересечений слов;
    QString sDeleteWords;//Координаты пересечений, разделённых пробелами
    QStringList slWordCheck;//Список слов для проверки
    QStringList slSelectWord;//Список параметров выбранных слов
    QVector<ushort> vNumbersDeletedWord;//Номера слов, начальные буквы которых находятся на одной ячейке
    QVector<ushort> vBeginCoordWordRow;//Список начальных координат слова (строка)
    QVector<ushort> vBeginCoordWordColumn;//Список начальных координат слова (столбец)
    QVector<QChar> vDirectionWord;//Список напрвлений слов
    QVector<ushort> vLengthWord;//Список длин слов
    QStringList slWordT;//Список слов
    QVector<ushort> slQuestionL;//Список длин толкований
    QStringList slQuestionT;//Список толкований
    QStringList slRightSelectedWord;//Список состояний слов. "YES" - слово выбрано правильно. "NO" - слово не выбрано или требуется его заменить
    QStringList slCrossingWord;//Номера пересекающихся слов

    QChar cDirectionWord;//Направление слова
    TFileOptions *FileOptions;

    explicit TCrosswordGrid(QTableWidget *pwgt = 0);
    ~TCrosswordGrid();

    bool isSaved() const;//Проверить состояние сохранения
    bool isAcrossLite() const;//Получить состояние открытия кроссворда в формате AcrossLite
    bool isSelectWord() const;//Получить состояние индикатора выбора слов
    bool isInsertCoordinates() const;//Получить состояние вставки координат слов
    int numberRowFirstLetterWord() const;//Получить номер строки первой буквы слова
    int numberColumnFirstLetterWord() const;//Получить номер столбца первой буквы слова
    int countLetterInWord() const;//Получить количество букв в слове
    int numberCurrentQuestion() const;//Получить номер текущего вопроса
    bool isEditDataCrossword() const;//Получить состояние индикатора редактирования данных кроссворда
    bool isGridEditHand() const;//Получить состояние ручного редактирования сетки
    void saveToFile(QString sFile);//При сохранении кроссворда
    void saveToFileTemporary();//При сохранении неполностью отгаданного кроссворда
    void crosswordToData(uchar State);//Передача данных из кроссворда в переменную
    void clearVariable();//Очистка переменных
    bool openFile(QString sFile);//При открытии кроссворда
    void slotExport(bool State, int iNum, int iTypeExport);//При экспорте кроссворда
    int loadTemporary();//При загрузке сохранения
    void dataCrosswordEdit();//Редактирование данных кроссворда
    void makeSelectWordCrossword(int Row, int Column, int State);//Для выделения слова в сетке кроссворда или в массиве
    bool InsertCoordinatesWords(bool bAcross);//Вставка координат слов
    void openAcrossLiteTXT(QString FileName);//Открытие текстового файла AcrossLite
    void openOpenKlestFile(QString sImportName);//Импорт файла формата OpenKlest
    void dataToCrossword();//Передача данных из переменной в кроссворд
    int messageChangeGrid();//Сообщение при изменении табцицы кроссворда
    void print();//Печать кроссворда
    void create_usTable();//Создать массив сетки кроссворда
    void delete_usTable();//Очистить массив сетки кроссворда
    void tableUnselectAll();//При снятии выделения со всёй таблицы
    bool makeCompile();//Составление кроссворда

private:
    bool b_Saved;//Состояние сохранения кроссворда
    bool b_AcrossLite;//Состояние открытого кроссворда в текстовом формате AcrossLite
    bool b_SelectWord;//Индикатор выбора слов
    bool b_InsertCoordinates;//Состояние вставки координат слов
    int i_NumberRowFirstLetterWord;//Номер строки первой буквы слова
    int i_NumberColumnFirstLetterWord;//Номер столбца первой буквы слова
    int i_CountLetterInWord;//Количество букв в слове
    int i_NumberCurrentQuestion;//Номер текущего вопроса
    bool b_EditDataCrossword;//Индикатор редактирования данных кроссворда
    bool b_GridEditHand;//Ручное редактирование сетки
    ushort **usTable;//Массив сетки кроссворда

    void saveToHTMLFile(QString sExportName, QString strHTML, QString strNumberImage);//Экспорт в HTML
    void saveToPortable(QString sExportName, QString sFormat, QImage imgExport, QString sHtml);//Выполнить экспорт в файл PDF или PostScript
    void saveToAcrossLiteTXT(QString sExportName);//Экспорт в текстовый AcrossLite
    void saveToOpenKlest(QString FileName);//Экспорт кроссворда в формат OpenKlest
    QString makeOneHTML(QString ListWordH, QString ListWordV, QStringList ListOutV, QStringList ListOutH, QString sImage);//Сделать одну html-страницу
    void makeTableToMassive();//Заполнение таблицы массива
    void usTableUnselectAll();//Снятие выделения со всего масиива кроссворда
    QImage makeCrosswordImage();//Подготовить изображение
    TCrosswordText makeCrosswordText();
    void makeCheck4Cell();//Проверка сетки на возможность автоматического составления кроссворда
    void firstCellWord(int Row, int Column);//Находятся первые ячейки выбранного слова
    QStringList searchWordsToClear();//Происходит поиск пересекающихся слов
    bool checkWordsLong18();//Проверка длины слов на превышение 18-ти букв
    QString searchCrossingLetter(int CurrentNumber);//Удаление слов, пересекающихся с выделенным
    void deleteSelectedWords(int CurrentNumber);//Удаление выделенного слова из сетки
    QString wordToUnicode(QString sWord);
    void saveToRTF(QString sExportName, QString strRTF, QImage imgExport);
    QString makeRTFText(QString ListWordH, QString ListWordV, QStringList ListOutV, QStringList ListOutH);

protected:
    virtual void mouseReleaseEvent(QMouseEvent* pe);
    //virtual void mouseMoveEvent(QMouseEvent* pe);

signals:
    void ChangeLine();//Сигнал, отправляемый при щелчке правой кнопкой мыши
    void sigCallFunction(uchar ucType);

public slots:
    void setSaved(bool bSaved);//Задать состояние сохранения
    void setAcrossLite(bool bStateAcrossLite);//Задать состояние открытия кроссворда в формате AcrossLite
    void setSelectWord(bool bSelectWord);//Задать состояние индикатора выбора слов
    void setInsertCoordinates(bool bInsertCoordinates);//Задать состояние вставки координат слов
    void setNumberRowFirstLetterWord(int iNumberRowFirstLetterWord);//Задать номер строки первой буквы слова
    void setNumberColumnFirstLetterWord(int iNumberColumnFirstLetterWord);//Задать номер столбца первой буквы слова
    void setCountLetterInWord(int iCountLetterInWord);//Задать количество букв в слове
    void setNumberCurrentQuestion(int iNumberCurrentQuestion);//Задать номер текущего вопроса
    void setEditDataCrossword(bool bEditDataCrossword);//Задать состояние индикатора редактирования данных кроссворда
    void setGridEditHand(bool bGridEditHand);//Задать состояние ручного редактирования сетки
    void slotRotateRight();//Поворот кроссворда направо
    void slotRotateLeft();//Поворот кроссворда налево
    void slotAutoGrid();//При автоматическом создании сетки
    void slotInversion();//Инвертировать сетку кроссворда
    void slotMirrorVertically();//Отразить кроссворд вертикально
    void slotMirrorHorisontally();//Отразить кроссворд горизонтально
    void slotChangeFont();//При изменении шрифта
    void slotSaveAs();//При сохранении кроссворда под другим именем

};
#endif // TCROSSWORDGRID_H
