/* ======================================================================
**  TKlestCrossword.cpp - Исходный файл главного модуля кроссворда.
**  Является частью программы Клёст-кроссворд, распространяемой под
**  Стандартной общественной лицензией, версии 2 (GNU GPL v2).
** ======================================================================
**  Copyleft 2009 - 2012 by Lab Free Technologies <admin@LabFreeTech.org>
** ==============================================================================
**  Copyleft 2009 - 2012 Лаборатория свободных технологий <admin@LabFreeTech.org>
** ==============================================================================
*/

#include "TKlestCrossword.h"

TKlestCrossword::TKlestCrossword(QMainWindow *pwgt /* = 0*/): QMainWindow(pwgt),
    FormStatistics(new TStatistics),
    FormHelpViewer(new THelpViewer),
    CrosswordGrid(new TCrosswordGrid),
    ui(new Ui::TKlestCrossword),
    wvUpdate(new QWebView)
{
    ui->setupUi(this);
    bEditTable = false;
    bEditTableSymmetrically = false;
    bEditTableNotSymmetrically = false;
    bEditWords = true;
    CrosswordGrid->cDirectionWord = 'v';
    ui->actionCircumcision->setVisible(false);

    QPalette pal1;
    QBrush br1(Qt::SolidPattern);
    QColor col1(227, 226, 228);
    col1.setAlpha(255);
    br1.setColor(col1);
    pal1.setBrush(QPalette::Base, br1);
    CrosswordGrid->setPalette(pal1);
    CrosswordGrid->resize(1024,768);
    CrosswordGrid->horizontalHeader()->setVisible(false);
    CrosswordGrid->verticalHeader()->setVisible(false);
    CrosswordGrid->setEditTriggers(QTableWidget::NoEditTriggers);
    CrosswordGrid->setSelectionMode(QTableWidget::SingleSelection);

    connect(ui->actionExit, SIGNAL(triggered()), this, SLOT(slotExitProgram()));
    connect(ui->actionCreate, SIGNAL(triggered()), this, SLOT(slotCreate()));
    connect(ui->actionOpen, SIGNAL(triggered()), this, SLOT(slotOpen()));
    connect(ui->actionClose, SIGNAL(triggered()), this, SLOT(slotClose()));
    connect(ui->actionSave, SIGNAL(triggered()), this, SLOT(slotSave()));
    connect(ui->actionSave_as, SIGNAL(triggered()), this, SLOT(slotSave()));
    connect(ui->actionUser_manual, SIGNAL(triggered()), this, SLOT(slotUserManual()));
    connect(ui->actionAbout_program, SIGNAL(triggered()), this, SLOT(slotAboutProgram()));
    connect(ui->actionGridOptions, SIGNAL(triggered()), this, SLOT(slotChange()));
    connect(CrosswordGrid, SIGNAL(cellClicked(int,int)), SLOT(slotCellClicked()));
    connect(CrosswordGrid, SIGNAL(cellChanged(int,int)), SLOT(slotCellChanged()));
    connect(ui->actionEditing, SIGNAL(triggered()), this, SLOT(slotRegimeEditing()));
    connect(ui->actionGuessing, SIGNAL(triggered()), this, SLOT(slotRegimeViewing()));
    connect(ui->actionFont, SIGNAL(triggered()), CrosswordGrid, SLOT(slotChangeFont()));
    connect(ui->actionSymmetrically, SIGNAL(triggered()), this, SLOT(slotCellColorSymmetrically()));
    connect(ui->actionNotSymmetrically, SIGNAL(triggered()), this, SLOT(slotCellColorNotSymmetrically()));
    connect(ui->actionWords, SIGNAL(triggered()), this, SLOT(slotWords()));
    connect(ui->actionTemplatesOpen, SIGNAL(triggered()), this, SLOT(slotOpenTemplate()));
    connect(ui->actionCheck, SIGNAL(triggered()), this, SLOT(slotCheckRightWords()));
    connect(ui->actionGridCompiling, SIGNAL(triggered()), CrosswordGrid, SLOT(slotAutoGrid()));
    connect(ui->actionCrosswordCompiling, SIGNAL(triggered()), this, SLOT(slotCompile()));
    connect(ui->actionDictionariesOpen, SIGNAL(triggered()), this, SLOT(slotDictionaryOpen()));
    connect(ui->lwQuestions, SIGNAL(clicked(QModelIndex)), this, SLOT(questionToSelect()));
    connect(ui->lwQuestions, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(slotSemiautomaticInput()));
    connect(ui->pbDataOk, SIGNAL(clicked()), this, SLOT(inputDataToCrossword()));
    connect(ui->actionExport, SIGNAL(triggered()), this, SLOT(slotMakeExport()));
    connect(CrosswordGrid, SIGNAL(ChangeLine()), SLOT(slotChangeLine()));
    connect(CrosswordGrid, SIGNAL(doubleClicked(QModelIndex)), SLOT(slotSemiautomaticInput()));
    connect(ui->actionImport, SIGNAL(triggered()), this, SLOT(slotImportCrossword()));
    connect(ui->leWord, SIGNAL(returnPressed()), this, SLOT(inputDataToCrossword()));
    connect(ui->actionDonate, SIGNAL(triggered()), this, SLOT(openFileDonate()));
    connect(ui->actionGetSource, SIGNAL(triggered()), this, SLOT(slotGetSource()));
    connect(ui->actionStatistics, SIGNAL(triggered()), this, SLOT(slotShowStatistics()));
    connect(ui->actionPrint, SIGNAL(triggered()), this, SLOT(slotPrint()));
    connect(ui->actionGetSupport, SIGNAL(triggered()), this, SLOT(openFileGetSupport()));
    connect(ui->actionClearGrid, SIGNAL(triggered()), this, SLOT(slotClearGrid()));
    connect(ui->actionClearWords, SIGNAL(triggered()), this, SLOT(slotClearWords()));
    connect(ui->actionClearAll, SIGNAL(triggered()), this, SLOT(slotClearAll()));
    connect(CrosswordGrid, SIGNAL(cellEntered(int,int)), SLOT(slotMouseMove()));
    connect(ui->actionOptions, SIGNAL(triggered()), this, SLOT(slotShowOptions()));
    connect(ui->actionInversion, SIGNAL(triggered()), CrosswordGrid, SLOT(slotInversion()));
    connect(ui->actionVertically, SIGNAL(triggered()), CrosswordGrid, SLOT(slotMirrorVertically()));
    connect(ui->actionHorizontally, SIGNAL(triggered()), CrosswordGrid, SLOT(slotMirrorHorisontally()));
    connect(ui->actionToTheRight, SIGNAL(triggered()), CrosswordGrid, SLOT(slotRotateRight()));
    connect(ui->actionToTheLeft, SIGNAL(triggered()), CrosswordGrid, SLOT(slotRotateLeft()));
    connect(ui->actionReportBug, SIGNAL(triggered()), this, SLOT(slotReportBug()));
    connect(ui->actionCheckUpdates, SIGNAL(triggered()), this, SLOT(slotCheckForUpdates()));
    connect(wvUpdate, SIGNAL(loadFinished(bool)), this, SLOT(slotMakeCheckUpdates(bool)));
    connect(CrosswordGrid, SIGNAL(sigCallFunction(uchar)), this, SLOT(slotCallFunction(uchar)));

    ui->sbCrossword->addWidget(&lMode);
    ui->sbCrossword->addWidget(&lUserName);
    ui->sbCrossword->addWidget(&lCurrentCell);
    ui->sbCrossword->addWidget(&lCountLetters);
    ui->sbCrossword->addWidget(&lTitle);
    ui->sbCrossword->addWidget(&lAuthor);
    ui->sbCrossword->addWidget(&lCopyright);
    lMode.setToolTip(tr("Mode work of the program"));
    lUserName.setToolTip(tr("User name of the program"));
    lCurrentCell.setToolTip(tr("The position of the currently selected cell"));
    lCountLetters.setToolTip(tr("The number of letters the selected word"));
    lTitle.setToolTip(tr("Title crossword Across Lite puzzle format"));
    lAuthor.setToolTip(tr("Author crossword Across Lite puzzle format"));
    lCopyright.setToolTip(tr("Copyright crossword Across Lite puzzle format"));

    //Создание размещения виджетов на форме
    QSplitter* splitEditH;
    QSplitter* splitEditV;
    splitEditH = new QSplitter(Qt::Horizontal);
    splitEditV = new QSplitter(Qt::Vertical);
    setCentralWidget(splitEditV);
    splitEditV->addWidget(splitEditH);
    splitEditV->addWidget(ui->lwQuestions);
    splitEditV->resize(100, 100);
    splitEditV->show();
    splitEditH->addWidget(CrosswordGrid);
    splitEditH->addWidget(ui->gbData);
    splitEditH->resize(100,100);
    splitEditH->show();

    createToolBar();
    getUserName();
}

TKlestCrossword::~TKlestCrossword()
{
    delete FormStatistics;
    delete FormHelpViewer;
    delete CrosswordGrid;
    delete FileOptions;
    delete ui;
}

void TKlestCrossword::closeEvent(QCloseEvent *event)//функция закрытия приложения
{
    slotExitProgram();
    event->ignore();//игнорировать закрытие
}

//Создание панели инструментов
void TKlestCrossword::createToolBar()
{
    ui->tbStandard->addWidget(&tbNew);
    tbNew.setIcon(QPixmap(":/toolbar/toolbar/document-new.png"));
    tbNew.setText(tr("New"));
    tbNew.setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    tbNew.setToolTip(tr("New"));
    connect(&tbNew, SIGNAL(clicked()), SLOT(slotCreate()));

    ui->tbStandard->addWidget(&tbOpen);
    tbOpen.setIcon(QPixmap(":/toolbar/toolbar/document-open.png"));
    tbOpen.setText(tr("Open"));
    tbOpen.setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    tbOpen.setToolTip(tr("Open"));
    connect(&tbOpen, SIGNAL(clicked()), SLOT(slotOpen()));

    ui->tbStandard->addSeparator();

    ui->tbStandard->addWidget(&tbClose);
    tbClose.setIcon(QPixmap(":/toolbar/toolbar/dialog-close.png"));
    tbClose.setText(tr("Close"));
    tbClose.setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    tbClose.setToolTip(tr("Close"));
    connect(&tbClose, SIGNAL(clicked()), SLOT(slotClose()));

    ui->tbStandard->addWidget(&tbSave);
    tbSave.setIcon(QPixmap(":/toolbar/toolbar/document-save.png"));
    tbSave.setText(tr("Save"));
    tbSave.setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    tbSave.setToolTip(tr("Save"));
    connect(&tbSave, SIGNAL(clicked()), SLOT(slotSave()));

    ui->tbStandard->addWidget(&tbSaveAs);
    tbSaveAs.setIcon(QPixmap(":/toolbar/toolbar/document-save-as.png"));
    tbSaveAs.setText(tr("Save as"));
    tbSaveAs.setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    tbSaveAs.setToolTip(tr("Save as"));
    connect(&tbSaveAs, SIGNAL(clicked()), this,  SLOT(slotSave()));

    ui->tbStandard->addSeparator();

    ui->tbStandard->addWidget(&tbCheck);
    tbCheck.setIcon(QPixmap(":/toolbar/toolbar/dialog-ok-apply.png"));
    tbCheck.setText(tr("Check"));
    tbCheck.setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    tbCheck.setToolTip(tr("Check"));
    connect(&tbCheck, SIGNAL(clicked()), SLOT(slotCheckRightWords()));

    ui->tbStandard->addSeparator();

    ui->tbStandard->addWidget(&tbGrid);
    tbGrid.setIcon(QPixmap(":/toolbar/toolbar/system-suspend-hibernate.png"));
    tbGrid.setText(tr("Grid"));
    tbGrid.setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    tbGrid.setToolTip(tr("Grid"));
    connect(&tbGrid, SIGNAL(clicked()), CrosswordGrid, SLOT(slotAutoGrid()));

    ui->tbStandard->addWidget(&tbCrosswordCompiling);
    tbCrosswordCompiling.setIcon(QPixmap(":/toolbar/toolbar/system-run.png"));
    tbCrosswordCompiling.setText(tr("Compile"));
    tbCrosswordCompiling.setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    tbCrosswordCompiling.setToolTip(tr("Compile of crossword puzzles"));
    connect(&tbCrosswordCompiling, SIGNAL(clicked()), SLOT(slotCompile()));

    ui->tbStandard->addSeparator();

    ui->tbStandard->addWidget(&tbSettings);
    tbSettings.setIcon(QPixmap(":/toolbar/toolbar/preferences.png"));
    tbSettings.setText(tr("Options"));
    tbSettings.setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    tbSettings.setToolTip(tr("Options"));
    connect(&tbSettings, SIGNAL(clicked()), SLOT(slotShowOptions()));

    ui->tbStandard->addSeparator();

    ui->tbStandard->addWidget(&tbExit);
    tbExit.setIcon(QPixmap(":/toolbar/toolbar/application-exit.png"));
    tbExit.setText(tr("Exit"));
    tbExit.setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    tbExit.setToolTip(tr("Exit"));
    connect(&tbExit, SIGNAL(clicked()), SLOT(slotExitProgram()));

    return;
}

//Создание новой сетки кроссворда, с заданными параметрами
void TKlestCrossword::slotCreate()
{
    //Если кроссворд был изменён выдаётся сообщение о возможности сохранения
    if (checkSaved(SLOT_CREATE) == 1) return;

    TInputParametrGridCrossword FormInputParametrGridCrossword;
    QTableWidgetItem* CellCrossword;
    QBrush BrushWhite = QBrush(Qt::SolidPattern);
    QColor col1(227, 226, 228);
    col1.setAlpha(255);
    BrushWhite.setColor(col1);

    //Открывается форма для задания параметров сетки нового кроссворда
    if (FormInputParametrGridCrossword.exec() == QDialog::Accepted)
    {
        bEditTableSymmetrically = ui->actionSymmetrically->isChecked();
        bEditTableNotSymmetrically = ui->actionNotSymmetrically->isChecked();

        if(bEditTableSymmetrically == true || bEditTableNotSymmetrically == true)
            bEditTable = true;

        if(bEditTableSymmetrically == false || bEditTableNotSymmetrically == false)
            bEditTable = false;

        bEditWords = ui->actionWords->isChecked();

        if(FormInputParametrGridCrossword.ui->sbRowCount != 0)
            CrosswordGrid->setRowCount(FormInputParametrGridCrossword.ui->sbRowCount->value());

        if(FormInputParametrGridCrossword.ui->sbColumnCount != 0)
            CrosswordGrid->setColumnCount(FormInputParametrGridCrossword.ui->sbColumnCount->value());

        CrosswordGrid->create_usTable();

        for(int i = 0; i < CrosswordGrid->rowCount(); i++)
            for(int j = 0; j < CrosswordGrid->columnCount(); j++)
            {
            CrosswordGrid->setRowHeight(i, FormInputParametrGridCrossword.ui->sbCellSize->value());
            CrosswordGrid->setColumnWidth(j, FormInputParametrGridCrossword.ui->sbCellSize->value());
            CellCrossword = new QTableWidgetItem;
            CrosswordGrid->setItem(i,j, CellCrossword);
            CrosswordGrid->item(i, j)->setBackground(BrushWhite);
        }

        if(FormInputParametrGridCrossword.ui->sbRowCount != 0 ||
           FormInputParametrGridCrossword.ui->sbColumnCount != 0)
        {
            CrosswordGrid->setSaved(false);
            ui->pbDataOk->setEnabled(true);
            ui->menuAutomation->setEnabled(true);
            ui->menuEdit->setEnabled(true);
            ui->actionClose->setEnabled(true);
            tbClose.setEnabled(true);
            ui->menuAutomation->setEnabled(true);
            tbGrid.setEnabled(true);
            tbCrosswordCompiling.setEnabled(true);
            ui->actionExport->setEnabled(true);
        }
    }
    return;
}

//Открытие кроссворда
void TKlestCrossword::openFile()
{
    QString stmpFileName;
    QString sFilter;

    QFileDialog *pOpenCrossword;

    if (FileOptions->bModeWork == MODE_GUESSING)
        pOpenCrossword = new QFileDialog(0, tr("Crossword open"), tr("crossword1"),
                                         tr("Crossword format program of Klest-crossword") + " (*.xwc);;" +
                                         tr("Saving crossword format") + " (*.xwcsv);;" +
                                         tr("Saving of crossword text format") + " (*.txtsv);;");
    else
        pOpenCrossword = new QFileDialog(0, tr("Crossword open"), tr("crossword1"),
                                         tr("Crossword format program of Klest-crossword") + " (*.xwc);;" +
                                         tr("Template crossword format") + " (*.xwctpl);;");

    pOpenCrossword->setAcceptMode(QFileDialog::AcceptOpen);

    if(pOpenCrossword->exec() == QFileDialog::Accepted)
    {
        stmpFileName = pOpenCrossword->selectedFiles().at(0);

        if (stmpFileName.isEmpty())
        {
            delete pOpenCrossword;
            return;
        }

        sFilter = pOpenCrossword->selectedFilter();
        sFilter.remove(0, sFilter.indexOf("*"));
        sFilter.remove(sFilter.length() - 1, 1);
        sFilter.replace("*", "");
        if(stmpFileName.indexOf(sFilter) == -1) stmpFileName += sFilter;
    }
    else
    {
        delete pOpenCrossword;
        return;
    }

    delete pOpenCrossword;

    if(sFilter == ".txtsv")
    {
        CrosswordGrid->sFileNameCrossword = stmpFileName;
        CrosswordGrid->loadTemporary();
        CrosswordGrid->sFileNameCrossword.replace(".txtsv", ".txt");
        //Открытие сохранения неполностью отгаданного кроссворда(создание сетки)
        openAcrossLiteTXT(CrosswordGrid->sFileNameCrossword);
        dataToCrossword();
    }

    if(sFilter == ".xwcsv")
    {
        CrosswordGrid->sFileNameCrossword = stmpFileName;
        CrosswordGrid->loadTemporary();
        CrosswordGrid->sFileNameCrossword.replace(".xwcsv", ".xwc");
        //Открытие сохранения неполностью отгаданного кроссворда(создание сетки)
        CrosswordGrid->openFile("");
    }

    if(sFilter == ".xwctpl")
    {
        if (!CrosswordGrid->openFile(stmpFileName)) return;
    }

    if(sFilter == ".xwc")
    {
        CrosswordGrid->sFileNameCrossword = stmpFileName;

        if (!CrosswordGrid->openFile("")) return;
    }

    QString sFileNameTitle;
    sFileNameTitle.append(CrosswordGrid->sFileNameCrossword);
    sFileNameTitle.remove(0, sFileNameTitle.lastIndexOf("/") + 1);
    sFileNameTitle.truncate(sFileNameTitle.indexOf("."));

    ui->pbDataOk->setEnabled(true);
    ui->actionClose->setEnabled(true);
    tbClose.setEnabled(true);
    ui->actionSave->setEnabled(false);
    tbSave.setEnabled(false);

    if(FileOptions->bModeWork == MODE_GUESSING)
    {
        ui->actionCheck->setEnabled(true);
        tbCheck.setEnabled(true);
    }
    else
    {
        ui->menuEdit->setEnabled(true);
        ui->menuAutomation->setEnabled(true);
        tbGrid.setEnabled(true);
        tbCrosswordCompiling.setEnabled(true);
        ui->actionExport->setEnabled(true);
    }

    if (!CrosswordGrid->sFileNameCrossword.isEmpty())
    {
        setWindowTitle(sFileNameTitle + " - " + KLEST_VERSION);
        sStatNameCrossword = sFileNameTitle;
    }
}

//Проверка изменения кроссворда, перед закрытием
int TKlestCrossword::slotClose()
{
    if (checkSaved(SLOT_CLOSE) == 1) return 1;

    return 0;
}

//Проверка изменения кроссворда, перед сохранением
void TKlestCrossword::slotSave()
{
    //Если режим редактирования, то сохраняется кроссворд
    //Если режим просмотра, то сохраняется текущее состояние неполностью отгаданного кроссворда
    if(!CrosswordGrid->isSaved() && CrosswordGrid->sFileNameCrossword != "" && FileOptions->bModeWork == MODE_GUESSING)
        CrosswordGrid->saveToFileTemporary();
    else
    {

        if(!CrosswordGrid->isSaved() && CrosswordGrid->sFileNameCrossword != "" && FileOptions->bModeWork == MODE_EDITING)
            CrosswordGrid->saveToFile("");
        else
            CrosswordGrid->slotSaveAs();
    }

    if (!CrosswordGrid->sFileNameCrossword.isEmpty())
    {
        QString sFileNameTitle;
        sFileNameTitle.append(CrosswordGrid->sFileNameCrossword);
        sFileNameTitle.remove(0, sFileNameTitle.lastIndexOf("/") + 1);
        sFileNameTitle.truncate(sFileNameTitle.indexOf("."));
        setWindowTitle(sFileNameTitle + " - " + KLEST_VERSION);
        ui->actionSave->setEnabled(false);
        tbSave.setEnabled(false);
    }
}

//Открытие руководства пользователя
void TKlestCrossword::slotUserManual()
{
    FormHelpViewer->ui->tbHelpViewer->setSearchPaths(QStringList() << PATH_SHARE_DOC);
    FormHelpViewer->ui->tbHelpViewer->setSource(QString("klest-help.html"));
    FormHelpViewer->show();
}

//Открытие окна с информацией о программе
void TKlestCrossword::slotAboutProgram()
{
    TAboutProgram FormAboutProgram;
    FormAboutProgram.exec();
}

//Изменение параметров сетки кроссворда
void TKlestCrossword::slotChange()
{
    TInputParametrGridCrossword FormInputParametrGridCrossword;
    ui->actionSymmetrically->setChecked(bEditTableSymmetrically);
    ui->actionNotSymmetrically->setChecked(bEditTableNotSymmetrically);

    if(bEditTableSymmetrically == true || bEditTableNotSymmetrically == true) bEditTable = true;
    if(bEditTableSymmetrically == false || bEditTableNotSymmetrically == false) bEditTable = false;

    ui->actionWords->setChecked(bEditWords);
    QTableWidgetItem* CellCrossword;
    QBrush BrushWhite = QBrush(Qt::SolidPattern);
    QColor col1(227, 226, 228);
    col1.setAlpha(255);
    BrushWhite.setColor(col1);
    FormInputParametrGridCrossword.setWindowTitle(tr("Change parametr grid crossword"));
    int LastRow, LastColumn, LastFormat;

    if(CrosswordGrid->rowCount() != 0)
        FormInputParametrGridCrossword.ui->sbRowCount->setValue(CrosswordGrid->rowCount());

    if(CrosswordGrid->columnCount() != 0)
        FormInputParametrGridCrossword.ui->sbColumnCount->setValue(CrosswordGrid->columnCount());

    if(CrosswordGrid->columnWidth(0) != 0)
        FormInputParametrGridCrossword.ui->sbCellSize->setValue(CrosswordGrid->columnWidth(0));


    LastRow = CrosswordGrid->rowCount();
    LastColumn = CrosswordGrid->columnCount();
    LastFormat = CrosswordGrid->columnWidth(0);

    //Открытие формы для изменения параметров сетки уже открытого кроссворда
    if (FormInputParametrGridCrossword.exec() == QDialog::Accepted)
    {
        CrosswordGrid->delete_usTable();
        bEditTableSymmetrically = ui->actionSymmetrically->isChecked();
        bEditTableNotSymmetrically = ui->actionNotSymmetrically->isChecked();

        if(bEditTableSymmetrically == true || bEditTableNotSymmetrically == true)
            bEditTable = true;

        if(bEditTableSymmetrically == false || bEditTableNotSymmetrically == false)
            bEditTable = false;

        bEditWords = ui->actionWords->isChecked();
        CrosswordGrid->setRowCount(FormInputParametrGridCrossword.ui->sbRowCount->value());
        CrosswordGrid->setColumnCount(FormInputParametrGridCrossword.ui->sbColumnCount->value());
        CrosswordGrid->create_usTable();

        if(LastRow < FormInputParametrGridCrossword.ui->sbRowCount->value())
            for(int i = LastRow; i < FormInputParametrGridCrossword.ui->sbRowCount->value(); i++)
                CrosswordGrid->setRowHeight(i, FormInputParametrGridCrossword.ui->sbCellSize->value());

        if(LastColumn < FormInputParametrGridCrossword.ui->sbColumnCount->value())
            for(int j = LastColumn; j < FormInputParametrGridCrossword.ui->sbColumnCount->value(); j++)
                CrosswordGrid->setColumnWidth(j, FormInputParametrGridCrossword.ui->sbCellSize->value());

        if(LastRow < FormInputParametrGridCrossword.ui->sbRowCount->value() || LastColumn < FormInputParametrGridCrossword
           .ui->sbColumnCount->value() || LastFormat < FormInputParametrGridCrossword.ui->sbCellSize->value())

        for(int i = 0; i < CrosswordGrid->rowCount(); i++)
            for(int j = 0; j < CrosswordGrid->columnCount(); j++)
            {
            //Создание ячеек сетки кроссворда
            CellCrossword = new QTableWidgetItem;

            if(!CrosswordGrid->item(i,j))
                CrosswordGrid->setItem(i,j, CellCrossword);

            CrosswordGrid->item(i, j)->setBackground(BrushWhite);
        }

        if(LastFormat != FormInputParametrGridCrossword.ui->sbCellSize->value())
            for(int i = 0; i < CrosswordGrid->rowCount(); i++)
                for(int j = 0; j < CrosswordGrid->columnCount(); j++)
                {
            CrosswordGrid->setRowHeight(i, FormInputParametrGridCrossword.ui->sbCellSize->value());
            CrosswordGrid->setColumnWidth(j, FormInputParametrGridCrossword.ui->sbCellSize->value());
        }

        if(LastRow != FormInputParametrGridCrossword.ui->sbRowCount->value() ||
           LastColumn != FormInputParametrGridCrossword.ui->sbColumnCount->value() ||
           LastFormat != FormInputParametrGridCrossword.ui->sbCellSize->value())

            CrosswordGrid->setSaved(false);
    }
}

//Закрытие файла кроссворда
void TKlestCrossword::closeFile()
{
    CrosswordGrid->clearVariable();
    CrosswordGrid->slCrossingWord.clear();
    CrosswordGrid->slRightSelectedWord.clear();
    CrosswordGrid->sWord.clear();
    CrosswordGrid->sQuestion.clear();
    sWordsCheck.clear();
    CrosswordGrid->setAcrossLite(ACROSS_DISABLE);
    CrosswordGrid->slWordCheck.clear();
    ui->lwQuestions->clear();
    ui->leWord->clear();
    ui->pteQuestion->clear();
    ui->pbDataOk->setEnabled(false);
    ui->menuAutomation->setEnabled(false);
    ui->actionSave->setEnabled(false);
    ui->actionClose->setEnabled(false);
    lTitle.clear();
    lAuthor.clear();
    lCopyright.clear();
    CrosswordGrid->setInsertCoordinates(false);
    ui->actionPrint->setEnabled(false);
    tbSave.setEnabled(false);
    tbClose.setEnabled(false);

    if(FileOptions->bModeWork == MODE_EDITING)
    {
        tbGrid.setEnabled(false);
        tbCrosswordCompiling.setEnabled(false);
        tbSaveAs.setEnabled(false);
        ui->actionSave_as->setEnabled(false);
        ui->actionExport->setEnabled(false);
        ui->menuEdit->setEnabled(false);
    }
    else
    {
        tbCheck.setEnabled(false);
        ui->actionCheck->setEnabled(false);
    }

    setWindowTitle(KLEST_VERSION);

    if (!CrosswordGrid->sFileNameCrossword.isEmpty()) CrosswordGrid->delete_usTable();
}

//Проверка изменения кроссворда, перед открытием файла
void TKlestCrossword::slotOpen()
{
    if (checkSaved(SLOT_OPEN) == 1) return;
}

//Щелчёк по ячейке сетки кроссворда
void TKlestCrossword::slotCellClicked()
{
    int CrosswordRow;
    int CrosswordColumn;
    CrosswordRow = CrosswordGrid->currentRow();
    CrosswordColumn = CrosswordGrid->currentColumn();
    QBrush BrushBlack = QBrush(Qt::black, Qt::Dense1Pattern);
    QBrush BrushWhite = QBrush(Qt::SolidPattern);
    QColor col1(227, 226, 228);
    col1.setAlpha(255);
    BrushWhite.setColor(col1);
    lCurrentCell.setText(tr("Current cell") +": " + QString::number(CrosswordGrid->currentRow() + 1) + ":" +
                         QString::number(CrosswordGrid->currentColumn() + 1));

    //Если включён режим просмотра
    if(FileOptions->bModeWork == MODE_GUESSING)
    {
        //Если выделенная ячейка не чёрная
        if(CrosswordGrid->item(CrosswordRow, CrosswordColumn)->background() != BrushBlack)
        {
            ui->leWord->clear();
            ui->pteQuestion->clear();
            CrosswordGrid->tableUnselectAll();
            CrosswordGrid->makeSelectWordCrossword(CrosswordRow, CrosswordColumn, 2);
            slotEditWordTable(false);

            if(CrosswordGrid->sWord.indexOf(" ", 0) != -1) CrosswordGrid->sWord.clear();

            if(CrosswordGrid->sWord != "") ui->leWord->setText(CrosswordGrid->sWord);
            else ui->leWord->clear();

            if(CrosswordGrid->sQuestion != "") ui->pteQuestion->setPlainText(CrosswordGrid->sQuestion);
            else ui->pteQuestion->clear();

            CrosswordGrid->sWord.clear();
            CrosswordGrid->sQuestion.clear();
        }
        else CrosswordGrid->item(CrosswordRow, CrosswordColumn)->setSelected(false);
    }
    else
        //Если включён режим редактирования
    {
        //Если включён режим симметричного редактирования происходит симметричное закрашивание ячейки в чёрный цвет
        if(bEditTableSymmetrically == true)
        {
            if(CrosswordRow + 1 <= (CrosswordGrid->rowCount()) / 2 && bEditTable == true && CrosswordGrid->item(
                    CrosswordRow, CrosswordColumn)->background() != BrushBlack && CrosswordGrid->item(CrosswordRow, CrosswordColumn)->text() == "")
            {
                CrosswordGrid->item(CrosswordRow,CrosswordColumn)->setBackground(BrushBlack);
                CrosswordGrid->item(CrosswordGrid->rowCount() -  (CrosswordRow + 1),CrosswordColumn)->setBackground(BrushBlack);
                CrosswordGrid->item(CrosswordRow, CrosswordGrid->columnCount() - (CrosswordColumn + 1))->setBackground(BrushBlack);
                CrosswordGrid->item(CrosswordGrid->rowCount() - (CrosswordRow + 1), CrosswordGrid->columnCount() - (CrosswordColumn + 1))
                        ->setBackground(BrushBlack);
            }
            else
                if(CrosswordRow + 1 <= (CrosswordGrid->rowCount()) / 2 && bEditTable == true && CrosswordGrid->item(
                        CrosswordRow, CrosswordColumn)->background() == BrushBlack && CrosswordGrid->item(CrosswordRow, CrosswordColumn)->text()
                    == "")
                    {
                CrosswordGrid->item(CrosswordRow,CrosswordColumn) ->setBackground(BrushWhite);
                CrosswordGrid->item(CrosswordGrid->rowCount() - (CrosswordRow + 1),CrosswordColumn)->setBackground(BrushWhite);
                CrosswordGrid->item(CrosswordRow, CrosswordGrid->columnCount() - (CrosswordColumn + 1))->setBackground(BrushWhite);
                CrosswordGrid->item(CrosswordGrid->rowCount() - (CrosswordRow + 1), CrosswordGrid->columnCount() - (CrosswordColumn + 1))
                        ->setBackground(BrushWhite);
            }

            if(CrosswordRow + 1 >= (CrosswordGrid->rowCount()) / 2 && bEditTable == true && CrosswordGrid->item(
                    CrosswordRow, CrosswordColumn)->background() != BrushBlack && CrosswordGrid->item(CrosswordRow, CrosswordColumn)->text() == "")
            {
                CrosswordGrid->item(CrosswordRow,CrosswordColumn)->setBackground(BrushBlack);
                CrosswordGrid->item(CrosswordGrid->rowCount() - (CrosswordRow + 1),CrosswordColumn)->setBackground(BrushBlack);
                CrosswordGrid->item(CrosswordRow, CrosswordGrid->columnCount() - (CrosswordColumn + 1))->setBackground(BrushBlack);
                CrosswordGrid->item(CrosswordGrid->rowCount() - (CrosswordRow + 1), CrosswordGrid->columnCount() - (CrosswordColumn + 1))
                        ->setBackground(BrushBlack);
            }
            else
                if(CrosswordRow + 1 >= (CrosswordGrid->rowCount()) / 2 && bEditTable == true && CrosswordGrid->item(CrosswordRow,                                                              CrosswordColumn)->background() == BrushBlack &&
                   CrosswordGrid->item(CrosswordRow, CrosswordColumn)->text() == "")
                {
                CrosswordGrid->item(CrosswordRow,CrosswordColumn)->setBackground(BrushWhite);
                CrosswordGrid->item(CrosswordGrid->rowCount() - (CrosswordRow + 1),CrosswordColumn)->setBackground(BrushWhite);
                CrosswordGrid->item(CrosswordRow, CrosswordGrid->columnCount() - (CrosswordColumn + 1))->setBackground(BrushWhite);
                CrosswordGrid->item(CrosswordGrid->rowCount() - (CrosswordRow + 1), CrosswordGrid->columnCount() - (CrosswordColumn + 1))
                        ->setBackground(BrushWhite);
            }
        }
        else
            //Если включён режим не симметричного редактирования происходит закрашивание одной выделенной ячейки в чёрный цвет
            if(bEditTableNotSymmetrically == true)
            {
            if(CrosswordGrid->item(CrosswordRow, CrosswordColumn)->background() == BrushBlack)
                CrosswordGrid->item(CrosswordRow, CrosswordColumn)->setBackground(BrushWhite);
            else
                CrosswordGrid->item(CrosswordRow, CrosswordColumn)->setBackground(BrushBlack);
        }
        //Если включён режим редактирования слов, то происходит выделение слова в сетке
        if(bEditWords == true)
        {
            CrosswordGrid->tableUnselectAll();
            CrosswordGrid->makeSelectWordCrossword(CrosswordRow, CrosswordColumn, 2);
        }

        slotEditWordTable(false);
        ui->leWord->setText(CrosswordGrid->sWord);
        ui->pteQuestion->setPlainText(CrosswordGrid->sQuestion);
        CrosswordGrid->sWord.clear();
        CrosswordGrid->sQuestion.clear();
        ui->leWord->setFocus(Qt::TabFocusReason);
    }

    lCountLetters.setText(tr("Count letters: ") + QString::number(CrosswordGrid->SelectedWordToCheck.length()));

    if(CrosswordGrid->currentItem()->isSelected()) CrosswordGrid->currentItem()->setSelected(false);
}

//Изменение состояния сохранения кроссворда
void TKlestCrossword::slotCellChanged()
{
    CrosswordGrid->setSaved(false);

    if(FileOptions->bModeWork == MODE_EDITING)
    {
        ui->actionSave_as->setEnabled(true);
        tbSaveAs.setEnabled(true);
    }

    ui->actionSave->setEnabled(true);
    tbSave.setEnabled(true);
}

//При переключении программы в режим редактирования
void TKlestCrossword::slotRegimeEditing()
{
    if(slotClose() == 1)
    {
        ui->actionEditing->setChecked(false);
        return;
    }

    ui->actionGuessing->setChecked(false);
    ui->actionEditing->setChecked(true);
    setRegimeEditing();
}

//При переключении программы в режим просмотра
void TKlestCrossword::slotRegimeViewing()
{
    if(slotClose() == 1)
    {
        ui->actionGuessing->setChecked(false);
        return;
    }

    ui->actionEditing->setChecked(false);
    ui->actionGuessing->setChecked(true);
    setRegimeViewing();
}

//Передача данных из переменной в кроссворд
void TKlestCrossword::dataToCrossword()
{
    ui->lwQuestions->clear();
    ui->lwQuestions->addItems(CrosswordGrid->slQuestionT);
    CrosswordGrid->dataToCrossword();
}

//При выделении вопроса в списке вопросов
void TKlestCrossword::questionToSelect()
{
    ushort Row;
    ushort Column;
    int CurrentItem;

    CurrentItem = ui->lwQuestions->currentRow();
    CrosswordGrid->setNumberCurrentQuestion(CurrentItem);

    //Поиск координат слова для выделения
    CrosswordGrid->setNumberRowFirstLetterWord(int(CrosswordGrid->vBeginCoordWordRow.at(CurrentItem)));
    CrosswordGrid->setNumberColumnFirstLetterWord(int(CrosswordGrid->vBeginCoordWordColumn.at(CurrentItem)));
    CrosswordGrid->cDirectionWord = CrosswordGrid->vDirectionWord.at(CurrentItem);

    Row = CrosswordGrid->numberRowFirstLetterWord();
    Column = CrosswordGrid->numberColumnFirstLetterWord();
    CrosswordGrid->tableUnselectAll();
    CrosswordGrid->makeSelectWordCrossword(int(Row), int(Column), 3);
    CrosswordGrid->setFocus(Qt::TabFocusReason);
    CrosswordGrid->sWord = CrosswordGrid->slWordT.at(CurrentItem);
    CrosswordGrid->sQuestion = ui->lwQuestions->item(CurrentItem)->text();
    ui->leWord->clear();
    ui->pteQuestion->clear();

    if(CrosswordGrid->sWord.indexOf(" ") == -1) ui->leWord->setText(CrosswordGrid->sWord);
    if(CrosswordGrid->sQuestion != " ") ui->pteQuestion->setPlainText(CrosswordGrid->sQuestion);

    ui->leWord->setFocus(Qt::TabFocusReason);
}

//Передача данных из кроссворда в переменную
void TKlestCrossword::crosswordToData(uchar State)
{
    CrosswordGrid->crosswordToData(State);

    if(State == 2)
    {
        CrosswordGrid->slQuestionT.clear();
        CrosswordGrid->slQuestionL.clear();

        for(int i = 0; i < ui->lwQuestions->count(); i++)
        {
            CrosswordGrid->slQuestionL.append(ushort(ui->lwQuestions->item(i)->text().length()));
            CrosswordGrid->slQuestionT.append(ui->lwQuestions->item(i)->text());
        }
    }
}

//Подготовка данных к редактированию
void TKlestCrossword::slotEditWordTable(bool State)
{
    int CrosswordRow;
    int CrosswordColumn;
    int CurrentItem;
    int CountListWord;
    int CountListQuestion;
    int CountTemp;
    CrosswordRow = CrosswordGrid->currentRow();
    CrosswordColumn = CrosswordGrid->currentColumn();
    QBrush BrushBlack = QBrush(Qt::black, Qt::Dense1Pattern);
    CurrentItem = -1;
    CountListWord = 0;
    CountListQuestion = 0;
    CountTemp = 0;

    if(CrosswordGrid->slWordT.count() != 0) CountListWord = CrosswordGrid->slWordT.count();
    if(ui->lwQuestions->count() != 0) CountListQuestion = ui->lwQuestions->count();
    if(CountListWord != 0) CountTemp = CountListWord;

    //Если слово, выделенное в сетке, совпадает со словом в списке, то выделяется вопрос в списке вопросов
    if(State == false)
        for(int i = 0; i < CountTemp; i++)
        {
        if(CrosswordGrid->slWordT.at(i) == CrosswordGrid->SelectedWordToEdit) CurrentItem = i;
        CrosswordGrid->setNumberCurrentQuestion(CurrentItem);

        if(CurrentItem != -1)
        {
            ui->lwQuestions->item(CurrentItem)->setSelected(true);
            CrosswordGrid->sWord = CrosswordGrid->slWordT.at(CurrentItem);
            CrosswordGrid->sQuestion = CrosswordGrid->slQuestionT.at(CurrentItem);
        }
    }

    if(FileOptions->bModeWork == MODE_EDITING && State == false && CrosswordGrid->SelectedWordToEdit == "")
    {
        for(int j = 0; j < CrosswordGrid->vBeginCoordWordRow.count(); j++)
        {
            if(CrosswordGrid->vBeginCoordWordRow.at(j) == CrosswordGrid->numberRowFirstLetterWord() && CrosswordGrid->vBeginCoordWordColumn.at(j) ==
               CrosswordGrid->numberColumnFirstLetterWord())
            {
                if(CrosswordGrid->cDirectionWord == 'v' && CrosswordGrid->vDirectionWord.at(j) == CrosswordGrid->cDirectionWord)
                {
                    CrosswordGrid->setNumberCurrentQuestion(j);
                    ui->lwQuestions->item(j)->setSelected(true);
                    break;
                }

                if(CrosswordGrid->cDirectionWord == 'h' && CrosswordGrid->vDirectionWord.at(j) == CrosswordGrid->cDirectionWord)
                {
                    CrosswordGrid->setNumberCurrentQuestion(j);
                    ui->lwQuestions->item(j)->setSelected(true);
                    break;
                }
            }
        }
    }

    //Если слово в сетке кроссворда не обнаружено - идёт поиск по координатам
    if(FileOptions->bModeWork == MODE_GUESSING && CurrentItem == -1)
    {
        for(int j = 0; j < CrosswordGrid->vBeginCoordWordRow.count(); j++)
        {
            if(CrosswordGrid->vBeginCoordWordRow.at(j) == CrosswordGrid->numberRowFirstLetterWord() && CrosswordGrid->vBeginCoordWordColumn.at(j) ==
               CrosswordGrid->numberColumnFirstLetterWord())
            {
                CrosswordGrid->setNumberCurrentQuestion(j);

                if(CrosswordGrid->cDirectionWord == 'h' && CrosswordGrid->isAcrossLite() == ACROSS_ENABLE) break;
                if(CrosswordGrid->cDirectionWord == 'v' && CrosswordGrid->isAcrossLite() == ACROSS_DISABLE) break;
            }
        }

        if(CrosswordGrid->numberCurrentQuestion() != 65000 && CrosswordGrid->numberCurrentQuestion() != 65535)
            CurrentItem = CrosswordGrid->numberCurrentQuestion();

        if(CurrentItem != -1)
        {
            //ListWord->item(CurrentItem)->setSelected(true);
            ui->lwQuestions->item(CurrentItem)->setSelected(true);
            CrosswordGrid->sWord = CrosswordGrid->slWordT.at(CurrentItem);
            CrosswordGrid->sQuestion = ui->lwQuestions->item(CurrentItem)->text();
        }
    }

    if(FileOptions->bModeWork == MODE_GUESSING)
        if(CrosswordGrid->item(CrosswordRow, CrosswordColumn)->background() != BrushBlack
           /*&& EditWords == true*/)
        {
        CrosswordGrid->setEditDataCrossword(DATA_EDIT);

        if (!ui->leWord->text().isEmpty()) CrosswordGrid->sWord = ui->leWord->text();
        if (!ui->pteQuestion->toPlainText().isEmpty()) CrosswordGrid->sQuestion = ui->pteQuestion->toPlainText();

        CrosswordGrid->dataCrosswordEdit();
    }
    //SelectedWordToEdit.clear();
    if(State == true && FileOptions->bModeWork == MODE_EDITING)
    {
        CrosswordGrid->sWord = ui->leWord->text();
        CrosswordGrid->sQuestion = ui->pteQuestion->toPlainText();
        ui->lwQuestions->item(CrosswordGrid->numberCurrentQuestion())->setText(CrosswordGrid->sQuestion);

        if(CrosswordGrid->item(CrosswordRow, CrosswordColumn)->background() != BrushBlack
           && bEditWords == true && CrosswordGrid->SelectedWordToEdit == "" && CrosswordGrid->isGridEditHand() == HAND_DISABLE)
        {
            CrosswordGrid->setEditDataCrossword(DATA_ADD);
            CrosswordGrid->dataCrosswordEdit();
        }
        else
            if(CrosswordGrid->item(CrosswordRow, CrosswordColumn)->background() != BrushBlack && bEditWords == true && CrosswordGrid->isGridEditHand() ==
               HAND_ENABLE)
            {
            CrosswordGrid->setEditDataCrossword(DATA_EDIT);
            CrosswordGrid->dataCrosswordEdit();
        }
    }

    ui->leWord->setFocus(Qt::TabFocusReason);
}

//При выборе симметричного редактирования цвета ячеек
void TKlestCrossword::slotCellColorSymmetrically()
{
    ui->actionWords->setChecked(false);
    ui->actionSymmetrically->setChecked(true);
    ui->actionNotSymmetrically->setChecked(false);
    bEditWords = false;
    bEditTableSymmetrically = true;
    bEditTableNotSymmetrically = false;
    bEditTable = true;
    ui->pbDataOk->setEnabled(false);
    makeDeleteWords();
}

//При выборе не симметричного редактирования цвета ячеек
void TKlestCrossword::slotCellColorNotSymmetrically()
{
    ui->actionWords->setChecked(false);
    ui->actionNotSymmetrically->setChecked(true);
    ui->actionSymmetrically->setChecked(false);
    bEditWords = false;
    bEditTableSymmetrically = false;
    bEditTableNotSymmetrically = true;
    bEditTable = true;
    ui->pbDataOk->setEnabled(false);
    makeDeleteWords();
}

//При выборе редактирования слов
void TKlestCrossword::slotWords()
{
    ui->actionSymmetrically->setChecked(false);
    ui->actionNotSymmetrically->setChecked(false);
    bEditTable = false;
    ui->actionWords->setChecked(true);
    bEditTableSymmetrically = false;
    bEditTableNotSymmetrically = false;
    bEditWords = true;
    CrosswordGrid->setSelectWord(true);
    ui->pbDataOk->setEnabled(true);

    if(!CrosswordGrid->isInsertCoordinates())
    {
        CrosswordGrid->setInsertCoordinates(CrosswordGrid->InsertCoordinatesWords(ACROSS_DISABLE));

        CrosswordGrid->setSelectWord(false);

        for(int i = 0; i < CrosswordGrid->vBeginCoordWordRow.count(); i++)
        {
            CrosswordGrid->slWordT.append(" ");
            CrosswordGrid->slQuestionT.append(" ");
        }

        ui->lwQuestions->addItems(CrosswordGrid->slQuestionT);
    }
}

//При открытии шаблона
void TKlestCrossword::slotOpenTemplate()
{
    //Если кроссворд был изменён выдаётся сообщение о возможности сохранения
    if (checkSaved(SLOT_CREATE) == 1) return;

    TTemplateCrossword FormTemplateCrossword;
    FormTemplateCrossword.FileOptions = FileOptions;
    FormTemplateCrossword.InitForm();

    if(FormTemplateCrossword.exec() == QDialog::Accepted)
    {
        QDir dir;
        dir.setPath(FileOptions->sPathTemplate);
        uchar uCurrentTable, uFileCount;
        uFileCount = 0;

        if (FormTemplateCrossword.uCurrentTable == 255) return;

        uCurrentTable = FormTemplateCrossword.uCurrentTable;

        foreach (QString strFileName, dir.entryList(QDir::Files))
        {
            if(uCurrentTable == uFileCount) CrosswordGrid->sFileNameCrossword = FileOptions->sPathTemplate + "/" + strFileName;
            uFileCount++;
        }

        QString sFile;
        sFile = CrosswordGrid->sFileNameCrossword;
        CrosswordGrid->sFileNameCrossword.clear();
        //Загрузка шаблона
        CrosswordGrid->openFile(sFile);

        ui->menuAutomation->setEnabled(true);
        ui->actionClose->setEnabled(true);
        tbClose.setEnabled(true);
        ui->actionExport->setEnabled(true);
        ui->menuEdit->setEnabled(true);
        ui->menuAutomation->setEnabled(true);
        tbGrid.setEnabled(true);
        tbCrosswordCompiling.setEnabled(true);
        CrosswordGrid->setSaved(true);
    }
}

//Ввод слова в кроссворд
void TKlestCrossword::inputDataToCrossword()
{
    if(ui->leWord->text().length() < CrosswordGrid->SelectedWordToCheck.length())
        QMessageBox::warning(0, tr("Detection warning"), tr("Word is less than ") + QString::number(CrosswordGrid->SelectedWordToCheck.length()) + tr(" letters!"));
    else
        if(ui->leWord->text().length() > CrosswordGrid->SelectedWordToCheck.length())
            QMessageBox::warning(0, tr("Detection warning"), tr("Word more than ") + QString::number(CrosswordGrid->SelectedWordToCheck.length()) + tr(" letters!"));
    else
        if(ui->leWord->text() != "" && ui->pteQuestion->toPlainText() != "")
        {
        CrosswordGrid->setGridEditHand(HAND_ENABLE);
        slotEditWordTable(true);
    }
}

//Применение режима просмотра
void TKlestCrossword::setRegimeViewing()
{
    ui->actionCreate->setEnabled(false);
    ui->actionImport->setEnabled(true);
    ui->actionClose->setEnabled(false);
    ui->actionSave->setEnabled(false);
    ui->actionSave_as->setEnabled(false);
    ui->actionExport->setEnabled(false);
    ui->actionCheck->setEnabled(false);
    ui->actionPrint->setEnabled(false);
    ui->menuEdit->setEnabled(false);
    ui->menuAutomation->setEnabled(false);
    ui->actionTemplatesOpen->setEnabled(false);
    ui->actionDictionariesOpen->setEnabled(false);
    ui->pteQuestion->setReadOnly(true);

    tbNew.setEnabled(false);
    tbClose.setEnabled(false);
    tbSave.setEnabled(false);
    tbSaveAs.setEnabled(false);
    tbCheck.setEnabled(false);
    tbGrid.setEnabled(false);
    tbCrosswordCompiling.setEnabled(false);
    lMode.setText(tr("Guessing"));

    FileOptions->bModeWork = MODE_GUESSING;
}

//Применение режима редактирования
void TKlestCrossword::setRegimeEditing()
{
    ui->actionCreate->setEnabled(true);
    ui->actionImport->setEnabled(false);
    ui->actionClose->setEnabled(false);
    ui->actionSave->setEnabled(false);
    ui->actionSave_as->setEnabled(false);
    ui->actionExport->setEnabled(false);
    ui->actionCheck->setEnabled(false);
    ui->actionPrint->setEnabled(false);
    ui->menuEdit->setEnabled(false);
    ui->menuAutomation->setEnabled(false);
    ui->actionTemplatesOpen->setEnabled(true);
    ui->actionDictionariesOpen->setEnabled(true);
    ui->pteQuestion->setReadOnly(false);

    tbNew.setEnabled(true);
    tbClose.setEnabled(false);
    tbSave.setEnabled(false);
    tbSaveAs.setEnabled(false);
    tbCheck.setEnabled(false);
    tbGrid.setEnabled(false);
    tbCrosswordCompiling.setEnabled(false);
    lMode.setText(tr("Editing"));

    FileOptions->bModeWork = MODE_EDITING;
}

//Проверка правильности отгадывания кроссворда
void TKlestCrossword::slotCheckRightWords()
{
    //Количество процентов отгадывания кроссворда
    float RightPercent;//Процент отгаданного

    RightPercent = 0;
    QString strMessage;

    if(CrosswordGrid->rowCount() > 0)
    {
        int CountRightWord = 0;//Количество правильных слов

        for(int i = 0; i < CrosswordGrid->slWordT.count(); i++)
        {
            if(CrosswordGrid->slWordT.at(i) == CrosswordGrid->slWordCheck.at(i)) CountRightWord++;
        }

        RightPercent = (CountRightWord * 100) / CrosswordGrid->slWordT.count();
        strMessage.clear();
        strMessage.append(QString::number(RightPercent));
        strMessage.append(" % " + tr("crossword complete") + "!");

        QMessageBox::information(0, tr("Information"), strMessage);

        QString strStat;
        QString strDate;
        QString strTime;
        QDate date(QDate::currentDate());
        QTime time(QTime::currentTime());
        strDate.clear();
        strDate.append(date.toString(Qt::LocalDate));
        strTime.clear();
        strTime.append(time.toString(Qt::LocalDate));
        QFile file(PATH_STAT);
        QDataStream sfile(&file);
        if(!file.exists())
        {
            file.open(QIODevice::WriteOnly);
            strStat.clear();
            strStat.append("<HTML>\n"
                           " <BODY>\n"
                           "  <TABLE border=\"3\">\n"
                           "   <TR>\n"
                           "    <TD> " + tr("User name") + " </TD>\n"
                           "    <TD> " + tr("Name crossword") + " </TD>\n"
                           "    <TD> % </TD>\n"
                           "    <TD> " + tr("Date") + "  </TD>\n"
                           "    <TD> " + tr("Time") + "  </TD>\n"
                           "   </TR>\n"
                           "   <TR>\n"
                           "    <TD>" + lUserName.text() + "</TD>\n"
                           "    <TD>" + sStatNameCrossword + "</TD>\n"
                           "    <TD>" + QString::number(RightPercent) + "</TD>\n"
                           "    <TD>" + strDate + "</TD>\n"
                           "    <TD>" + strTime + "</TD>\n"
                           "   </TR>\n"
                           "  </TABLE>\n"
                           " </BODY>\n"
                           "</HTML>\n");
            /*for(int i = 0; i < strStat.length(); i++)
                           {
                            //cfile = char(ushort(strStat.at(i).unicode()));
                            sfile << strStat.at(i);
                           }*/
            sfile << strStat;
            file.close();
            FormStatistics->ui->teStatistics->setHtml(strStat);
        }
        else
        {
            QFile rfile(PATH_STAT);
            QDataStream srfile(&rfile);
            rfile.open(QIODevice::ReadOnly);
            strStat.clear();
            srfile >> strStat;
            rfile.close();
            QString strTemp;
            strTemp.clear();
            strTemp.append(strStat);
            int i = strTemp.indexOf("</TABLE>");
            strTemp.insert(i, "   <TR>\n"
                           "    <TD>" + lUserName.text() + "</TD>\n"
                           "    <TD>" + sStatNameCrossword + "</TD>\n"
                           "    <TD>" + QString::number(RightPercent) + "</TD>\n"
                           "    <TD>" + strDate + "</TD>\n"
                           "    <TD>" + strTime + "</TD>\n"
                           "   </TR>\n"
                           );
            strStat.clear();
            strStat.append(strTemp);
            QFile wfile(PATH_STAT);
            QDataStream swfile(&wfile);
            wfile.open(QIODevice::WriteOnly);
            swfile << strStat;
            wfile.close();
            FormStatistics->ui->teStatistics->setHtml(strStat);
        }

        closeFile();

        slotShowStatistics();
    }
}

//Открытие словаря
void TKlestCrossword::slotDictionaryOpen()
{
    TDictionary FormDictionary;
    QSqlDatabase db;
    db = QSqlDatabase::addDatabase("QSQLITE");

    if(FileOptions->bCurrentDictionary == LANGUAGE_RUSSIAN)
        db.setDatabaseName(PATH_SHARE_APP  "/dictionaries/dictionary_rus.xwcb");

    if(FileOptions->bCurrentDictionary == LANGUAGE_ENGLISH)
        db.setDatabaseName(PATH_SHARE_APP  "/dictionaries/dictionary_eng.xwcb");

    db.setUserName("user");
    db.setHostName("host");
    db.setPassword("password");

    if(!db.open())
    {
        QMessageBox::critical(0, tr("Detection error"), tr(
                "Error opennig dictionary base!\nRussian or English dictionarys "
                "can download on official site of the program."));
        return;
    }

    QSqlTableModel model;
    model.setTable("w0");
    model.select();
    model.setEditStrategy(QSqlTableModel::OnManualSubmit);
    FormDictionary.ui->tvDictionary->setModel(&model);
    FormDictionary.model2 = &model;

    FormDictionary.exec();
}

void TKlestCrossword::slotCompile()
{
    for(int i = 0; i < FileOptions->iNumberTimes; i++)
    {
        if (!makeCompile()) return;

        CrosswordGrid->setSaved(false);
        ui->lwQuestions->clear();
        ui->lwQuestions->addItems(CrosswordGrid->slQuestionT);

        CrosswordGrid->sFileNameCrossword = FileOptions->sPathCrossword + "/crossword" + QString::number(i + 1) + ".xwc";
        CrosswordGrid->saveToFile("");

        if(FileOptions->bStateExport == EXPORT_YES) CrosswordGrid->slotExport(true, i + 1, FileOptions->iTypeExport);
    }

}

//Выбор слов
bool TKlestCrossword::makeCompile()
{
    if (!CrosswordGrid->makeCompile()) return false;

    ui->actionPrint->setEnabled(true);
    ui->actionExport->setEnabled(true);
    ui->pbDataOk->setEnabled(true);

    return true;
}

//Выход из программы
void TKlestCrossword::slotExitProgram()
{
    if (checkSaved(SLOT_EXIT_PROGRAM) == 1) return;
}

//Изменение направления слова
void TKlestCrossword::slotChangeLine()
{
    if(CrosswordGrid->cDirectionWord == 'h') CrosswordGrid->cDirectionWord = 'v';
    else CrosswordGrid->cDirectionWord = 'h';
}

/*bool TKlestCrossword::checkRightSelect()
{
    QString slR;
    QString slC;
    QString slV;
    QString sW;
    QString slCW;
    int ir, ic;
    bool bCheck;
    bCheck = false;

    for(int i = 0; i < CrosswordGrid->slWordT.count(); i++)
    {
        slR = CrosswordGrid->slWordR.at(i);
        slC = CrosswordGrid->slWordC.at(i);
        slV = CrosswordGrid->slWordV.at(i);
        ir = slR.at(0).unicode();
        ic = slC.at(0).unicode();
        CrosswordGrid->cDirectionWord = slV.at(0);
        CrosswordGrid->setCurrentCell(ir, ic);
        selectWordCrossword(ir, ic, 0);
        sW = CrosswordGrid->slWordT.at(i);

        if(SelectedWordToEdit != CrosswordGrid->slWordT.at(i) && CrosswordGrid->slWordT.at(i) != "" && sW.indexOf(" ") == -1)
        {
            slCW = ListCrossingWord.at(i);
            RightSelectedWord.replace(i, "no");

            for(int j = 0; j < slCW.count(); j++)
            {
                RightSelectedWord.replace(slCW.at(j).unicode(), "no");
            }

            bCheck = true;
        }
    }

    //TableUnselectAll();
    return bCheck;
}*/

void TKlestCrossword::slotImportCrossword()
{
    if (checkSaved(SLOT_CLOSE) == 1) return;

    QString sImportName;
    QString sFilter;
    QFileDialog *pDialogImport;
    pDialogImport = new QFileDialog(0, tr("Import"), tr(""),
                                    tr("Text Format") + " AcrossLite (*.txt);;"
                                    "OK - OpenKlest Format (*.ok)");
    pDialogImport->setAcceptMode(QFileDialog::AcceptOpen);

    if(pDialogImport->exec() == QFileDialog::Accepted)
    {
        sImportName = pDialogImport->selectedFiles().at(0);
        sFilter = pDialogImport->selectedFilter();

        if(sFilter.indexOf("*.txt") != -1) sFilter = ".txt";
        if(sFilter.indexOf("*.ok") != -1) sFilter = ".ok";


        if(sImportName.indexOf(sFilter) == -1) sImportName += sFilter;
    }
    else return;

    delete pDialogImport;

    if(!sImportName.isEmpty())
    {
        if (sFilter == ".txt") openAcrossLiteTXT(sImportName);
        if (sFilter == ".ok") slotImportOK(sImportName);
    }
}

void TKlestCrossword::openAcrossLiteTXT(QString FileName)
{
    ui->lwQuestions->clear();
    CrosswordGrid->openAcrossLiteTXT(FileName);
    lAuthor.setText(CrosswordGrid->sAcrossAuthor);
    lTitle.setText(CrosswordGrid->sAcrossTitle);
    lCopyright.setText(CrosswordGrid->sAcrossCopyright);
    ui->lwQuestions->addItems(CrosswordGrid->slQuestionT);
    ui->actionClose->setEnabled(true);
    ui->actionCheck->setEnabled(true);
    tbClose.setEnabled(true);
    tbCheck.setEnabled(true);
    ui->pbDataOk->setEnabled(true);
    ui->actionSave->setEnabled(false);
    tbSave.setEnabled(false);

    QString sFileNameTitle;
    sFileNameTitle.append(FileName);
    sFileNameTitle.remove(0, sFileNameTitle.lastIndexOf("/") + 1);
    sFileNameTitle.truncate(sFileNameTitle.indexOf("."));
    setWindowTitle(sFileNameTitle + " - " + KLEST_VERSION);
    sStatNameCrossword = sFileNameTitle;
}

void TKlestCrossword::slotSemiautomaticInput()
{
    if(!CrosswordGrid->isInsertCoordinates() || ui->actionWords->isChecked() != true || FileOptions->bModeWork == MODE_GUESSING) return;

    TSemiautomaticWindow FormSemiautomaticWindow;

    QSqlDatabase db;
    db = QSqlDatabase::addDatabase("QSQLITE");

    if(FileOptions->bCurrentDictionary == LANGUAGE_RUSSIAN)
        db.setDatabaseName(PATH_SHARE_APP  "/dictionaries/dictionary_rus.xwcb");

    if(FileOptions->bCurrentDictionary == LANGUAGE_ENGLISH)
        db.setDatabaseName(PATH_SHARE_APP  "/dictionaries/dictionary_eng.xwcb");

    db.setUserName("user");
    db.setHostName("host");
    db.setPassword("password");

    if(!db.open())
    {
        QMessageBox::critical(0, tr("Detection error"), tr(
                "Error opennig dictionary base!\nRussian or English dictionarys "
                "can download on official site of the program."));
        return;
    }

    QSqlQueryModel modelWord;
    QString strQuery;
    QString strInterpretation;
    QStringList ListInterpretation;
    QStringList ListQueryWord;

    modelWord.setQuery("SELECT Word FROM w" + QString::number(CrosswordGrid->countLetterInWord() - 1) + ";",  db);
    modelWord.query();

    QSqlQuery QueryInterpretation,
    QueryWord;
    strQuery = "SELECT Interpretation FROM w" + QString::number(CrosswordGrid->countLetterInWord() - 1) + ";";
    QueryInterpretation.exec(strQuery);
    strQuery = "SELECT Word FROM w" + QString::number(CrosswordGrid->countLetterInWord() - 1) + ";";
    QueryWord.exec(strQuery);
    QSqlRecord InterpretationRecord,
    WordRecord;
    InterpretationRecord = QueryInterpretation.record();
    WordRecord = QueryWord.record();
    ListInterpretation.clear();
    ListQueryWord.clear();

    while(QueryInterpretation.next())
    {
        strInterpretation = QueryInterpretation.value(InterpretationRecord.indexOf("Interpretation")).toString();
        ListInterpretation << strInterpretation;
    }

    while(QueryWord.next())
    {
        strInterpretation = QueryWord.value(WordRecord.indexOf("Word")).toString();
        ListQueryWord << strInterpretation;
    }

    bool AllSpace = false;
    if(CrosswordGrid->SelectedWordToCheck.length() != 0)
    {
        for(int k = 0; k < CrosswordGrid->SelectedWordToCheck.length(); k++)
            if(CrosswordGrid->SelectedWordToCheck.at(k) == ' ') AllSpace = true;
        else
        {
            AllSpace = false;
            break;
        }
    }

    if(AllSpace == false && CrosswordGrid->SelectedWordToCheck.count() > 1)
    {
        int i = 0;

        while( i < ListQueryWord.count())
        {
            strInterpretation = ListQueryWord.at(i);

            for(int j = 0; j < strInterpretation.count();  j++)
                if(CrosswordGrid->SelectedWordToCheck.at(j) != ' ' &&  strInterpretation.at(j) != CrosswordGrid->SelectedWordToCheck.at(j))
                {
                ListQueryWord.removeAt(i);
                ListInterpretation.removeAt(i);
                i--;
                break;
            }

            i++;
        }

        //for(int i = 0; i < ListQueryWord.count();  i++)
        FormSemiautomaticWindow.ui->listWidget->addItems(ListQueryWord);
        FormSemiautomaticWindow.ui->lvWords->setModel(FormSemiautomaticWindow.ui->listWidget->model());
    }
    else FormSemiautomaticWindow.ui->lvWords->setModel(&modelWord);

    FormSemiautomaticWindow.ListInterpretation = &ListInterpretation;
    //FormSemiautomaticWindow.ListWords->setModel(&modelWord);

    if(FormSemiautomaticWindow.exec() == QDialog::Accepted )
    {
        ui->leWord->setText(FormSemiautomaticWindow.Word);
        ui->pteQuestion->setPlainText(FormSemiautomaticWindow.Question);

        CrosswordGrid->sWord = FormSemiautomaticWindow.Word;
        CrosswordGrid->sQuestion = FormSemiautomaticWindow.Question;
        ui->lwQuestions->item(CrosswordGrid->numberCurrentQuestion())->setText(CrosswordGrid->sQuestion);

        if(ui->leWord->text() != "" && ui->pteQuestion->toPlainText() != "")
        {
            CrosswordGrid->setGridEditHand(HAND_ENABLE);
            slotEditWordTable(true);
        }
    }
}

void TKlestCrossword::slotShowStatistics()
{
    QString strStat;
    QFile rfile(PATH_STAT);
    QDataStream srfile(&rfile);
    rfile.open(QIODevice::ReadOnly);
    strStat.clear();
    srfile >> strStat;
    rfile.close();
    FormStatistics->ui->teStatistics->setHtml(strStat);

    if(FileOptions->bModeWork == MODE_GUESSING) FormStatistics->ui->pbStatisticsClear->hide();
    else FormStatistics->ui->pbStatisticsClear->show();

    FormStatistics->show();
}

void TKlestCrossword::getUserName()
{
    bool bOk;
    QString str = QInputDialog::getText(0, tr("Input user name"), tr("User name") + ":", QLineEdit::Normal, tr("User1"), &bOk);
    if (bOk)
    {
        lUserName.setText(str);
    }
    else
    {
        lUserName.setText(tr("User1"));
    }
}

void TKlestCrossword::slotOpenDonate()
{
    if(FileOptions->bLanguageProgram == LANGUAGE_RUSSIAN) QDesktopServices::openUrl(QUrl("http://labfreetech.org/information.html"));
    if(FileOptions->bLanguageProgram == LANGUAGE_ENGLISH) QDesktopServices::openUrl(QUrl("http://en.labfreetech.org/information.html"));
}

void TKlestCrossword::slotGetSource()
{
    QDesktopServices::openUrl(QUrl("http://sourceforge.net/scm/?type=git&group_id=556915"));
}

void TKlestCrossword::slotOpenGetSupport()
{
    if(FileOptions->bLanguageProgram == LANGUAGE_RUSSIAN) QDesktopServices::openUrl(QUrl("http://labfreetech.org/support.html"));
    if(FileOptions->bLanguageProgram == LANGUAGE_ENGLISH) QDesktopServices::openUrl(QUrl("http://en.labfreetech.org/support.html"));
}

void TKlestCrossword::makeDeleteWords()
{
    CrosswordGrid->cDirectionWord = 'v';
    CrosswordGrid->vBeginCoordWordRow.clear();
    CrosswordGrid->vBeginCoordWordColumn.clear();
    CrosswordGrid->vDirectionWord.clear();
    CrosswordGrid->vLengthWord.clear();
    CrosswordGrid->slSelectWord.clear();
    CrosswordGrid->slWordT.clear();
    CrosswordGrid->slQuestionT.clear();
    ui->lwQuestions->clear();
    CrosswordGrid->setInsertCoordinates(false);
}

void TKlestCrossword::slotClearAll()
{
    for(int i = 0; i < CrosswordGrid->rowCount(); i++)
        for(int j = 0; j < CrosswordGrid->columnCount(); j++)
            if(CrosswordGrid->item(i, j)->text() != "") CrosswordGrid->item(i, j)->setText("");

    QBrush BrushWhite = QBrush(Qt::SolidPattern);
    QColor col1(227, 226, 228);
    col1.setAlpha(255);
    BrushWhite.setColor(col1);

    for(int i = 0; i < CrosswordGrid->rowCount(); i++)
        for(int j = 0; j < CrosswordGrid->columnCount(); j++)
            if(CrosswordGrid->item(i, j)->background() != BrushWhite ) CrosswordGrid->item(i, j)->setBackground(BrushWhite);

    makeDeleteWords();
}

void TKlestCrossword::slotClearWords()
{
    for(int i = 0; i < CrosswordGrid->rowCount(); i++)
        for(int j = 0; j < CrosswordGrid->columnCount(); j++)
            if(CrosswordGrid->item(i, j)->text() != "") CrosswordGrid->item(i, j)->setText("");

    makeDeleteWords();
}

void TKlestCrossword::slotClearGrid()
{
    QBrush BrushWhite = QBrush(Qt::SolidPattern);
    QColor col1(227, 226, 228);
    col1.setAlpha(255);
    BrushWhite.setColor(col1);

    for(int i = 0; i < CrosswordGrid->rowCount(); i++)
        for(int j = 0; j < CrosswordGrid->columnCount(); j++)
            if(CrosswordGrid->item(i, j)->background() != BrushWhite ) CrosswordGrid->item(i, j)->setBackground(BrushWhite);

    if (CrosswordGrid->isInsertCoordinates())
    {
        slotClearWords();
    }
}

void TKlestCrossword::slotMouseMove()
{

    if(bEditTable == true)
    {
        slotCellClicked();
    }
}

/*          void TKlestCrossword::slotCheckNewVersion()
            {
              Update FormUpdate;

                if (FormUpdate.exec() == QDialog::Accepted)
                    {

                    }
            }*/

void TKlestCrossword::slotShowOptions()
{
    TOptions *FormOptions = new TOptions;
    FormOptions->ui->sbNumberTimes->setValue(FileOptions->iNumberTimes);

    if(FileOptions->bStateExport == EXPORT_YES)
    {
        FormOptions->ui->cbExport->setChecked(true);
        FormOptions->ui->cbTypeExport->setEnabled(true);
        FormOptions->ui->lTypeExport->setEnabled(true);
    }
    else FormOptions->ui->cbExport->setChecked(false);

    FormOptions->ui->cbTypeExport->setCurrentIndex(FileOptions->iTypeExport);

    if(FileOptions->bCurrentDictionary == LANGUAGE_RUSSIAN) FormOptions->ui->cbDictionary->setCurrentIndex(0);
    else FormOptions->ui->cbDictionary->setCurrentIndex(1);

    if(FileOptions->bLanguageProgram == LANGUAGE_RUSSIAN) FormOptions->ui->cbLanguage->setCurrentIndex(0);
    else FormOptions->ui->cbLanguage->setCurrentIndex(1);

    if(FileOptions->bStateStandardToolbar == TOOLBAR_SHOW) FormOptions->ui->cbStandardPanel->setChecked(true);
    else FormOptions->ui->cbStandardPanel->setChecked(false);

    FormOptions->ui->lePathCrossword->setText(FileOptions->sPathCrossword);
    FormOptions->ui->lePathTemplates->setText(FileOptions->sPathTemplate);

    if(FormOptions->exec() == QDialog::Accepted)
    {
        FileOptions->iNumberTimes = FormOptions->ui->sbNumberTimes->value();

        if(FormOptions->ui->cbExport->isChecked())
            FileOptions->bStateExport = EXPORT_YES;
        else
            FileOptions->bStateExport = EXPORT_NO;

        FileOptions->iTypeExport = FormOptions->ui->cbTypeExport->currentIndex();

        if(FormOptions->ui->cbDictionary->currentIndex() == 0)
            FileOptions->bCurrentDictionary = LANGUAGE_RUSSIAN;
        else
            FileOptions->bCurrentDictionary = LANGUAGE_ENGLISH;

        if(FormOptions->ui->cbLanguage->currentIndex() == 0)
            FileOptions->bLanguageProgram = LANGUAGE_RUSSIAN;
        else
            FileOptions->bLanguageProgram = LANGUAGE_ENGLISH;

        if(FormOptions->ui->cbStandardPanel->isChecked())
        {
            FileOptions->bStateStandardToolbar = TOOLBAR_SHOW;
            ui->tbStandard->setVisible(TOOLBAR_SHOW);
        }
        else
        {
            FileOptions->bStateStandardToolbar = TOOLBAR_HIDE;
            ui->tbStandard->setVisible(TOOLBAR_HIDE);
        }

        FileOptions->sPathCrossword = FormOptions->ui->lePathCrossword->text();
        FileOptions->sPathTemplate = FormOptions->ui->lePathTemplates->text();
    }

    delete FormOptions;
}

void TKlestCrossword::slotMakeExport()
{
    CrosswordGrid->slQuestionT.clear();

    for (int i = 0; i < ui->lwQuestions->count(); i++)
        CrosswordGrid->slQuestionT.append(ui->lwQuestions->item(i)->text());

    CrosswordGrid->slotExport(false, 0, -1);
}

void TKlestCrossword::slotImportOK(QString sImportName)
{
    CrosswordGrid->openOpenKlestFile(sImportName);
    ui->pbDataOk->setEnabled(true);
    ui->actionClose->setEnabled(true);
    tbClose.setEnabled(true);
    ui->actionSave->setEnabled(false);
    tbSave.setEnabled(false);
    ui->actionCheck->setEnabled(true);
    tbCheck.setEnabled(true);

    QString sFileNameTitle;
    sFileNameTitle.append(sImportName);
    sFileNameTitle.remove(0, sFileNameTitle.lastIndexOf("/") + 1);
    sFileNameTitle.truncate(sFileNameTitle.indexOf("."));
    setWindowTitle(sFileNameTitle + " - " + KLEST_VERSION);
    sStatNameCrossword = sFileNameTitle;
}

void TKlestCrossword::slotReportBug()
{
    QDesktopServices::openUrl(QUrl("http://sourceforge.net/tracker/?group_id=556915&atid=2259683"));
}

void TKlestCrossword::writeOptions()
{
    QSettings setFileOptions(QSettings::IniFormat, QSettings::UserScope, QCoreApplication::organizationName(), QCoreApplication::applicationName());

    setFileOptions.beginGroup("/Options");
    setFileOptions.setValue("/bModeWork", FileOptions->bModeWork);
    setFileOptions.setValue("/bLanguageProgram", FileOptions->bLanguageProgram);
    setFileOptions.setValue("/bStateStandardToolbar", FileOptions->bStateStandardToolbar);
    setFileOptions.setValue("/bCurrentDictionary", FileOptions->bCurrentDictionary);
    setFileOptions.setValue("/bStateExport", FileOptions->bStateExport);
    setFileOptions.setValue("/iNumberTimes", FileOptions->iNumberTimes);
    setFileOptions.setValue("/iTypeExport", FileOptions->iTypeExport);
    setFileOptions.setValue("/bUseCrosswordPassword", FileOptions->bUseCrosswordPassword);
    setFileOptions.setValue("/bUseModeEditingPassword", FileOptions->bUseModeEditingPassword);
    setFileOptions.setValue("/sModeEditingPassword", FileOptions->sModeEditingPassword);
    setFileOptions.setValue("/sCrosswordPassword", FileOptions->sCrosswordPassword);
    setFileOptions.setValue("/sPathCrossword", FileOptions->sPathCrossword);
    setFileOptions.setValue("/sPathTemplate", FileOptions->sPathTemplate);
    setFileOptions.endGroup();
    setFileOptions.sync();
}

int TKlestCrossword::checkSaved(uchar ucAction)
{
    if(!CrosswordGrid->isSaved())
    {
        int inbox = CrosswordGrid->messageChangeGrid();

        if(inbox == QMessageBox::Yes)
        {
            switch(ucAction)
            {
            case SLOT_CREATE:
                slotSave();
                closeFile();
                break;

            case SLOT_CLOSE:

                if(FileOptions->bModeWork == MODE_EDITING) slotSave();
                else CrosswordGrid->saveToFileTemporary();

                closeFile();
                break;

            case SLOT_OPEN:
                slotSave();
                closeFile();
                openFile();
                break;

            case SLOT_EXIT_PROGRAM:
                slotSave();
                closeFile();
                writeOptions();
                exit(0);
                break;
            }

            CrosswordGrid->setSaved(true);
        }
        else
            if(inbox == QMessageBox::No)
            {
            switch(ucAction)
            {
            case SLOT_CREATE:
                closeFile();
                break;

            case SLOT_CLOSE:
                closeFile();
                break;

            case SLOT_OPEN:
                closeFile();
                openFile();
                break;

            case SLOT_EXIT_PROGRAM:
                closeFile();
                writeOptions();
                exit(0);
                break;
            }

            CrosswordGrid->setSaved(true);
        }
        else
            if(inbox == QMessageBox::Cancel)
            {
            return 1;
        }
    }
    else
    {
        switch(ucAction)
        {
        case SLOT_CREATE:
            closeFile();
            break;

        case SLOT_CLOSE:
            closeFile();
            break;

        case SLOT_OPEN:
            closeFile();
            openFile();
            break;

        case SLOT_EXIT_PROGRAM:
            closeFile();
            writeOptions();
            exit(0);
            break;
        }
    }

    return 0;
}

void TKlestCrossword::slotCheckForUpdates()
{
    wvUpdate->load(QUrl("http://labfreetech.org/klest_1.0.0.html"));
}

void TKlestCrossword::slotMakeCheckUpdates(bool bError)
{
    if (bError && wvUpdate->page()->findText("new_version")) QMessageBox::information(0, tr("Information"), tr("On the official website available a new version!"));
    else QMessageBox::information(0, tr("Information"), tr("You have the latest version!"));
}

void TKlestCrossword::slotPrint()
{
    CrosswordGrid->slQuestionT.clear();

    for (int i = 0; i < ui->lwQuestions->count(); i++)
        CrosswordGrid->slQuestionT.append(ui->lwQuestions->item(i)->text());

    CrosswordGrid->print();
}

void TKlestCrossword::slotCallFunction(uchar ucType)
{
    switch(ucType)
    {
    case DATA_TO_CROSSWORD:
        dataToCrossword();
        break;

    case S_CLOSE:
        closeFile();
        break;

    }
}
