/* ======================================================================
**  TKlest-crossword.h - Заголовочный файл модуля главной формы программы.
**  Является частью программы Клёст-кроссворд, распространяемой под
**  Стандартной общественной лицензией, версии 2 (GNU GPL v2).
** ======================================================================
**  Copyleft 2009 - 2012 by Lab Free Technologies <admin@LabFreeTech.org>
** ==============================================================================
**  Copyleft 2009 - 2012 Лаборатория свободных технологий <admin@LabFreeTech.org>
** ==============================================================================
*/

#ifndef TKLESTCROSSWORD_H
#define TKLESTCROSSWORD_H
#include "ui_TKlestCrossword.h"
#include "../TInputParametrGridCrossword/TInputParametrGridCrossword.h"
#include "../TTemplateCrossword/TTemplateCrossword.h"
#include "../TDictionary/TDictionary.h"
#include "../TAboutProgram/TAboutProgram.h"
#include "../TCrosswordGrid/TCrosswordGrid.h"
#include "../TSemiautomaticWindow/TSemiautomaticWindow.h"
#include "../TStatistics/TStatistics.h"
#include "../TOptions/TOptions.h"
#include "../THelpViewer/THelpViewer.h"
#include <QtCore/QCoreApplication>
#include <QtCore/QTime>
#include <QtCore/QDate>
#include <QtCore/QSettings>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QFile>
#include <QtCore/QDataStream>
#include <QtCore/QTextStream>
#include <QtCore/QByteArray>
#include <QtCore/QBuffer>
#include <QtCore/QChar>
#include <QtCore/QDir>
#include <QtGui/QApplication>
#include <QtGui/QSplitter>
#include <QtGui/QPainter>
#include <QtGui/QInputDialog>
#include <QtGui/QDesktopServices>
#include <QtGui/QPrinter>
#include <QtGui/QPalette>
#include <QtGui/QBrush>
#include <QtGui/QColor>
#include <QtGui/QTableWidget>
#include <QtGui/QTextEdit>
#include <QtGui/QPixmap>
#include <QtGui/QTableWidgetItem>
#include <QtGui/QFileDialog>
#include <QtGui/QDialog>
#include <QtGui/QLineEdit>
#include <QtGui/QDesktopServices>
#include <QtCore/QUrl>
#include <QtSql/QSqlDatabase>
#include <QtSql/QSqlTableModel>
#include <QtSql/QSqlQueryModel>
#include <QtSql/QSqlQuery>
#include <QtWebKit/QWebView>
#include "const.h"
#include "struct.h"

namespace Ui {
    class TKlestCossword;
}

class TKlestCrossword : public QMainWindow, public Ui::TKlestCrossword {
    Q_OBJECT

public:

#define KLEST_VERSION tr("Klest-crossword") + " 1.0.0"

    QString sLastCell;//Последняя ячейка
    QString sWordsCheck;//Слово для проверки
    QString sStatNameCrossword;//Имя пользователя в статистике

    bool bEditTableSymmetrically;//Симметричное редактирование сетки кроссворда
    bool bEditTableNotSymmetrically;//Несимметричное редактирование сетки кроссворда
    bool bEditTable;//Редактировать сетку кроссворда
    bool bEditWords;//Редактировать слова

    QLabel lMode;//Режим работы программы
    QLabel lUserName;//Имя пользователя
    QLabel lCurrentCell;//Координаты ячейки
    QLabel lCountLetters;//Количество букв
    //AcrossLite
    QLabel lTitle;//Заголовок
    QLabel lAuthor;//Автор
    QLabel lCopyright;//Права

    QToolButton tbNew;
    QToolButton tbOpen;
    QToolButton tbClose;
    QToolButton tbSave;
    QToolButton tbSaveAs;
    QToolButton tbCheck;
    QToolButton tbGrid;
    QToolButton tbCrosswordCompiling;
    QToolButton tbSettings;
    QToolButton tbExit;

    QStringList BackWords;//Возвращаемые слова

    TStatistics *FormStatistics;
    THelpViewer *FormHelpViewer;
    TCrosswordGrid *CrosswordGrid;
    TFileOptions *FileOptions;

    explicit TKlestCrossword(QMainWindow *pwgt = 0);
    ~TKlestCrossword();

    Ui::TKlestCrossword *ui;

    void getUserName();//Ввод имени пользователя

private:
        QWebView *wvUpdate;//Для проверки наличия новой версии

        void createToolBar();//Создание панелей инструментов

protected:
       void closeEvent(QCloseEvent *event);//Событие закрытия окна

public slots:
    void slotCreate();//При создании кроссворда
    void openFile();//При открытии кроссворда
    int slotClose();//При закрытии кроссворда
    void slotSave();//При сохранении кроссворда
    void slotUserManual();//При открытии руководства пользователя
    void slotAboutProgram();//При открытии информации опрограмме
    void slotChange();//При изменении параметров таблицы кроссворда
    void closeFile();//При закрытии кроссворда
    void slotOpen();//При открытии кроссворда
    void slotCellClicked();//При щелчке по ячейке кроссворда
    void slotCellChanged();//При изменении текста в ячейке таблицы кроссворда
    void slotRegimeEditing();//При переключении программы в режим редактирования
    void slotRegimeViewing();//При переключении программы в режим просмотра
    void dataToCrossword();//Передача данных из переменной в кроссворд
    void questionToSelect();//При выделении вопроса в списке вопросов
    void crosswordToData(uchar State);//Передача данных из кроссворда в переменную
    void slotEditWordTable(bool);//Подготовка данных к редактированию
    void slotCellColorSymmetrically();//При выборе симметричного редактирования цвета ячеек
    void slotCellColorNotSymmetrically();//При выборе несимметричного редактирования цвета ячеек
    void slotWords();//При выборе редактирования слов
    void slotOpenTemplate();//При открытии шаблона
    void inputDataToCrossword();//При вводе слова в кроссворд
    void setRegimeViewing();//Применение к кроссворду режима просмотра
    void setRegimeEditing();//Применение к кроссворду режима редактирования
    void slotCheckRightWords();//При проверке правильности отгадывания кроссворда
    void slotDictionaryOpen();//При открытии словаря
    void slotCompile();//При составлении кроссворда
    bool makeCompile();//Составление кроссворда
    void slotExitProgram();//При выходе из программы
    void slotChangeLine();//При смене направления слова
    //bool checkRightSelect();//Проверка правильности выбора слов из словаря
    void slotImportCrossword();//Импорт кроссвордов
    void openAcrossLiteTXT(QString FileName);//Открытие текстового файла AcrossLite
    void slotSemiautomaticInput();//Полуавтоматический ввод слов в сетку
    void slotShowStatistics();//Показ статистики
    void slotOpenDonate();//Открытие страницы сайта о программе
    void slotGetSource();//Открытие страницы сайта о программе
    void slotOpenGetSupport();//Получить техподдержку
    void makeDeleteWords();//Произвести удаление слов
    void slotClearGrid();//Очистка сетки
    void slotClearWords();//Очистка сетки от слов
    void slotClearAll();//Очстить сетку и слова
    void slotMouseMove();//При движении мышкой по сетке кроссворда
    void slotShowOptions();//Показать параметры
    void slotMakeExport();//Экспорт кроссворда
    void slotImportOK(QString sImportName);//Импорт файла формата OpenKlest
    void slotReportBug();//Открыть страницу баг-трекера
    void writeOptions();//Сохранение свойств
    int checkSaved(uchar ucAction);//Проверка сохранения
    void slotCheckForUpdates();//Загрузка страницы обновления
    void slotMakeCheckUpdates(bool bError);//Проверить наличие обновления
    void slotPrint();//Печать кроссворда
    void slotCallFunction(uchar ucType);

};
#endif
