/* ======================================================================
**  main.cpp - Исходный файл главного модуля кроссворда.
**  Является частью программы Клёст-кроссворд, распространяемой под
**  Стандартной общественной лицензией, версии 2 (GNU GPL v2).
** ======================================================================
**  Copyleft 2009 - 2012 by Lab Free Technologies <admin@LabFreeTech.org>
** ==============================================================================
**  Copyleft 2009 - 2012 Лаборатория свободных технологий <admin@LabFreeTech.org>
** ==============================================================================
*/

#include "TKlestCrossword/TKlestCrossword.h"
#include <QtCore/QCoreApplication>
#include <QtCore/QTranslator>
#include <QtCore/QSettings>
#include <QtGui/QApplication>
#include <QtGui/QIcon>
#include "const.h"
#include "struct.h"

//Загрузка свойств

void LoadOptions(TFileOptions *FileOptions)
{
    QSettings setFileOptions(QSettings::IniFormat, QSettings::UserScope, QCoreApplication::organizationName(), QCoreApplication::applicationName());

    setFileOptions.beginGroup("/Options");
    FileOptions->bModeWork = setFileOptions.value("/bModeWork", MODE_GUESSING).toBool();
    FileOptions->bLanguageProgram = setFileOptions.value("/bLanguageProgram", LANGUAGE_RUSSIAN).toBool();
    FileOptions->bStateStandardToolbar = setFileOptions.value("/bStateStandardToolbar", TOOLBAR_SHOW).toBool();
    FileOptions->bCurrentDictionary = setFileOptions.value("/bCurrentDictionary", LANGUAGE_RUSSIAN).toBool();
    FileOptions->bStateExport = setFileOptions.value("/bStateExport", EXPORT_NO).toBool();
    FileOptions->iNumberTimes = setFileOptions.value("/iNumberTimes", 1).toInt();
    FileOptions->iTypeExport = setFileOptions.value("/iTypeExport", TYPE_PNG).toInt();
    FileOptions->bUseCrosswordPassword = setFileOptions.value("/bUseCrosswordPassword", false).toBool();
    FileOptions->bUseModeEditingPassword = setFileOptions.value("/bUseModeEditingPassword", false).toBool();
    FileOptions->sModeEditingPassword = setFileOptions.value("/sModeEditingPassword", "").toByteArray();
    FileOptions->sCrosswordPassword = setFileOptions.value("/sCrosswordPassword", "").toByteArray();
    FileOptions->sPathCrossword = setFileOptions.value("/sPathCrossword", QDir::homePath()).toString();
    FileOptions->sPathTemplate = setFileOptions.value("/sPathTemplate", PATH_SHARE_APP  "/templates").toString();
    setFileOptions.endGroup();

    return;
}

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);
    app.setWindowIcon(QIcon(QString::fromUtf8(":/toolbar/toolbar/klest-crossword.png")));

    QCoreApplication::setOrganizationName("Lab Free Technologies");
    QCoreApplication::setOrganizationDomain("http://labfreetech.org");
    QCoreApplication::setApplicationName("Klest-crossword");
    QCoreApplication::setApplicationVersion("1.0.0");

    TFileOptions *FileOptions = new TFileOptions;
    LoadOptions(FileOptions);

    QTranslator tr1, tr2;

    if(FileOptions->bLanguageProgram == LANGUAGE_RUSSIAN)
    {
        tr1.load(PATH_SHARE_APP  "/languages/russian/klest_ru.qm", ".");
        tr2.load(PATH_SHARE_APP  "/languages/russian/qt_ru.qm", ".");
    }

    app.installTranslator(&tr1);
    app.installTranslator(&tr2);

    TKlestCrossword FormKlestCrossword;

    FormKlestCrossword.show();
    FormKlestCrossword.FileOptions = FileOptions;
    FormKlestCrossword.CrosswordGrid->FileOptions = FileOptions;

    if (FileOptions->bStateStandardToolbar == TOOLBAR_SHOW) FormKlestCrossword.ui->tbStandard->setVisible(TOOLBAR_SHOW);
    else FormKlestCrossword.ui->tbStandard->setVisible(TOOLBAR_HIDE);

    switch(FileOptions->bModeWork)
    {
    case MODE_GUESSING:
        FormKlestCrossword.ui->actionEditing->setChecked(false);
        FormKlestCrossword.ui->actionGuessing->setChecked(true);
        FormKlestCrossword.slotRegimeViewing();
        break;

    case MODE_EDITING:
        FormKlestCrossword.ui->actionEditing->setChecked(true);
        FormKlestCrossword.ui->actionGuessing->setChecked(false);
        FormKlestCrossword.slotRegimeEditing();
        break;
    }

    return app.exec();
}
